<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace Ron\Controls;
use Ron\GuiUtils;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Tracy\Debugger;

/**
 * Zobrazeni skupiny filtru nebo odkazu (prepinacich nebo zapinacich)
 */
class LinksList implements ViewObject
{
    /** @var string */
    public $url;

    /** @var bool   Urcuje, ze se nema do URL pridavat kod skupiny (pouziva se hlavne v renderLinks) */
    public $noGroupInParam;

    /** @var LinksListItem[] */
    public $items = [];

    /** @var bool   Umoznuje zakazat zobrazeni vybrane polozky */
    public $showSelectedItem = true;

    /** @var string     Obalujici HTML element - vychozi je DIV */
    public $wrapElement;
    /** @var string[]   Asociativni pole parametru obalujiciho tagu, defaultne ['class' => 'text_link'] */
    public $wrapElementParams;

    /** @var string     Nadpis zobrazeny pred odkazy */
    public $caption;

    /** @var string */
    //public $separator;

    public $cssClass = 'linkslist';

    /** @var string     Pokud je zadan, budou mit jednotlive polozky nastavene id (tento prefix + index z $items) */
    public $itemsIdPrefix = null;

    /** @var string     Pokud je zadano, vyrenderuje se hidden pole se zadanym jmenem a hodnota se odesila pomoci POST */
    public $hiddenFieldName = null;

    /** @var string[]   Atributy hidden pole. */
    public $hiddenFieldAttrs = [];


    /**
     * LinksList constructor.
     * @param array $data           Asociativni pole polozek
     * @param string|string[] $actualValue     Aktivni polozka (pripadne vice aktivnich polozek)
     * @param null $url
     */
    public function __construct($data = null, $actualValue = null, $url = null)
    {
        /** @noinspection HtmlUnknownTarget */
        //$this ->separator = '<div class="linksSeparator"></div>';

        $this ->wrapElement = 'div';
        $this ->wrapElementParams = ['class' => 'text_link'];

        $this ->url = $url;
        $this ->noGroupInParam = true;  // pro prepinaci polozky se generuji linky ve tvaru $url.$key (a ne $url $group = $key)

        $group = is_array($actualValue) ? null : 'group1';
        if ($data) {
            $this->addItemsFromArray($data, $group);
        }

        $this ->activateItem($actualValue);
    }


    /**
     * Vraci true, pokud neexistuje zadna polozka
     * @return bool
     */
    function isEmpty()
    {
        return !count($this ->items);
    }


    /**
     * Najde polozku se zadanym kodem
     * @param string $key
     * @return LinksListItem|null
     */
    public function findItem($key)
    {
        foreach ($this ->items as $item) {
            if (!strcasecmp($item ->key, $key ?? '')) {
                return $item;
            }
        }

        return null;
    }


    /**
     * Najde prvni polozku ze zadane skupiny
     * @param string $group
     * @param bool $active    Pokud je true, tak hleda jen aktivni polozku.
     * @return null|LinksListItem
     * @internal param string $key
     */
    public function findItemFromGroup($group, $active = false)
    {
        foreach ($this ->items as $item) {
            if (!strcasecmp($item ->group ?? '', $group)) {
                if (!$active  ||  $item ->active) {
                    return $item;
                }
            }
        }

        return null;
    }


    /**
     * Prida zadanou polozku. Pokud patri do skupiny, tak ji zneaktivni, pokud uz je ve skupine nejaka polozka aktivni.
     * @param LinksListItem $item
     * @return $this
     */
    public function addItem($item)
    {
        if ($item ->group  &&  $item ->active) {
            // zjisti, jestli uz je nejaka aktivni polozka ze stejne skupiny
            $pom = $this ->findItemFromGroup($item ->group, true);
            // a pokud ano, tak tuto novou zneaktivni
            if ($pom) {
                $item ->active = false;
            }
        }

        $this ->items[] = $item;

        return $this;
    }


    /**
     * Prida polozku se zadanymi parametry. Pokud uz existuje tak u ni pripadne doplni caption a group
     * @param string $key
     * @param string $caption
     * @param string $group
     * @return $this
     */
    public function addOrUpdateItem($key, $caption, $group = null)
    {
        if ($item = $this ->findItem($key)) {
            if ($caption && !$item ->caption) {
                $item ->caption = $caption;
            }
            if ($group && !$item ->group) {
                $item ->group = $group;
            }
        } else {
            $item = new LinksListItem($key, $caption);
//            $item ->key = $key;
//            $item ->caption = $caption;
            $item ->group = $group;

            $this ->addItem($item);
        }

        return $this;
    }

    /**
     * Vytvori polozky ze zadaneho asociativniho pole
     * @param string[] $data Asociativni pole polozek, ktere se maji pridat (key => text)
     * @param string [$group]
     * @param bool $defaultActive   Priznak, jestli maji byt vytvorene polozky aktivni
     * @return $this
     */
    public function addItemsFromArray($data, $group = null, $defaultActive = true)
    {
        foreach ($data as $key => $value) {
            if ($value instanceof LinksListItem) {
                $item = $value;
            } else if ($value instanceof Option) {
                $item = new LinksListItem($value ->key, $value ->caption);
                $item ->group = $group;
                $item ->active = $defaultActive;
                $item ->cssClass = $value ->cssClass;
                $item ->title = $value ->title;
            } else {
                $item = new LinksListItem($key, $value);
//                $item->key = $key;
//                $item->caption = $value;
                $item->group = $group;
                $item->active = $defaultActive;
            }

            $this ->addItem($item);
        }

        return $this;
    }


    /**
     * Zaktivni zadanou polozku. Pokud patri do skupiny, tak zneaktivni vsechny ostatni ze stejne skupiny.
     * V pripade ze je $key pole, tak zaktivni vsechny zadane polozky a ostatni zneaktivni - skupiny nehlida!
     * @param string|string[] $key
     * @return $this
     */
    public function activateItem($key)
    {
        if (is_array($key)) {
            foreach ($this ->items as $item) {
                $item ->active = in_array($item ->key, $key);
            }
        } else {
            $item = $this->findItem($key);
            if (!$item) {
                return $this;
            }

            $item->active = true;

            // pokud je zapnuta polozka ve skupine, tak vypne vsechny ostatni polozky teto skupiny
            if ($item->group) {
                foreach ($this->items as $otherItem) {
                    if ($otherItem->group  &&  $otherItem !== $item  &&  !strcasecmp($item->group, $otherItem->group)) {
                        $otherItem->active = false;
                    }
                }
            }
        }

        return $this;
    }


    public function getSelectedValue()
    {
        foreach ($this ->items as $item) {
            if ($item ->active) {
                return $item ->key;
            }
        }

        return null;
    }


    public function itemIsActive($key)
    {
        foreach ($this ->items as $item) {
            if ($item ->key == $key) {
                return $item ->active;
            }
        }

        return false;
    }

    /**
     * Vraci konfiguracni retezec (aktualni nastaveni jednotlivych sekci)
     * @return string
     */
    public function getSettingsString()
    {
        $result = [];

        foreach ($this ->items as $item) {
            if ($item->group) {
                if ($item ->active) {
                    $result[$item ->group] = $item ->key;
                }
            } else {
                $result[$item ->key] = $item ->active ? '' : '0';
            }
        }

        $resultStr = '';
        foreach ($result as $key => $value) {
            $resultStr .= ($resultStr ? ';' : '') . $key . ($value != '' ? '=' . $value : '');
        }

        return $resultStr;
    }


    /**
     * Nacte konfiguracni retezec (vytvoreny pomoci getSettingsString)
     * @param $settingsStr
     * @return $this
     */
    public function loadSettingString($settingsStr)
    {
        $strItems = (isset($settingsStr)  &&  $settingsStr !== '')
            ? explode(';', $settingsStr)
            : [];

        foreach ($strItems as $strItem) {
            // $strItem je polozka ve formatu group=key nebo key nebo key=0
            $pom = explode('=', $strItem, 2);
            foreach ($this ->items as $item) {
                if ($item ->group  &&  $item ->group == $pom[0]) {
                    $item ->active = $item ->key == $pom[1];
                } else if ($item ->key == $pom[0]) {
                    $item ->active = count($pom) == 1 || $pom[1];
                }
            }
        }

        return $this;
    }


    function getHtml()
    {
        $result = '';

        //$first = true;
        if ($this ->wrapElement) {
            $result .= "<{$this ->wrapElement}";
            foreach ($this ->wrapElementParams as $key => $value) {
                $result .= ' ' . $key . '="' . StringUtils::htmlTextConvert($value) . '"';
            }
            $result .= '>';
        }

        if ($this ->caption) {
            $result .= '<span class="linkscaption">' . $this ->caption . '</span> ';
        }

        $hiddenFieldId = $this->hiddenFieldAttrs['id'] ?? $this->hiddenFieldName;

//        if ($this ->hiddenFieldName) {
//            $attrs = array_merge(
//                ['type' => 'hidden', 'name' => $this ->hiddenFieldName, 'id' => $hiddenFieldId, 'value' => $this ->getSelectedValue()],
//                $this ->hiddenFieldAttrs ?: []);
//            $result .= '<input ' . GuiUtils::getAttributesHtml($attrs) . '/>';
//        }

        $result .= "<ul class='{$this ->cssClass}'>";

        $hiddenFields = [];

        foreach ($this ->items as $itemKey => $item) {
            $groupId = ($item ->group  &&  $item ->group !== true)
                ? $item ->group
                : '';

            if (!$item ->getUrl()  &&  $this->hiddenFieldName) {

                $hiddenFields[$groupId] = $item->active
                    ? $item->key
                    : ($hiddenFields[$groupId] ?? '');
            }

            if ($this ->showSelectedItem  || !$item ->active) {

                $cssClass =
                    ($item->active ? 'selectedlink' : '') .
                    ($item ->cssClass ? ' ' . $item ->cssClass : '');

                $result .= '<li' .
                    ($cssClass ? " class='$cssClass'" : '') .
                    ($this->itemsIdPrefix ? " id='{$this ->itemsIdPrefix}{$item ->key}'" : '') .
                    ($groupId ? " data-group='{$groupId}'" : '') .
                    '>';

                if (!$item ->getUrl()  &&  $this->hiddenFieldName) {
                    $selectorId = $hiddenFieldId . ($groupId ? '_' . $groupId : '');
                    $result .=
                        "<span class='js setValue {$item ->cssClass}' data-submit='1' data-value='{$item ->key}' data-selector='#$selectorId'" .
                        ($item->title ? 'title="' . StringUtils::htmlTextConvert($item->title) . '"' : '') .
                        ">" .
                        StringUtils::htmlTextConvert($item->caption) .
                        '</span>';
                } else {
                    if ($item ->getUrl()) {
                        $url = $item ->getUrl();
                    } else {
                        $urlParam = $item->group
                            ? ($this->noGroupInParam ? '' : $item->group . '=') . $item->key // prepinaci skupinova polozka
                            : $item->key . '=' . ($item->active ? '0' : '1');     // samostatna vy/zapinaci polozka

                        $url = $this ->url . $urlParam;
                    }

                    $result .=
                        "<a href='$url'" .
                        ($item->title ? ' title="' . StringUtils::htmlTextConvert($item->title) . '"' : '') .
                        '>' .
                        StringUtils::htmlTextConvert($item->caption) .
                        "</a>";
                }

                $result .= '</li>';
            }
        }

        $result .= '</ul>';

        if ($hiddenFields) {
            foreach ($hiddenFields as $key => $value) {
                $attrs = array_merge(
                    [
                        'type' => 'hidden',
                        'name' => $this ->hiddenFieldName . ($key ? '_' . $key : ''),
                        'id' => $hiddenFieldId . ($key ? '_' . $key : ''),
                        'value' => $value
                    ],
                    $this ->hiddenFieldAttrs ?: []);

                $result .= '<input ' . GuiUtils::getAttributesHtml($attrs) . '/>';
            }
        }

        if ($this ->wrapElement) {
            $result .= "</{$this ->wrapElement}>";
        }

        $result .= "\n";

        return $result;
    }

    function render()
    {
        echo $this ->getHtml();
    }


    public function loadGet()
    {
        foreach ($this ->items as $item) {
            if ($item ->group) {
                if (isset($_GET[$item ->group])) {
                    $item ->active = $item ->key == $_GET[$item ->group];
                }
            } else {
                if (isset($_GET[$item ->key])) {
                    $item ->active = (int)$_GET[$item ->key];
                }
            }
        }
    }


    /**
     * Vyrenderuje seznam odkazu ze zadaneho pole
     * @param string[]|Option[] $data           Asociativni pole zobrazenych hodnot "odkaz" => "text"
     * @param string|string[]   $actualValue    Aktualne vybrana (zvyraznena) polozka - muze jich byt i vice
     * @param string    $url        Vysledna url se slozi z $url . $key
     * @param string    $caption
     * @param bool      $showDiv    Priznak, jestli se ma zobrazit obalujici div
     * @param array|bool $options   Umoznuje nastavit dalsi volby LinksListu. Hodnota true nastavi links list, aby pouzival hidden pole.
     */
    public static function renderLinks($data, $actualValue, $url, $caption = '', $showDiv = true, $options = false)
    {
        $list = new static();

        $list ->caption = $caption;
        $list ->noGroupInParam = true;  // pro prepinaci polozky se generuji linky ve tvaru $url . $key (a ne $url $group = $key)

        if (!$showDiv) {
            $list->wrapElement = null;
        }

        if (is_array($options)) {
            foreach ($options as $key => $value) {
                $list ->{$key} = $value;
            }
        } else if ($options) {
            $list ->hiddenFieldName = $url;
        } else {
            $list->url = $url;
        }

        $group = is_array($actualValue) ? null : true; // - zmeneno 13.5.2024 pri predelavce linklist (doplneni hidden poli pro groups)
        $list ->addItemsFromArray($data, $group,false);

        $list ->activateItem($actualValue);

        $list ->render();
    }
}
