<?php
/**
 * stanislav.elbl, 28.03.2022 15:29
 */

namespace Ron\Controls;

use Ron\TextManager;
use Ron\XmlLoader;


/**
 * Obecna pomocna trida pro nacitani konfigurace z XML textu. Konfigurovany objekt se zadava v konstruktoru a musi implementovat rozhrani ConfigurableObject.
 */
class ObjectConfiguration
{
    /**
     * Objekt, pro ktery se konfigurace nacita
     * @var ConfigurableObject
     */
    protected $configurableObject;

    /** @var string     Element, ktery se nacita jako "RAW" data - tedy vcetne vnorenych tagu. Uklada se do pole $htmlContents. */
    public $rawDataElement = 'html';

    /** @var string[]   Obsahy elementu <html> pro jednotliva umisteni (napr. "top" => "bla bla") */
    public $htmlContents = [];


    /**
     * ObjectConfiguration constructor.
     * @param ConfigurableObject $configurableObject
     */
    public function __construct(ConfigurableObject $configurableObject)
    {
        $this->configurableObject = $configurableObject;
    }


    /**
     * Vraci spojeny text vsech obsahu ($htmlContents) ze zadanych pozic
     * @param string|string[] $positions
     * @return string
     */
    public function getHtmlContent($positions)
    {
        $result = '';

        foreach ($this ->htmlContents as $position => $value) {
            if (is_array($positions)) {
                if (in_array($position, $positions)) {
                    $result .= $value;
                }
            } else {
                if ($position === $positions) {
                    return $value;
                }
            }
        }

        return $result;
    }

    /**
     * Umoznuje zablokovat pouziti konfiguracni polozky.
     * @param $item
     * @return bool
     */
    public function configurationItemEnabled($item)
    {
        return true;
    }


    protected function getValueText($value)
    {
        $valueText = null;
        foreach ($value as $valA) {
            if ($this ->configurationItemEnabled($valA)) {
                if (!isset($valueText)) {
                    $valueText = '';
                }

                if (isset($valA['#text'])) {
                    $valueText .= ($valueText ? "\n" : '') . $valA['#text'];
                }
            }
        }

        return $valueText;
    }


    /**
     * Nacte a zpracuje jednu polozku z pole.
     * @param $key
     * @param $value
     * @return void
     */
    protected function loadConfigurationItem($key, $value)
    {
        if ($key === $this ->rawDataElement) {
            foreach ($value as $htmlItem) {
                if ($this ->configurationItemEnabled($htmlItem)) {
                    $content = XmlLoader::rawDataToString($htmlItem);
                    $position = $htmlItem['#attr']['position'] ?? '';
                    $this->htmlContents[$position] = ($this->htmlContents[$position] ?? '') . $content;
                }
            }

            return;
        }

        if ($key === 'translation') {
            TextManager::getInstance() ->loadArray($value);
        }

        if ($value instanceof \ArrayAccess) {
            // hodnoty definovane pomoci vnorenych tagu spoji do jednoho textu (napr. style)
            $valueText = $this ->getValueText($value);
            if (!isset($valueText)) {
                return;
            }
        } else {
            $valueText = $value;
        }

        if ($this ->configurableObject) {
            $this ->configurableObject ->loadConfigurationItem($key, $value, $valueText);
        }

    }


    /**
     * Nacte konfiguraci z asociativniho pole.
     * @param $a array  Pole obsahujici konfiguraci
     * @return void
     */
    public function loadConfigurationArray($a)
    {
        if ($a) {
            foreach ($a as $key => $value) {
                $this->loadConfigurationItem($key, $value);
            }
        }
    }


    /**
     * Nacte konfiguraci ze zadaneho xmlTextu
     * @param string $xmlText
     */
    public function loadConfigurationXml($xmlText)
    {
        if (!$xmlText) {
            return;
        }

        $a = XmlLoader::loadXmlToArray($xmlText, [
                XmlLoader::PARAM_CONVERT_FROM_UTF8 => true,
                XmlLoader::PARAM_INCLUDE_ATTRIBUTES => true,
                XmlLoader::PARAM_RAW_DATA_ELEMENT => $this ->rawDataElement
            ]
        );

        $this ->loadConfigurationArray($a);
    }

}
