<?php
/**
 * stanislav.elbl, 20.02.2021
 */

namespace Ron\DB;

use Ron\TextManager;

class IbaseDatabase extends Database
{

    /** @var resource */
    private $connection;

    /** @var string */
    private $connectionString;

    /** @var resource       Aktualni transakce */
    private $transaction;

    public function __construct($connectionString)
    {
        $this->connectionString = $connectionString;
        $this ->concatenateOperator = '||';
        $this ->functions = [
            'GETDATE' => '#current_timestamp',
            'LEN' => 'CHAR_LENGTH',
            'EXTRACTDATE' => 'CAST (? AS DATE)',
            'EXTRACTTIME' => 'CAST (? AS TIME)'
        ];
    }


    /**
     * @return resource
     */
    public function getConnection()
    {
        return $this->connection;
    }

    /**
     * @return string
     */
    public function getConnectionString()
    {
        return $this->connectionString;
    }

    /**
     * @return string
     */
    public function getDbType()
    {
        return 'interbase';
    }

    /**
     * @param string $user
     * @param string $password
     * @throws DBException
     */
    function connect($user, $password)
    {
        $charset = null;

        global $db_charset;
        if (isset($db_charset)) {
            if ($db_charset) $charset = $db_charset;
        } else {
            switch (TextManager::getInstance() ->getCharSet()) {
                case 'windows-1250' :
                    $charset = 'win1250';
                    break;
            }
        }

        $this ->connection = ibase_connect($this ->connectionString, $user, $password, $charset);  // , 'windows-1250'   , 'win1250'

        if (!$this ->connection) {
            throw new DBException($this, '', \Ron\ronText('Connection_fail'));
        }
    }


    /**
     * @return bool
     */
    function close()
    {
        return ibase_close($this ->connection);
    }


    public function createParameters()
    {
        return new PlaceholderParameters($this);
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @return Cursor
     * @throws DBException
     */
    function executeSelect($sqlText, $params = null)
    {
        if ($params) {
            $stmt = ibase_prepare($sqlText);
            if (!$stmt) {
                throw new DBException($this, $sqlText);
            }

            $parameters = $params ->getSqlParameters();

            array_unshift($parameters, $stmt);
            $data = call_user_func_array('ibase_execute', $parameters);
        } else {
            $data = ibase_query($this->connection, $sqlText);
        }

        if (!$data) {
            throw new DBException($this, $sqlText);
        }

        return new IbaseCursor($this, $data);
    }


    /**
     * @param string $sqlText
     * @param Parameters $params
     * @throws DBException
     */
    function executeSql($sqlText, $params = null)
    {
        if ($params) {
            $stmt = ibase_prepare($sqlText);
            if (!$stmt) {
                throw new DBException($this, $sqlText);
            }

            $parameters = $params ->getSqlParameters();

            array_unshift($parameters, $stmt);
            $data = call_user_func_array('ibase_execute', $parameters);
        } else {
            $data = ibase_query($this->connection, $sqlText);
        }

        if (!$this ->transaction) {
            ibase_commit_ret(); // nutne napr. pred spustenim agenta! (jinak agent pracuje jeste se starymi daty)
        }

        if (!$data) {
            throw new DBException($this, $sqlText);
        }
    }



    function lastErrorText()
    {
        return 'Chyba ' . ibase_errcode() . ':<BR>' . ibase_errmsg();
    }


    /**
     * @throws \Exception
     */
    public function beginTransaction()
    {
        if (!$this ->transaction) {
            $this ->transaction = ibase_trans($this ->connection);

            if (!$this ->transaction) {
                throw new \Exception('Chyba v IbaseDatabase::beginTransaction: ' . $this ->lastErrorText() );
            }
        }
    }

    /**
     * @throws \Exception
     */
    public function commitTransaction()
    {
        if ($this ->transaction) {
            if (!ibase_commit($this ->transaction)) {
                throw new \Exception('Chyba pri ukoncovani transakce (IbaseDatabase::commitTransaction): ' . $this ->lastErrorText());
            };

            $this ->transaction = null;
        }
    }

    /**
     * @throws \Exception
     */
    public function rollbackTransaction()
    {
        if ($this ->transaction) {
            ibase_rollback($this ->transaction);
            $this ->transaction = null;
        }
    }

    /**
     * @param string $str
     * @return string
     */
    public function addSlashes($str)
    {
        return str_replace("'", "''", $str);
    }

    /**
     * @param string $text
     * @return string
     */
    public function castWideMemo($text)
    {
        return $text;
    }

    /**
     * @param string $text
     * @return string
     */
    public function extractYear($text)
    {
        return "EXTRACT(YEAR FROM $text)";
    }

    /**
     * @param string $str
     * @return int|null
     */
    public function stringToTimestamp($str)
    {
        if (!isset($str)) return NULL;

        // format: 2012-04-02 08:00:00
        //         1899-12-30 09:00:00
        $str = str_replace('1899', '1970', $str); // pokud je tam 1899, tak je to nejspis jenom cas - prevede ho - finta, aby se cas dobre zobrazoval
        //echo ' ('.$str.' -> '.strtotime($str).' -> '.date('d.m.Y H:i:s', strtotime($str)).') ';
        return strtotime($str);

    }

    /**
     * @param int $timestamp
     * @return string
     */
    public function timestampToString($timestamp)
    {
        if (!isset($timestamp)) return 'null';

        return "'" . date('m/d/Y H:i:s', $timestamp) . "'";
    }

    /**
     * Prevede timestamp na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function dateToString($timestamp)
    {
        if (!isset($timestamp)) return 'null';

        return "'" . date('m/d/Y', $timestamp) . "'";
    }

    /**
     * prevede cas na sql konstantu
     * @param int $timestamp
     * @return string
     */
    public function timeToString($timestamp)
    {
        if ($timestamp >= 0 && $timestamp < 86400) {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = str_pad((int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
            $timestamp = $timestamp - 3600 * $h;
            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT);
            $s = str_pad($timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);
            return "'12/30/1899 $h:$m:$s'";
        }

        return "'" . date('12/30/1899 H:i:s', $timestamp) . "'";
    }


    public function timestampToParam($timestamp)
    {
        if (!isset($timestamp)) return null;

        return date('m/d/Y H:i:s', $timestamp);
    }

    public function dateToParam($timestamp)
    {
        if (!isset($timestamp)) return null;

        return date('m/d/Y', $timestamp);
    }

    public function timeToParam($timestamp)
    {
        if ($timestamp >= 0 && $timestamp < 86400) {
            // Jedna se pouze o cas. Kvuli letni/zimni to neprevadi pomoci date()
            $h = str_pad((int)($timestamp / 3600), 2, '0', STR_PAD_LEFT);
            $timestamp = $timestamp - 3600 * $h;
            $m = str_pad((int)($timestamp / 60), 2, '0', STR_PAD_LEFT);
            $s = str_pad($timestamp - 60 * $m, 2, '0', STR_PAD_LEFT);
            return "12/30/1899 $h:$m:$s";
        }

        return date('12/30/1899 H:i:s', $timestamp);
    }


    /**
     * prevede retezec z databaze na denni cas - vraci pocet hodin (realne cislo)\
     * napr "... 7:30" => 7.5
     * @param string $str
     * @return integer
     */
    public function stringToTimeOfDay($str)
    {
        if ( !isset($str) ) return null;

        $timestr = strrchr($str, ' '); // vraci vse za posledni mezerou

        $poma = explode (":", $timestr);

        return $poma[0] + $poma[1] / 60.0 + $poma[2] / 3600.0 ;
    }


    public function nextSequenceValueSQL($sequenceName)
    {
        $standalone = true; // TODO: pridat parametr $standalone?
        return "NEXT VALUE FOR $sequenceName" . ($standalone ? ' FROM RDB$DATABASE' : '');
    }


    /**
     * @param string $sequenceName
     * @return bool
     * @throws DBException
     */
    protected function sequenceExistsInternal(string $sequenceName):bool
    {
        $sql =
            'SELECT RDB$GENERATOR_NAME AS name FROM RDB$GENERATORS' .
            ' WHERE UPPER(RDB$GENERATOR_NAME) = ' . "'" . strtoupper($sequenceName) . "'";

        if ($sql) {
            $cursor = $this ->getSqlResult($sql);
            $row = $cursor ->fetch();
            $cursor ->close();

            return (bool)$row;
        }

        return false;
    }
}


class IbaseCursor extends Cursor
{
    /**
     * @var resource
     */
    protected $data;


    /**
     * @param IbaseDatabase $database
     * @param resource $data
     */
    function __construct($database, $data) {
        parent::__construct($database);
        $this ->data = $data;
    }


    public function close()
    {
        if ($this ->data) {
            ibase_free_result($this->data);
            $this ->data = null;
        }
    }


    /**
     * @return resource
     */
    public function getData()
    {
        return $this->data;
    }

    /**
     * @return array|bool
     */
    public function fetch()
    {
        return ibase_fetch_row($this ->data, IBASE_TEXT);
    }

    // TODO: Implement numRows() method.

}
