<?php

/**
 * UTF8
 * Spravce souboru - moznost uploadu souboru, downloadu, mazani
 */

namespace Ron;

use Ron\MVC\ViewObject;

class FileManager implements ViewObject
{
    const FIELD_ICON = 'icon';
    const FIELD_NAME = 'name';
    const FIELD_NAME_AND_ICON = 'nameandicon';
    const FIELD_SIZE = 'size';
    const FIELD_DATE = 'date';
    const FIELD_TIME = 'time';

    public $dir;

    private $subdir = '';

    public $url;
    public $subdirUrl;  // Url pro prechod do podadresare. Musi obsahovat podretezec %SUBDIR%, ktery se nahradi cestou k podadresari
    public $deleteUrl;
    public $downloadUrl;
    public $uploadUrl;

    public $columns = [self::FIELD_NAME_AND_ICON, self::FIELD_SIZE, self::FIELD_DATE, self::FIELD_TIME];
    public $showDirs = false;
    public $header;

    public $iconClass = 'fal inline-glyph margin-right';
    public $defaultIconClass = '';

    /**
     * @var FileInfo[]
     */
    protected $files = [];


    /**
     * FileManager constructor.
     * @param string $dir
     * @param string $url
     * @param string $downloadUrl
     * @param string $deleteUrl
     */
    public function __construct($dir, $url = null, $downloadUrl = null, $deleteUrl = null)
    {
        $this->url = $url;
        $this->dir = $dir;
        $this->deleteUrl = $deleteUrl;
        $this->downloadUrl = $downloadUrl;
    }


    /**
     * @param string $subdir
     * @return FileManager
     */
    public function setSubdir($subdir)
    {
        if ($subdir) {
            $subdir = str_replace('\\', '/', $subdir);
            $folders = explode('/', $subdir);

            $subdir = '';

            // zkontroluje, ze vsechny adresare existuji
            foreach ($folders as $folder) {
                if ($folder != '.'  &&  $folder != '..') {
                    if (is_dir($this ->dir . ($subdir ? '/' . $subdir : '') . '/' . $folder)) {
                        $subdir .= ($subdir ? '/' : '') . $folder;
                    }
                }
            }

            $this->subdir = $subdir;
        } else {
            $this ->subdir = '';
        }

        return $this;
    }




    public function getFullPath($fileName = null)
    {
        // TODO: confirm validity and existence of all subdirs
        $path = $this ->dir;
        if ($this ->subdir) {
            $path .= '/' . $this ->subdir;
        }

        if ($fileName) {
            $path .= '/' . $fileName;
        }

        return $path;
    }


    /**
     * @param bool $loadDetails
     * @param string|bool $sort
     */
    public function load($loadDetails = true, $sort = true)
    {
        $path = $this->getFullPath();

        $this ->files = file_exists($path)
            ? FileUtils::getFileList($path, $this ->showDirs, $loadDetails)
            : [];

        if ($sort) {
            if ($sort === true) {
                $sort = isset($_SESSION['FileManager.SORT']) ? $_SESSION['FileManager.SORT'] : 'name';
                if (!$sort) {
                    $sort = "name";
                }
            } else {
                $_SESSION['FileManager.SORT'] = $sort;
            }

            // a seradi panely
            usort($this->files,
                static function ($item1, $item2) use ($sort) {
                    /**
                     * @var FileInfo $item1
                     * @var FileInfo $item2
                     */
                    // nejdrive jsou adresare
                    if ($item1 ->isDir  &&  !$item2 ->isDir) {
                        return -1;
                    }
                    if (!$item1 ->isDir  &&  $item2 ->isDir) {
                        return 1;
                    }

                    if ($sort === "size") {
                        if (!$item1 ->isDir) {
                            if ($item1 ->size < $item2 ->size) {
                                return -1;
                            }
                            if ($item1 ->size > $item2 ->size) {
                                return 1;
                            }
                            // dale podle jmena
                        }
                    }

                    if ($sort === "time") {
                        if ($item1 ->modificationTime < $item2 ->modificationTime) {
                            return -1;
                        }
                        if ($item1 ->modificationTime > $item2 ->modificationTime) {
                            return 1;
                        }
                        // dale podle jmena
                    }

                    if ($item1 ->fileName < $item2 ->fileName) {
                        return -1;
                    }

                    if ($item1 ->fileName > $item2 ->fileName) {
                        return 1;
                    }

                    return 0;
                }
            );
        }
    }


    /**
     * @param FileInfo $fileInfo
     * @return string
     */
    public function getIcon($fileInfo)
    {
        if ($fileInfo ->isDir) {
            return 'fa-folder';
        }

        return FileUtils::getIcon($fileInfo ->fileName, $this ->defaultIconClass);
    }


    /**
     * @param FileInfo $fileInfo
     * @return string
     */
    public function getFileUrl($fileInfo)
    {
        if (!$fileInfo  ||  !$fileInfo ->fileName) {
            return '';
        }

        if ($fileInfo ->isDir) {
            if (!$this ->subdirUrl) {
                return '';
            }

            if ($fileInfo ->fileName === '..') {
                if ($this ->subdir  &&  $this ->subdir[strlen($this ->subdir) - 1] === '/') {
                    // remove slash from the end
                    $subdir = substr($this ->subdir, 0, -1);
                } else {
                    $subdir = $this ->subdir;
                }

                $subdir = substr($subdir, 0, strrpos($subdir, '/'));
            } else {
                $subdir = $this->subdir;
                if ($subdir && $subdir[strlen($subdir) - 1] !== '/') {
                    $subdir .= '/';
                }

                $subdir .= $fileInfo ->fileName;
            }

            return str_replace('%SUBDIR%', FileUtils::translateFileNameToLink($subdir), $this ->subdirUrl);
        }

        if (!$this ->downloadUrl) {
            return '';
        }

        return str_replace(
            ['%SUBDIR%', '%FILENAME%'],
            [FileUtils::translateFileNameToLink($this->subdir), FileUtils::translateFileNameToLink($fileInfo->fileName)],
            $this ->downloadUrl);
//        return $this ->downloadUrl
//            ? $this ->downloadUrl . '&filename=' . FileUtils::translateFileNameToLink($fileInfo ->fileName)
//            : '';
    }


    /**
     * @param FileInfo $fileInfo
     * @return string
     */
    public function getTableRow($fileInfo)
    {
        $result = "<tr>";

        foreach ($this ->columns as $column) {
            switch ($column) {
                case self::FIELD_ICON :
                    $icon = $this ->getIcon($fileInfo);
                    $result .=
                        '<td>' .
                        ($icon
                            ? "<i class='{$this ->iconClass} $icon'></i>"
                            : '') .
                        '</td>';
                    break;

                case self::FIELD_NAME :
                case self::FIELD_NAME_AND_ICON :
                    $icon = $column === self::FIELD_NAME_AND_ICON
                        ? $this ->getIcon($fileInfo)
                        : '';

                    $url = $this ->getFileUrl($fileInfo);
                    $result .=
                        '<td class="txt">' .
                        ($url ? '<a href="' . $this ->getFileUrl($fileInfo) . '" ' . ($fileInfo ->isDir ? '' : ' target="_blank"') . '>' : '').
                        (($column === self::FIELD_NAME_AND_ICON)
                            ? "<i class='{$this ->iconClass} $icon'></i>"
                            : '') .
                        $fileInfo ->fileName .
                        ($url ? '</a>' : '') .
                        "</td>\n";
                    break;

                case self::FIELD_SIZE :
                    $title = StringUtils::format('number0', $fileInfo ->size) . ' B';
                    $result .=
                        '<td class="align-right" title="' . $title . '"> ' .
                        ($fileInfo ->isDir ? '' : FileUtils::getFileSizeText($fileInfo ->size)) .
                        '</td>';
                    break;

                case self::FIELD_DATE :
                    $result .= "<td class='align-center'> ".date("j.n.Y" ,$fileInfo ->modificationTime) . "</td>";
                    break;

                case self::FIELD_TIME :
                    $result .= "<td> ".date("H:i" ,$fileInfo ->modificationTime) . '</td>';
                    break;
            }
        }

        // Odkaz pro smazani souboru - javascript
        if ($this ->deleteUrl  &&  !$this ->subdir) {   // pro soubory v podadresarich zatim neni mazani podporovano!
            $question = sprintf( ronText('Really_delete_file'), $fileInfo ->fileName );
            //$fileName = FileUtils::translateFileNameToLink($fileInfo ->fileName);
            //$href = "{$this ->deleteUrl}&filename=$fileName";
            $href = str_replace('%FILENAME%', FileUtils::translateFileNameToLink($fileInfo ->fileName), $this ->deleteUrl);
            $result .= '<td>' . GuiUtils::changeLocationQuest( ronText('delete'), $question, $href ) . '</td>';
        }

        $result .= "</tr>\n";

        return $result;
    }


    public function renderUpload()
    {
        if ( !$this ->uploadUrl ) {
            return;
        }

        echo
            '<div>' .
            '<form enctype="multipart/form-data" method="post" action="' . $this ->uploadUrl . '">' .
            ronText('Upload_file') .
            '<input type="file" name="uploadfilename">' .
            "&nbsp;&nbsp;" .
            '<input type="submit" class="submitbutton" value="' . ronText('Upload') . '">' .
            "</form>\n" .
            "</div>\n";
    }


    private function getTableHeader()
    {
        $result = '<thead>';

        foreach ($this ->columns as $column) {
            switch ($column) {
                case self::FIELD_ICON :
                    $result .= '<th></th>';
                    break;

                case self::FIELD_NAME :
                case self::FIELD_NAME_AND_ICON :
                    $result .=
                        '<th class="txt">' .
                        ($this->url ? '<a href="' . $this ->url . '&sort=name">' : '') .
                        ronText('Jmeno_souboru') .
                        ($this->url ? '</a>' : '') .
                        '</th>';
                    break;

                case self::FIELD_SIZE :
                    $result .=
                        '<th class="txt align-right">' .
                        ($this->url ? '<a href="' . $this ->url . '&sort=size">' : '') .
                        ronText('Velikost') .
                        ($this->url ? '</a>' : '') .
                        '</th>';
                    break;

                case self::FIELD_DATE :
                    $result .=
                        '<th class="txt align-center">' .
                        ($this->url ? '<a href="' . $this ->url . '&sort=time">' : '') .
                        ronText('Date') .
                        ($this->url ? '</a>' : '') .
                        '</th>';
                    break;

                case self::FIELD_TIME :
                    $result .=
                        '<th class="txt">' .
                        ($this->url ? '<a href="' . $this ->url . '&sort=time">' : '') .
                        ronText('Cas') .
                        ($this->url ? '</a>' : '') .
                        '</th>';
                    break;
            }
        }

        $result .= "</thead>\n";

        return $result;
    }


    public function getHtml() {
        $result =
            '<table class="filetable rowOver">' .
            $this ->getTableHeader() .
            '<tbody class="border-top">';

            foreach ($this ->files as $fileInfo) {
                if ($fileInfo ->fileName !== '.'  &&  ($this ->subdir  || $fileInfo ->fileName !== '..')) {
                    $result .= $this->getTableRow($fileInfo); // $dir, $filename, $columns, $downloadlink, $deletelink);
                }
            }

        $result .=
            '</tbody>' .
            "</table>\n";

        return $result;
    }


    public function render()
    {
        if ($this ->header) {
            echo "<h1>{$this ->header}</h1>\n";
        }

        MessageManager::getInstance() ->showMessages();

        echo $this->getHtml();

        $this ->renderUpload();
    }

}
