<?php
/**
 * UTF8
 * stanislav.elbl, 19.12.2016
 */

namespace Ron;

/**
 * Class XmlLoader
 * XmlLoader nacita rekurzivne obsah XML do asociativniho pole. Pro kazdy atribut vytvori prvek pole
 * a pro vnorene elementy vytvori pole.
 * Text elementu nacita do indexu #text. Pole atributu do indexu #attr. Priklad vystupu pro nastaveni editace dochazky:
 * [ "#attr" => ["editovatvzdycelyden" => "0", "kontroly" => "1"],
 *   "sloupeczpracovanedochazky" => [ ["typ" => "SCHVAL1"], ["typ" => "DATUM"], ... ],
 *   "tlacitko" => [ [ "typ" => "ulozit", umisteni => "2"], ["typ" => "schvalit"], ... ],
 *   "editovatvzdycelyden" => "0", "kontroly" => "1"       - toto vlozi do pole jen kdyz je zapnuto $includeAttributes
 *   "#text" => "       "
 */

class XmlLoader
{
    const PARAM_IGNORE_PREFIXES = 'ignorePrefixes';
    const PARAM_PREFIX_TO_SUBARRAY = 'prefixToSubarray';
    const PARAM_INCLUDE_ATTRIBUTES = 'includeAttributes';
    const PARAM_CONVERT_FROM_UTF8 = 'convertFromUtf8';
    const PARAM_RETURN_PROXY_ARRAY = 'returnProxyArray';
    const PARAM_RAW_DATA_ELEMENT = 'rawDataElement';

    /** @var bool   Urcuje, jestli se maji ignorovat prefixy xml tagu a atributu  */
    public $ignorePrefixes = false;
    /** @var bool   Pokud je true, tak atributy s prefixem budou samostatne ve vnorenem poli (pro kazdy jednotlivy prefix)  */
    public $prefixToSubarray = false;

    /**
     * @var bool   Urcuje, jestli se hodnoty atributu maji ukladat i mezi vnorene elementy
     */
    public $includeAttributes = false;

    /**
     * @var bool   Urcuje, jestli se ma obsah konvertovat z utf8 do lokalniho kodovani (charset v ron_string.ini)
     */
    public $convertFromUtf8 = false;

    /**
     * @var bool   Urcuje, jestli se ma vratit instance ProxyArray nebo obycejne pole.
     */
    public $returnProxyArray = true;

    /**
     * @var string  Element, ktery definuje "raw" data - ta se nactou sekvencne vcetne textovych elementu
     */
    public $rawDataElement = null;

    /**
     * Nacte aktualni element
     * @param \XMLReader $xmlReader
     * @return array
     */
    public function readAttributes($xmlReader)
    {
        $attr = $this ->returnProxyArray ? new ProxyArray() : [];

        while ($xmlReader->moveToNextAttribute()) {
            $name = $xmlReader->localName;
            if (!$this ->ignorePrefixes  &&  $xmlReader->prefix) {
                $name = $xmlReader->prefix . ':' . $name;
            }

            $value = $xmlReader ->value;
            if ($this ->convertFromUtf8) $value = StringUtils::fromUTF8($value);

            if ($this ->prefixToSubarray  &&  $xmlReader->prefix) {
                if (!isset($attr[$xmlReader->prefix])  ||  !$attr[$xmlReader->prefix] instanceof \ArrayAccess) {
                    $attr[$xmlReader->prefix] = $this ->returnProxyArray ? new ProxyArray() : [];
                }
                $attr[$xmlReader->prefix][$name] = $value;
            } else {
                $attr[$name] = $value;
            }
        }

        return $attr;
    }

    /**
     * Nacte do pole zadany DOM element
     * @param $element \DOMNode
     * @return array
     */
    public function readDOMElement($element)
    {
        $result = $this ->returnProxyArray ? new ProxyArray() : [];

        $attr = $this ->returnProxyArray ? new ProxyArray() : [];
        if ($element ->hasAttributes()) {
            for ($i = 0; $i < $element ->attributes ->length; $i++) {
                $value = $element ->attributes ->item($i) ->nodeValue;
                if ($this ->convertFromUtf8) $value = StringUtils::fromUTF8($value);

                $attr[$element ->attributes ->item($i) ->nodeName] = $value;
            }
        }

        $result['#attr'] = $attr;
        $result['#children'] = $this ->returnProxyArray ? new ProxyArray() : [];

        if ($this ->includeAttributes) {
            // zkopiruje hodnoty atributu do vysledku
            foreach ($attr as $name => $value) {
                $result[$name] = $value;
            }
        }

        if ($element ->hasChildNodes()) {
            for ($i = 0;  $i < $element ->childNodes ->length;  $i++) {
                $node = $element ->childNodes ->item($i);

                if ($node ->nodeType == XML_ELEMENT_NODE) {

                    if (!isset($result[$node->nodeName])) {
                        $result[$node->nodeName] = $this ->returnProxyArray ? new ProxyArray() : [];
                    }

                    $child = $this->readDOMElement($node);
                    $result[$node->nodeName][] = $child;
                    $result['#children'][] = ['name' => $node->nodeName, 'element' => $child];
                }
            }
        }

        $result['#text'] = $this ->convertFromUtf8 ? StringUtils::fromUTF8($element ->nodeValue) : $element ->nodeValue;

        return $result;
    }


    /**
     * Nacte aktualni element
     * @param \XMLReader $xmlReader
     * @param bool $rawData     Pokud je false, tak se elementy nezpracovavaji - pouze se nacitaji pole #attr a #children (vcetne textovych nodu)
     * @return array
     * @throws \Exception
     */
    public function readXmlElement($xmlReader, $rawData = false)
    {
        $result = $this ->returnProxyArray ? new ProxyArray() : [];

        if ($xmlReader->nodeType != \XMLReader::ELEMENT) {
            throw new \Exception('Chyba v metode XmlLoader::readXmlElement!');
        }

        $isEmpty = $xmlReader->isEmptyElement;
        $elementName = $xmlReader->localName;

        // nacte atributy
        $attr = $this ->readAttributes($xmlReader);
        $result['#attr'] = $attr;

        if (!$rawData  &&  $this ->includeAttributes) {
            // zkopiruje hodnoty atributu do vysledku
            foreach ($attr as $name => $value) {
                $result[$name] = $value;
            }
        }

        $rawData = $rawData || ($this ->rawDataElement  &&  $elementName === $this ->rawDataElement);

        $OK = $xmlReader->read(); // prejde za atributy

        $text = '';

        if (!$isEmpty) {
            $result['#children'] = $this ->returnProxyArray ? new ProxyArray() : [];

            while ($OK  &&  $xmlReader->nodeType != \XMLReader::END_ELEMENT) {

                switch ($xmlReader->nodeType) {
                    case \XMLReader::COMMENT :
                    case \XMLReader::WHITESPACE :
                        $OK = $xmlReader->read(); // preskoci comment
                        break;

                    case \XMLReader::TEXT :
                    case \XMLReader::CDATA :
                    case \XMLReader::SIGNIFICANT_WHITESPACE :
                        if ($rawData) {
                            $result['#children'][] = $this ->convertFromUtf8
                                ? StringUtils::fromUTF8($xmlReader->value)
                                : $xmlReader->value;
                        } else {
                            $text .= $xmlReader->value;
                        }
                        $OK = $xmlReader->read();
                        break;

                    case \XMLReader::ELEMENT :
                        $name = $xmlReader->localName;
                        if (!$this->ignorePrefixes && $xmlReader->prefix) {
                            $name = $xmlReader->prefix . ':' . $name;
                        }

                        $child = $this->readXmlElement($xmlReader, $rawData);

                        if (!$rawData) {
                            if (!isset($result[$name])) {
                                $result[$name] = $this ->returnProxyArray ? new ProxyArray() : [];
                            }

                            $result[$name][] = $child;
                        }

                        $result['#children'][] = ['name' => $name, 'element' => $child];
                        break;

                    default :
                        MessageManager::addErrorMessage('Neznamy typ dat v XML souboru! (' . $xmlReader->nodeType . ')');
                        $OK = $xmlReader->read(); // preskoci
                        return null;
                }
            }

            if (!$rawData) {
                $result['#text'] = $this->convertFromUtf8
                    ? StringUtils::fromUTF8($text)
                    : $text;
            }

            if ($xmlReader->localName != $elementName) {
                throw new \Exception("Chyba v XML. Je ocekavan konec tagu $elementName ale byl nalezen {$xmlReader->localName}!");
            }

            if (!$OK) {
                throw new \Exception("Chyba v XML v tagu $elementName!");
            }

            $xmlReader->read(); // preskoci END_ELEMENT
        }

        return $result;
    }


    /**
     * Nacte zadany XML text do asociativniho pole.
     * @param string $xmlText
     * @param string $charSet   Znakova sada, ve ktere je zadane XML. Pokud neni zadana aplikace
     * @return array
     * @throws \Exception
     */
    public function readXml($xmlText, $charSet = null)
    {
        $xmlReader = new \XMLReader();
        $xmlReader->XML($xmlText, $charSet ?: TextManager::getInstance()->charSet);

        while ($xmlReader->read()) {
            if ($xmlReader->nodeType == \XMLReader::ELEMENT) {
                $result = $this ->readXmlElement($xmlReader);
                return $result;
            }
        }

        //if (!$xmlReader->isValid()) throw new \Exception('neni valid!');
        return [];
    }


    public static function loadXmlToArray($xmlText, $params = [self::PARAM_CONVERT_FROM_UTF8 => true], $charSet = null)
    {
        $loader = new XmlLoader();
        if ($params) {
            foreach ($params as $paramName => $paramValue) {
                $loader ->{$paramName} = $paramValue;
            }
        }
        //$loader ->convertFromUtf8 = $convertFromUtf8;
        return $loader ->readXml($xmlText, $charSet);
    }


    /**
     * @param \ArrayAccess $rawData
     * @return string
     */
    public static function rawDataToString($rawData, $rootElement = null)
    {
        // prevede element nacteny jako "rawData" zpet do retezce
        if (!$rawData) {
            return '';
        }

        $result = '';

        if (is_string($rawData)) {
            $result .= $rawData;
        } else {
            //$selfClosing = $rootElement  &&  (!$rawData['#children']  ||  !count($rawData['#children']));
            $selfClosing = $rootElement  &&  !isset($rawData['#children']);

            if ($rootElement) {
                $result .= '<' . $rootElement;

                if (isset($rawData['#attr'])) {
                    foreach ($rawData['#attr'] as $name => $value) {
                        $result .= " $name='$value'";
                    }
                }
                $result .= ($selfClosing ? '/' : '') . '>';
            }

            if ($rawData['#children']) {
                foreach ($rawData['#children'] as $child) {
                    if (is_string($child)) {
                        $result .= $child;
                    } else {
                        $result .= self::rawDataToString($child['element'], $child['name']);
                    }
                }
            }

            if ($rootElement  &&  !$selfClosing) {
                $result .= '</' . $rootElement . '>';
            }
        }



        return $result;
    }
}
