<?php

namespace RonPortal\Controls;

use Ron\Controls\ComboBox;
use Ron\Controls\ComboBoxes;
use Ron\Controls\Option;
use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\Rozdeleni;
use RonPortal\RozdeleniUtils;
use RonPortal\TypRozdeleni;
use RonPortal\User;
use Tracy\Debugger;


/**
 * Obecny filtr na rozdeleni. Ve formulari muze byt pouzit vicekrat (pro ruzny typ rozdeleni)
 */
class FilterFormFilterRozdeleni extends FilterFormFilter
{
    const SPEC_TYP_ROZDELENI = 'typRozdeleni';
    const SPEC_VCETNE_PODRIZENYCH = 'vcetnePodrizenych';

    /** @var ComboBoxes    Comboboxy pro vyber rozdeleni (podle popisu a kodu) */
    protected $comboBoxesRozdeleni;

    /** @var int    Kod typu rozdeleni */
    protected $typRozdeleni;

    /** @var ?PravaNaOsoby  Typ prava, podle ktereho se seznam rozdeleni muze filtrovat */
    protected $prava;

    /** @var bool Urcuje, jestli se ma nadpis filtru nacist z databaze (TypRozdeleni.Popis pro prirazeny typ rozdeleni) */
    protected $pouzitPopisTypuRozdeleni = true;

    /**
     * @var int Urcuje, jestli se nactou jen vlastni rozdeleni
     *          0=vse, 1=jen vlastni, 2=podle prav (prava mohou byt podle jineho typu rozdeleni!), 3=podle prav, nezobrazovat nadrazene
     */
    protected $jenVlastniRozdeleni = 0;

    /** @var bool   Urcuje, jestli se ma zobrazit volba jen vybrane/vcetne podrizenych/... */
    protected $showVcetnePodrizenych = true;

    /** @var int    Hodnota pro select "vcetne podrizenych" 0=jen vybrane, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene. Po load() se nastavi na skutecnou hodnotu. */
    protected $vcetnePodrizenych = 1;


    public function __construct(RonPortalFilterForm $filterForm, string $name)
    {
        parent::__construct($name, FilterFormFilter::TYPE_OPTIONS);

        $this ->filterForm = $filterForm;

        $this ->showCode = true;
    }


    /**
     * Nastavi zobrazovany nadpis. Pokud je null, tak se nacte nazev prislusneho typu rozdeleni
     * @param ?string $caption
     * @return FilterFormFilter
     */
    public function setCaption($caption): FilterFormFilter
    {
        $this ->pouzitPopisTypuRozdeleni = $caption === null;
        return parent::setCaption($caption);
    }


    /**
     * @param bool $pouzitPopisTypuRozdeleni
     * @return FilterFormFilterRozdeleni
     */
    public function setPouzitPopisTypuRozdeleni(bool $pouzitPopisTypuRozdeleni): FilterFormFilterRozdeleni
    {
        $this->pouzitPopisTypuRozdeleni = $pouzitPopisTypuRozdeleni;
        return $this;
    }


    /**
     * @return int
     */
    public function getTypRozdeleni(): int
    {
        return $this->typRozdeleni;
    }

    /**
     * @param int $typRozdeleni
     * @return FilterFormFilterRozdeleni
     */
    public function setTypRozdeleni(int $typRozdeleni): FilterFormFilterRozdeleni
    {
        $this->typRozdeleni = $typRozdeleni;
        return $this;
    }

    /**
     * @param PravaNaOsoby|null $prava
     * @return FilterFormFilterRozdeleni
     */
    public function setPrava(?PravaNaOsoby $prava): FilterFormFilterRozdeleni
    {
        $this->prava = $prava;
        return $this;
    }

    /**
     * @param int $jenVlastniRozdeleni
     * @return FilterFormFilterRozdeleni
     */
    public function setJenVlastniRozdeleni(int $jenVlastniRozdeleni): FilterFormFilterRozdeleni
    {
        $this->jenVlastniRozdeleni = $jenVlastniRozdeleni;
        return $this;
    }


    /**
     * @param bool $showVcetnePodrizenych
     * @param int $vcetnePodrizenych
     * @return FilterFormFilterRozdeleni
     */
    public function setShowVcetnePodrizenych($showVcetnePodrizenych, $vcetnePodrizenych = 1): FilterFormFilterRozdeleni
    {
        $this->showVcetnePodrizenych = $showVcetnePodrizenych;
        $this->vcetnePodrizenych = $vcetnePodrizenych;
        return $this;
    }


    /**
     * @return bool
     */
    public function getShowVcetnePodrizenych()
    {
        return $this->showVcetnePodrizenych;
    }



    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'typRozdeleni' :
                $this ->setTypRozdeleni((int)$value);
                break;

            case 'showVcetnePodrizenych' :
            case 'jenVlastniRozdeleni' :
            case 'pouzitPopisTypuRozdeleni' :
                $this ->{$key} = (int)$value;
                break;

            case 'caption' :
                $this ->setCaption($value); // aby se nastavilo "pouzitPopisTypuRozdeleni"
                break;

            case 'prava' :
                $this ->prava = $value === ''
                    ? null
                    : Prava::vratPrava($value);
                break;

            default :
                parent::loadConfigurationItem($key, $value, $valueText);
        }
    }


    public function getValue(?string $specification = null)
    {
        switch ($specification) {
            case self::SPEC_TYP_ROZDELENI :
                return $this ->typRozdeleni;

            case self::SPEC_VCETNE_PODRIZENYCH :
                return $this ->vcetnePodrizenych;
        }

        return parent::getValue($specification);
    }


    /**
     * @param Option $option1
     * @param Option $option2
     * @return int
     */
    protected static function compareOptionsRozdeleni($option1, $option2)
    {
        if (!$option1) {
            return $option2 ? -1 : 0;
        }

        if (!$option2) {
            return 1;
        }

        if ($option1 ->key < $option2 ->key) {
            return -1;
        }
        if ($option1 ->key > $option2 ->key) {
            return 1;
        }

        return 0;
    }


    /**
     * @param Database $database
     * @param int $podrizena    0=bez podrizenych, 1=vcetne podrizenych, 2=vcetne primo podrizenych
     * @param int $datumOd      Pocatek obdobi, ve kterem se nacitaji platna rozdeleni
     * @param int $datumDo      Konec obdobi, ve kterem se nacitaji platna rozdeleni
     * @return string
     */
    protected function sestavFiltrRozdeleni(Database $database, $podrizena, $datumOd, $datumDo)
    {
        // TODO: predelat to na parametry!
        if ($podrizena == 1) {
            // vcetne podrizenych
            return
                'EXISTS (SELECT * FROM RozdeleniOsob, HierarchieRozdeleni ' .
                '  WHERE HierarchieRozdeleni.TypPodrizene = Rozdeleni.KodTypuRozdeleni' .
                '    AND HierarchieRozdeleni.KodPodrizene = Rozdeleni.Kod' .
                '    AND RozdeleniOsob.KodTypuRozdeleni = HierarchieRozdeleni.TypNadrazene' .
                '    AND RozdeleniOsob.Rozdeleni = HierarchieRozdeleni.KodNadrazene' .
                "    AND RozdeleniOsob.RC = '" . $database->addSlashes(User::getRC()) . "'" .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $database->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $database->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')';
        } else if ($podrizena == 2) {
            // vcetne primo podrizenych
            return
                'EXISTS (SELECT * FROM RozdeleniOsob, Rozdeleni R2' .
                ' WHERE R2.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                '    AND R2.Kod = RozdeleniOsob.Rozdeleni' .
                '    AND R2.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                '    AND (R2.Kod = Rozdeleni.Kod OR R2.Kod = Rozdeleni.Rodic)' .
                "    AND RozdeleniOsob.RC = '" . $database->addSlashes(User::getRC()) . "'" .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $database->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $database->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')';
        } else {
            // jen vlastni`
            return
                'EXISTS (SELECT * FROM RozdeleniOsob ' .
                '  WHERE Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
                '    AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni' .
                "    AND RozdeleniOsob.RC = '" . $database ->addSlashes(User::getRC()) . "'" .
                '    AND RozdeleniOsob.PlatnostOd <= ' . $database ->dateToString($datumDo) .
                '    AND (RozdeleniOsob.PlatnostDo >= ' . $database ->dateToString($datumOd) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
                ')';
        }
    }


    /**
     * @param Database $database
     * @param int $datumOd      Pocatek obdobi, ve kterem se nacitaji platna rozdeleni
     * @param int $datumDo      Konec obdobi, ve kterem se nacitaji platna rozdeleni
     * @return string
     */
    protected function filtrRozdeleniPodlePrav($database, $datumOd, $datumDo)
    {
        if (!$this->prava) {
            return '';
        }

        if (!$this->prava->typRozdeleni) {
            // neni nastaven typ rozdeleni
            return '';
        }

        if ($this->prava->typRozdeleni != $this ->typRozdeleni) {
            // filtruje se podle jineho typu rozdeleni nez prava
            return '';
        }

        switch ($this->prava->prava) {
            case 10 : // vlastni rozdeleni
            case 11 : //   -//- krome sebe
                return $this->sestavFiltrRozdeleni($database, 0, $datumOd, $datumDo); // podrizena = 0 (bez podrizenych)

            case 12 : // vlastni a podrizena
            case 13 : //   -//- krome sebe
            case 14 : // podrizena
            case 15 : // podrizena a sebe
                return $this->sestavFiltrRozdeleni($database, 1, $datumOd, $datumDo); // (vcetne podrizenych)

            case 22 : // vlastni a primo podrizena
            case 23 : //   -//- krome sebe
            case 24 : // primo podrizena
            case 25 : //   -//- a sebe
            case 26 : //   -//- krome sebe
                return $this->sestavFiltrRozdeleni($database, 2, $datumOd, $datumDo); // vcetne primo podrizenych
        }

        return '';
    }



    /**
     * @param Database $database
     * @param string $htmlKod
     * @return ComboBoxes
     * @throws \Exception
     */
    protected function loadDataRozdeleni(Database $database, $htmlKod)
    {
        $datumOd = $this ->filterForm ->getOmezitObdobiOd();
        $datumDo = $this ->filterForm ->getOmezitObdobiDo();

        if (($this ->jenVlastniRozdeleni === 2 || $this ->jenVlastniRozdeleni === 3)  &&  $this ->prava) {  // $this ->filterForm ->f_pravarozdeleni
            // rozdeleni se filtruje podle prav
            $sql = $this ->prava ->sqlRozdeleni($database,
                'Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass',
                (int)$this ->typRozdeleni, $datumOd, $datumDo, $this ->jenVlastniRozdeleni == 2);
        } else {
            $sql =
                'SELECT Kod, Popis, cssClass FROM Rozdeleni ' .
                ' WHERE KodTypuRozdeleni = ' . (int)$this ->typRozdeleni .
                ' AND (DatumZruseni >= ' . $database ->dateToString($datumOd) . ' OR DatumZruseni IS NULL) ';

            if ($datumDo)
                $sql .=
                    ' AND DatumVytvoreni <= ' . $database ->dateToString($datumDo);

            $filtrRozdeleni = ($this ->jenVlastniRozdeleni === 1)
                ? $this->sestavFiltrRozdeleni($database, $this ->jenVlastniRozdeleni, $datumOd, $datumDo)  // vcetne podrizenych
                : $this->filtrRozdeleniPodlePrav($database, $datumOd, $datumDo);

            if ($filtrRozdeleni) {
                $sql .= ' AND (' . $filtrRozdeleni . ')';
            }
        }

        if ($this ->optionsSqlFilter) {
            $sql = SqlStatement::concatenate([$sql, $this ->optionsSqlFilter], SqlStatement::SEPARATOR_AND);
        }

        if (isset($_GET[$this ->name])) {  // if ($this ->useGet  && isset($_GET[$this ->name]))
            // pokud pouziva GET, tak povoli v URL vybrat i rozdeleni, do ktereho vybrana osoba nepatri
            // (jinak by neslo prepnout rozdeleni)
            // POZOR! V URL pak muze byt jina osoba nez skutecne nactena
        } else if (isset($_GET['osoba'])) {
            if (($this ->value ?? '') !== '' ) {
                // pokud je nastavena osoba v URL adrese, tak zkontroluje, jestli tato osoba patri do aktualne vybraneho rozdeleni
                // a pokud ne, tak rozdeleni vypne - je to nutne, aby fungovaly napr. odkazy v emailech
                if (!Rozdeleni::jeVRozdeleni($database, null, $_GET['osoba'], $this ->typRozdeleni, $this ->value, 3, $datumOd, $datumDo)) { // 3 = vcetne podrizenych
                    $this ->value = null;
                }
            }
        }

//        MessageManager::addDebugLog('Vyber rozdeleni ' . $this ->typRozdeleni . ':');
//        MessageManager::addDebugLog($sql);

        // vyber rozdeleni podle popisu
        $combo1 = new ComboBox($this ->name);
        $combo1
            ->setClassName('form-control adsfilterforminput_popis ronselect')
            ->setCssClassIndex(2)
            ->setShowCode($this ->showCode)
            ->setEmptyRow(TextManager::_('_vse_'))
            ->setTranslationPrefix("db_rozdeleni_{$this ->typRozdeleni}_popis_")
            ->setHtmlParams(
                array_merge(
                    [
                        'data-kod-typu-rozdeleni' => $this ->typRozdeleni,
                        'data-ron-select2' => "#ronportalfilter_{$this ->name}_kod",
                        'data-ron-autosubmit' => ($this ->filterForm ->autoSubmit ? '1' : '0'),
                        'data-insert-position' => 'before',
                        'readonly' => 'readonly'
                    ],
                    $this ->htmlAttributes ?? []
                )
            );

        $sqlPopis = $sql instanceof SqlStatement
            ? new SqlStatement($sql ->sqlText . ' ORDER BY 2', $sql ->parameters)
            : $sql . ' ORDER BY Rozdeleni.Popis';
        $combo1->loadData($database, $sqlPopis, $this ->value);

        $options2 = [];
        foreach ($combo1 ->data as $option) {
            $option->params['data-popis'] = $option ->caption;
            $options2[] = $option;
        }

        usort($options2, 'RonPortal\Controls\FilterFormFilterRozdeleni::compareOptionsRozdeleni');

        // vyber kodu rozdeleni
        $combo2 = new ComboBox(null); // $htmlKod . '_kod'
        $combo2->className = 'form-control adsfilterforminput_kod desktop-only ronselect';
        $combo2->cssClassIndex = 2;
        $combo2->captionIndex = 0; // kod - kod
        $combo2->emptyRow = TextManager::_('_vse_');

        $combo2 ->valueFieldName = 'key';
        $combo2 ->keyFieldName = 'key';
        $combo2 ->titleFieldName = 'caption';
        $combo2 ->cssClassFieldName = 'cssClass';

        $combo2 ->addOptions($options2);

        $combo2->setSelectedOption($this ->value);

        $combo2->htmlParams =
            "data-ron-select2='{$this ->name}'" . ($this ->filterForm ->autoSubmit ? ' data-ron-autosubmit="1"' : '') .
            ' data-columns="value;popis"' .
            ' data-drop-down-align="right"' .
            " id='ronportalfilter_{$htmlKod}_kod'";

        return new ComboBoxes($combo1, $combo2);
    }


    public function loadOptions(Database $database): FilterFormFilter
    {
        return parent::loadOptions($database); // TODO: Change the autogenerated stub
    }


    protected function loadSubmittedValue()
    {
        $result = parent::loadSubmittedValue();

        // krome kodu rozdeleni nacte jeste volbu "vcetne podrizenych"
        if ($result  &&  $this ->showVcetnePodrizenych) {
            $this ->vcetnePodrizenych = (int)($_GET[$this ->name . '_vcetne_podrizenych'] ?? 1);
        }

        return $result;
    }

    public function storeValue(Database $database)
    {
        parent::storeValue($database);

        if ($this ->showVcetnePodrizenych) {
            $this ->storeValueByName($database, $this ->vcetnePodrizenych, 1, '_vcetne_podrizenych');
        }
    }


    public function loadDefaultValue(Database $database)
    {
        parent::loadDefaultValue($database);

        if ($this ->showVcetnePodrizenych) {
            $this ->vcetnePodrizenych = (int)$this ->getDefaultValueByName($database, 1, '_vcetne_podrizenych');
        }
    }


    public function load(Database $database)
    {
        parent::load($database);

        if ($this->pouzitPopisTypuRozdeleni) {
            $this->caption = StringUtils::htmlTextConvert(TypRozdeleni::getPopisTypuRozdeleni($database, $this ->typRozdeleni));
        }

        $this ->comboBoxesRozdeleni = $this ->loadDataRozdeleni($database,'rozdeleni');

        if (isset($this ->comboBoxesRozdeleni ->combo1 ->selectedOption)) {
            $this ->value = $this ->comboBoxesRozdeleni ->combo1 ->selectedOption ->key;
            //RonPortalSession::setRozdeleni($this ->comboBoxesRozdeleni ->combo1 ->selectedOption ->key, $typRozdeleniSuffix);
        } else {
            $this ->value = null;
        }
    }

    public function render()
    {
        $this ->comboBoxesRozdeleni ->combo1 ->render();
        $this ->comboBoxesRozdeleni ->combo2 ->render();

        if ($this->showVcetnePodrizenych) {
            echo
                "<select name='{$this ->name}_vcetne_podrizenych'" . ($this ->filterForm ->autoSubmit ? ' data-ron-autosubmit="1"' : '') . '>';

            GuiUtils::renderOptions(
                [
                    0 => TextManager::_('Rozdeleni_jen_vybrane'),
                    1 => TextManager::_('Rozdeleni_vcetne_podrizenych'),
                    2 => TextManager::_('Rozdeleni_jen_podrizena'),
                    3 => TextManager::_('Rozdeleni_jen_primo_podrizena')
                ],
                $this ->vcetnePodrizenych
            );

            echo
                '</select>';
        }
    }


    /**
     * Vraci SQL filtr ( EXISTS ... ) pro vybrane rozdeleni
     * @param Database $database
     * @param string $tabulka   Jmeno tabulky, jejiz zaznamy se filtruji (musi obsahovat pole RC a pripadne take OsobniCislo a DatumNastupu - viz parametr $jenRC)
     * @param null $params      DB parametry
     * @param bool $jenRC       Urcuje, jestli se ma filtrovat jen podle RC (vychozi je i podle OsobniCislo a DatumNastupu)
     * @return string
     */
    public function sestavSQLFiltr($database, $tabulka, $params = null, $jenRC =  false)
    {
        if (!$this->enabled) {
            return '';
        }
        if (!$this->typRozdeleni) {
            return '';
        }
        if ($this ->getValue() === null) {
            return '';
        }

        $datumOd = $this ->filterForm ->getOmezitObdobiOd();
        $datumDo = $this ->filterForm ->getOmezitObdobiDo();

        return RozdeleniUtils::sqlFilterRozdeleniPP($database, $tabulka,
            $this->typRozdeleni, $this ->getValue(),
            $datumOd, $datumDo,
            $this ->vcetnePodrizenych,
            $params,
            $jenRC
        );
    }

}

