<?php

namespace RonPortal\Dochazka;

use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\DBRecord;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Controls\RozdeleniFormField;
use RonPortal\NastaveniUzivatele;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalXMLEvents;
use RonPortal\TypRozdeleni;
use RonPortal\User;
use Tracy\Debugger;


class VyberPruchoduFormField extends FormField
{
    /** @var OsobaFormField */
    private $osobaField;

    public function __construct($osobaField)
    {
        parent::__construct(FormField::FIELD_TYPE_SELECT);
        $this ->osobaField = $osobaField;

        $this ->setFieldId('pruchody'); // pouzito v JS!
    }


    public function loadOptions($database)
    {
        $this ->options = VkladaniDochazkyController::vyberPruchoduProVkladani($database, $this ->osobaField ->value);
    }
}


/**
 * Formular pro vlozeni zaznamu do puvodni dochazky
 */
class VkladaniDochazkyForm extends RonPortalForm
{
    const SETTINGS_PODLE_HISTORIE = 'vkladani_casypodlehistorie';

    /** @var string  jestli se ma pred vlozenim kontrolovat datum schvaleni/zkontrolovani/zpracovani
     *               0 = nekontrolovat, 1 = kontrolovat schvaleni, 2 = kontrolovat zkontrolovani, 3 = kontrolovat zpracovani */
    const SETTINGS_KONTROLA_SCHVALENI = 'kontrolavkladani';

    /** @var bool   Urcuje, jestli se jedna o vkladani do HistoriePruchodu */
    private $historie;

    /** @var bool   Nastavi se pri nacitani seznamu operaci na true, pokud ma nektera operace editovatelny cas */
    private $jeEditovatelnaOperace = false;

    private $backAction;

    /** @var OsobaFormField */
    private $fieldOsoba;

    public function __construct($database, $historie = false)
    {
        parent::__construct($database);
        $this ->historie = $historie;

        $this ->code = $historie
            ? 'formvkladanihistorie'
            : 'formvkladanidochazky';

        $this ->sqlTableName = $historie ? 'HistoriePruchodu' : 'PuvodniDochazka';
        $this ->title = $historie
            ? RonPortalSettings::getValue(RonPortalSettings::HISTORIE_NADPIS)
            : TextManager::_('Vkladani_operace');

        if ($this ->historie) {
            $this ->setXmlEventsName('HISTORIE_PRUCHODU');
        } else {
            $this->xmlEventBeforePost = RonPortalXMLEvents::VLOZENI_BEFORE;
            $this->xmlEventAfterPost = RonPortalXMLEvents::VLOZENI_AFTER;
        }

        $prava = Prava::vratPrava($this ->historie ? 'historie' : 'vkladani');
        $this ->htmlParams = [
            'id' => 'vkladani_dochazky_form',
            'data-muze-menit-cas' => $prava ->priznak ? 1 : 0
        ];


        $this ->backAction = StringUtils::secureString(trim($_POST['backaction'] ?? $_GET['backaction'] ?? ''));

        $this ->fieldOsoba = new OsobaFormField($this ->historie ? 'historie' : 'vkladani', 0); // datum platnosti PP = 0, lze editovat = 0
        $this ->fieldOsoba
            ->setCode('field-osoba') ->setFieldId('vyberOsobyProVkladani')
            //->setDefaultValue($this ->idPP)
            ->setCaption('Osoba')
            ->setReadOnly(true)
            ->setIncludeToDBRecord(true);
        $this ->fieldOsoba ->filtr = 'PracovniPomer.ZpracovatDochazku = 1';
        $this ->addField($this ->fieldOsoba);

        // operace
        $form = $this;
        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodOperace') ->setRequired(true)
            ->setCode('field-operace')
            ->setCaption('Operace')
            ->setFieldId('vyberOperaceProVkladani')
            ->addFieldCssClass('ronselect')
            ->setSelectSql(
                'SELECT Kod, Popis, LzeNastavitCas_Intraweb FROM OperaceDochazky' .
                ' WHERE %FILTER%' .
                ' ORDER BY %ORDER%', 'ManualneVkladat_Intraweb = 1', 'Popis')
            ->setItemsAttributes(['data-lze-nastavit-cas' => 2]) // u kazde option prida priznak, jestli lze pro danou operaci rucne nastavit cas
            ->setSelectFilterFunction( function($row) use ($form){
                return $form ->zpracujRadekOperace($row);
            });

        // rozdeleni
        if ($kodTypuRozdeleni = RonPortalSettings::getIntValue('vkladani_rozdeleni')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($this ->database, $kodTypuRozdeleni);

            $fRozdeleni = new RozdeleniFormField($kodTypuRozdeleni);
            $fRozdeleni
                ->setFieldName('Rozdeleni') ->setTypRozdeleniFieldName('KodTypuRozdeleni')
                ->setCode('field-rozdeleni')
                ->setCaption(StringUtils::htmlTextConvert($typRozdeleni ->popis))
                ->setRequired(RonPortalSettings::getIntValue('vkladani_povinnerozdeleni'))
                ->setDefaultValue($_COOKIE['vkladani_rozdeleni'] ?? null)
                ;
            $this ->addField($fRozdeleni);
        }

        // terminal
        if ($historie) {
            $this ->addField(FormField::FIELD_TYPE_CONSTANT)
                ->setFieldName('Terminal')
                ->setReadOnly(true)
                ->setCode('field-terminal')
                ->setDefaultValue(RonPortalSettings::getValue(RonPortalSettings::HISTORIE_TERMINAL));
        } else {
            if (RonPortalSettings::getIntValue('vkladani_zpusob_terminal') == VkladaniDochazkyController::TERMINAL_RUCNE) {
                // terminal vybere uzivatel
                $vybranyTerminal = NastaveniUzivatele::getUserSettingsItem($this->database, 'vkladani_vybrany_terminal');

                $this->addField(FormField::FIELD_TYPE_SELECT)
                    ->setFieldName('Terminal')
                    ->setCode('field-terminal')
                    ->setFieldId('vyberTerminaluProVkladani')
                    ->setCaption('Terminal')
                    ->setSelectSql(
                        'SELECT Kod, Popis FROM Terminal' .
                        ' WHERE %FILTER% ORDER BY %ORDER%', 'Aktivni = 1 AND KodUrceni IN (1, 6, 7)', 'Popis')
                    ->setDefaultValue($vybranyTerminal)
                    ->setRequired(RonPortalSettings::getIntValue(VkladaniDochazkyController::SETTINGS_POVINNY_TERMINAL));
            } else {
                $vybranyTerminal = VkladaniDochazkyController::nactiZvolenyTerminal($this ->database);
                if ($vybranyTerminal) {
                    $this ->addField(FormField::FIELD_TYPE_TEXT)
                        ->setCode('field-popis-terminalu')
                        ->setCaption('Terminal')
                        ->setDefaultValue(StringUtils::htmlTextConvert($vybranyTerminal['popis'] ?? ''));
                }

                $this ->addField(FormField::FIELD_TYPE_CONSTANT)
                    ->setFieldName('Terminal')
                    ->setReadOnly(true)
                    ->setCode('field-terminal')
                    ->setDefaultValue($vybranyTerminal['kod'] ?? null);
            }
        }

        // datum a cas
        $this ->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-datum-cas')
            ->addFieldCssClass('margin-top-3')
            ->setFieldId('group_datum_cas');

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('*DatumOperace')
            ->setCode('field-datum-operace')
            ->setCaption('Datum')   // vice dnu => nastavit na Datum_od
            ->setFieldId('datumOperace')
            ->setDefaultValue(StringUtils::format('date', time()));

        if (!$this ->historie  &&  $prava ->priznak) {
            // muze menit cas - zobrazi vkladani pro vice dnu
            $viceDnu = (int)($_POST['vicednu'] ?? 0);
            $this ->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('*vicednu')
                ->setCode('field-vice-dnu')
                ->setFieldId('vkladani_povolit_vice_dnu')
                ->setDefaultValue($viceDnu);

            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setFieldCssClass('vkladani-vice-dnu js toggleClass setValue' . ($viceDnu ? ' d-none' : ''))
                ->setText(TextManager::_('Vice_dnu'))
                ->setOtherHtmlParams([
                    'data-selector' => '.vkladani-vice-dnu',
                    'data-toggle-class' => 'd-none',
                    'data-setvalue-selector' => '#vkladani_povolit_vice_dnu',
                    'data-value' => '1'
                ]);

            $this->addField(FormField::FIELD_TYPE_DATE)
                ->setFieldName('*DatumOperaceDo')
                ->setCode('field-datum-do')
                ->setCaption('Datum_do')
                ->setFieldId('datumOperaceDo')
                ->setRowClass('row vkladani-vice-dnu' . ($viceDnu ? '' : ' d-none'))
                ->setDefaultValue(StringUtils::format('date', time()));
        }

        $this ->addField(FormField::FIELD_TYPE_TIME)
            ->setFieldName('*CasOperace')
            ->setCode('field-cas-operace')
            ->setCaption('Cas')
            ->setFieldId('casOperace')
            ->setDefaultValue(StringUtils::format('fulltime', time()));

        $this ->addField(FormField::FIELD_TYPE_GROUP_END);

        if (!$historie  &&  !$prava ->priznak  &&  RonPortalSettings::getIntValue(self::SETTINGS_PODLE_HISTORIE)) {
            // je povolen vyber casu podle historie pruchodu
            // a uzivatel nemuze editovat cas => nabidne mu casy podle pruchodovych terminalu
            $this ->addField(new VyberPruchoduFormField($this ->fieldOsoba))
                ->setFieldName('*pruchody')
                ->setCode('field-pruchody')
                ->setRowId('row_pruchody')
                ->setRowClass('row margin-top-3')
                ->setCaption('Datum_a_cas')
                ->setDefaultValue(0)
                ->setDataType('int');
        }

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('VlozenoManualne')
            ->setDefaultValue(1)
            ->setFieldId('field-vlozenomanualne')
            ->setDataType('int');

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('VlozenoOnline')
            ->setDefaultValue(0)
            ->setFieldId('field-vlozenoonline')
            ->setDataType('int');

        if ($this ->historie) {
            $this ->addField(FormField::FIELD_TYPE_CONSTANT)
                ->setFieldName('Uspesny')
                ->setDefaultValue(1)
                ->setFieldId('field-uspesny')
                ->setDataType('int');
        }

        $this ->addField()
            ->setFieldName('Poznamka') ->setCaption('Poznamka')
            ->setRowClass('row margin-top-3')
            ->setMaxLength(40);

        $this ->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('Uzivatel')
            ->setReadOnly(true)
            ->setDefaultValue(User::getLoginName());

        $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('*vkladaniaction') //->setSqlFieldName(null)
            ->setDefaultValue($historie ? 'vlozdohistorie' : 'provedvlozeni');

        if ($this ->backAction) {
            $this->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('*backaction') //->setSqlFieldName(null)
                ->setDefaultValue($this->backAction);
        }

        if ( RonPortalSettings::getIntValue(VkladaniDochazkyController::SETTINGS_KARTA, 1) == 3) {
            // pro nacteni IP pomoci Java - NEPODPOROVANO
            $this ->addField(FormField::FIELD_TYPE_HIDDEN)
                ->setFieldName('*IPAdresaJava')
                ->setDefaultValue('x');
        }
    }


    /**
     * Pri nacitani seznamu operaci pro combobox filtruje operace podle prav uzivatele (priznak3)
     * a zjistuje, jestli existuje operace s editovatelnym casem
     * @param array $row
     * @return bool
     */
    private function zpracujRadekOperace($row)
    {
        $result = VkladaniDochazkyController::lzeVlozitOperaci($row[0], $this ->historie);
        if ($result  &&  $row[2]) {
            $this ->jeEditovatelnaOperace = true;
        }
        return $result;
    }


    /**
     * Kontrola, jestli neni schvalena dochazka (dle nastaveni Portalu)
     * @param PracovniPomer $pPomer
     * @param int $datum
     * @param int $datumDo
     * @return void
     * @throws \Exception
     */
    private function zkontrolujSchvaleni($pPomer, $datum, $datumDo)
    {
        if (RonPortalSettings::getIntValue(self::SETTINGS_KONTROLA_SCHVALENI) > 0) {
            // zkontroluje, jestli se nevklada do schvalene dochazky
            // ma smysl predevsim kdyz je povoleno zpetne vkladani
            $schvaleni = new SchvaleniDochazky();
            $schvaleni->load($this->database, $pPomer->rc, $pPomer->osobniCislo, $pPomer->datumNastupu, $datum, $datumDo);

            switch (RonPortalSettings::getIntValue(self::SETTINGS_KONTROLA_SCHVALENI)) {
                case 1 :
                    if ($schvaleni && $datum <= $schvaleni->schvaleni3) {
                        throw new \Exception(TextManager::_('Datum_mensi_nez_schvaleni'));
                    }

                case 2 :
                    if ($schvaleni && $datum <= $schvaleni->schvaleni2) {
                        throw new \Exception(TextManager::_('Datum_mensi_nez_zkontrolovani'));
                    }

                case 3 :
                    if ($schvaleni && $datum <= $schvaleni->schvaleni1) {
                        throw new \Exception(TextManager::_('Datum_mensi_nez_zpracovano_do'));
                    }
            }
        }
    }


    /**
     * @return string|null
     */
    private function vratCisloKarty($datum) {
        // Puvodne se to pro historii urcilo vzdy z prirazeni karty. Uprava 10-2023 - pokud to jde urcit dle nastaveni portalu, tak uz to z prirazeni nenacita
        $result = VkladaniDochazkyController::vratCisloKarty();

        if ($this ->historie  &&  !$result) {
            $pracovniPomer = $this ->fieldOsoba ->pracovniPomer; // Az po parent::createDBRecordFields ! ! !

            $prirazeniKarty = PrirazeniKarty::vratCisloKarty(
                $this ->database, $datum,
                $pracovniPomer ->rc, $pracovniPomer ->osobniCislo, $pracovniPomer ->datumNastupu);

            if (!$prirazeniKarty) {
                throw new \Exception( TextManager::_('Neni_prirazena_karta') );
            }

            return $prirazeniKarty ->kod;
        }

        return $result;
    }


    public function createDBRecordFields($rec, $insert)
    {
        if (isset($_POST['Rozdeleni'])) { // TODO: umoznit nastavit cookiename obecne u FormField
            setcookie('vkladani_rozdeleni', $_POST['Rozdeleni'], time() + 30000000);
        }

        parent::createDBRecordFields($rec, $insert);

        // karta
        $datumCas = $rec[$this ->historie ? 'DatumCasPruchodu' : 'DatumCasOperace'];
        if (!$datumCas) {
            throw new \Exception('Chyba v metode VkladaniDochazkyForm::createDBRecordFields - neni nastaven cas');
        }

        $rec ->addStringField('Karta',true, 30, $this ->vratCisloKarty($datumCas));
    }


    function getXmlEventsParams($rec, $context, $eventName)
    {
        $pPomer = $this ->fieldOsoba ->pracovniPomer;
        return ['idpp' => $pPomer ->idPP, 'jmenoPrijmeni' => $pPomer ->jmenoPrijmeni];
    }


    /**
     * @param DBRecord $rec
     * @param int $datumCas
     * @return false|void
     * @throws \Ron\DB\DBException
     */
    private function vlozJednuOperaci($rec, $datumCas, $provestUpdate)
    {
        $rec[$this ->historie ? 'DatumCasPruchodu' : 'DatumCasOperace'] = $datumCas;

        $params = $this ->database ->createParameters();
        $filtr = 'DatumCasOperace = ' . $params ->addTimestampParameter($datumCas); // jen pro PuvodniDochazka

        if (!$this ->historie  &&  $rec ->loadRecord($filtr, $params)) {
            // pri vkladani podle historie muze provadet update
            if (!$provestUpdate) {
                throw new \Exception(TextManager::_('Chyba_vkladani_puvodni_existuje'));
            }

            if (!$this ->beforeUpdateRecord($rec)) {
                return false;
            }
            $rec ->updateRecord($filtr, $params);
            return $this ->afterUpdateRecord($rec);
        }

        if (!$this->beforeInsertRecord($rec)) {
            return false;
        }

        $rec->insertRecord();

        MessageManager::addInfoMessage( sprintf(TextManager::_('Vlozeni_probehlo'), date('j.n.Y G:i', $datumCas)) );

        return $this->afterInsertRecord($rec);
    }


    public function insertRecord()
    {
        // V pripade, ze se vybira cas z historie pruchodu, tak musi jiz existujici zaznam aktualizovat
        // - proto nepouziva parent...
        // x return parent::insertRecord();

        if (!$this->validate(false)) {
            return false;
        }

        $rec = $this->createDBRecord();

        $prava = Prava::vratPrava($this ->historie ? 'historie' : 'vkladani');
        $lzeMenitCas = $prava->priznak;

        $datum = null;
        $viceDnu = !$this ->historie && $prava->priznak  &&  $this ->findFieldByName('vicednu') ->value;
        $casStr = null;

        $kodOperace = $this ->findFieldByName('KodOperace') ->getPostValue($this ->utf8);

        if (!VkladaniDochazkyController::lzeVlozitOperaci($kodOperace, $this ->historie)) {
            throw new NeniPravoException();
        }


        if (!$lzeMenitCas) {
            $lzeMenitCas = OperaceDochazky::lzeNastavitCasOperace($this ->database, $kodOperace);
        }
        $provest_update = false;

        if ($lzeMenitCas) {
            // datum nacte rovnou jako hodnotu
            $datum = $this ->findFieldByName('DatumOperace') ->getPostValue($this ->utf8);
            if (!$datum) {
                throw new RonException(TextManager::_('Musite_zadat_datum_operace'), 'DatumOperace');
            }
            // cas nacte jako text - pro vkladani vice dnu jej musi prevadet pro kazdy den zvlast kvuli prechodu letni/zimni
            $casStr = $this ->findFieldByName('CasOperace') ->getPostValueText($this ->utf8);
            if (!$casStr) {
                throw new RonException(TextManager::_('Nutno_zadat_cas_operace'), 'CasOperace');
            }
            if ($viceDnu && !$_POST['DatumOperaceDo']) {
                throw new RonException(TextManager::_('Nutno_zadat_datum_operace_do'), 'DatumOperaceDo');
            }
            $datumCas = DateUtils::timeStringToTimestamp($casStr, $datum);
        } else {
            $viceDnu = false;

            if (!$this ->historie  &&  RonPortalSettings::getIntValue(self::SETTINGS_PODLE_HISTORIE) && ($_POST['pruchody'] ?? false)) {
                // je vybran zaznam z historie pruchodu
                // TODO: tady by mela probehnout kontrola existance pruchodu!
                $datumCas = (int)$_POST['pruchody'] ?: time();
                $provest_update = true; // v pripade existence zaznamu se pokusi provest update
            } else {
                // pouzije se aktualni cas
                $datumCas = time();
            }
        }

        // zkontroluje jestli nebylo zadano datum mensi nez posledni zpracovani
        $zpracovani = DochazkaUtils::datumCasZpracovaniDochazky($this->database);
        if (DateUtils::incDay($datumCas, RonPortalSettings::getIntValue(VkladaniDochazkyController::SETTINGS_POCET_DNU_ZPETNE)) <= $zpracovani) {
            throw new \Exception(TextManager::_('Datum_mensi_nez_posledni_zpracovani'));
        }

        // prida pole s casem
        $rec ->addTimestampField($this ->historie ? 'DatumCasPruchodu' : 'DatumCasOperace', true, $datumCas);

        $this ->createDBRecordFields($rec, true);

        // Az po createDBRecordFields ! ! !
        $pracovniPomer = $this ->fieldOsoba ->pracovniPomer; // Az po parent::createDBRecordFields ! ! !
        if (!$prava ->lzeZobrazitPP($pracovniPomer ->idPP)) {
            throw new NeniPravoException();
        }
        $this ->zkontrolujSchvaleni($pracovniPomer, $datumCas, $datumCas);

        if (RonPortalSettings::getIntValue(VkladaniDochazkyController::SETTINGS_POVINNY_TERMINAL)) {
            if (!$rec['Terminal']) {
                throw new \Exception(TextManager::_('Neni_terminal_vkladani'));
            }
        }


        $params = $this ->database ->createParameters();
        $filtr =
            'RC = ' . $params ->addParameter($pracovniPomer ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pracovniPomer ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addDateParameter($pracovniPomer ->datumNastupu);
            //' AND DatumCasOperace = ' . $params ->addTimestampParameter($datumCas);

        $rec ->setWhere($filtr, $params);

        if ($viceDnu  &&  $casStr  &&  $this ->findFieldByName('DatumOperaceDo') ->value) {
            $datumDo = $this ->findFieldByName('DatumOperaceDo') ->getPostValue($this ->utf8);
            if ($datumDo > DateUtils::lastDayOfMonth($datumCas)) {
                // datum do presahuje do dalsiho mesice - zjistim, jestli existuje obdobi a pokud ne, tak nepovolim vkladat
                // (jinak hrozi, ze uzivatel omylem navklada zaznamy do daleka budoucnosti)
                $maxObdobi = DBUtils::getFormatedValue($this ->database, 'date',
                    "SELECT MAX(PocatekObdobi) FROM Obdobi WHERE Aplikace = 'ADS'");
                if ($datumDo > DateUtils::lastDayOfMonth($maxObdobi)) {
                    throw new \Exception(TextManager::_('Datum_do_nesmi_byt_vetsi_nez_posledni_obdobi_dochazky'));
                }
            }

            for (;  $datum <= $datumDo;  $datum = DateUtils::incDay($datum, 1)) {
                $datumCas = DateUtils::timeStringToTimestamp($casStr, $datum);
                $result = $this->vlozJednuOperaci($rec, $datumCas, $provest_update);
            }
        } else {
            $result = $this->vlozJednuOperaci($rec, $datumCas, $provest_update);
        }

        VkladaniDochazkyController::aktualizujMonitorovani($this ->database,
            $pracovniPomer, $datumCas, $rec['KodOperace'], $rec['Poznamka'], $rec['Terminal'], $this);

        return $result;
    }


    public function getButtons($insert)
    {
        $result = parent::getButtons($insert);

        if (RonPortalSettings::getIntValue(self::SETTINGS_PODLE_HISTORIE)) {
            $result['bezuzavreni'] = [
                    'type' => 'submit',
                    'class' => 'button',
                    'name' => 'bezuzavreni',
                    'value' => TextManager::_('Vlozit_bez_uzavreni')
                ];
        }

        return $result;
    }


    public function renderHeader()
    {
        parent::renderHeader();

        if (!$this ->historie  &&  RonPortalSettings::getIntValue('vkladani_povolit_terminal', 1)) {
            echo
                '<div id="vlozeni_terminalem" class="vkladani_link margin-top-2">' .
                '<a href="?menuid=vlozeni&terminalmode=1' . ($this ->backAction ? "&backaction={$this ->backAction}" : "") . '">' .
                TextManager::_('Vlozeni_bez_podrobnosti') .
                '</a>' .
                '</div>';
        }

    }

}
