<?php
/**
 * stanislav.elbl, 22.02.2021
 */

namespace RonPortal\Dokumenty;

use Ron\DateUtils;
use Ron\DB\DBRecord;
use Ron\DB\DBUtils;
use Ron\FileUtils;
use Ron\MVC\JsonOutputObject;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use RonPortal\Log;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;


class DokumentyUtils
{
    const UPLOAD_MIME_TYPES = 'image/*;application/pdf;application/csv;application/msword;application/*opendocument*;application/*officedocument*;application/*powerpoint;application/*excel;application/rtf;text/plain';
    const UPLOAD_EXTENSIONS = 'jpg;jpeg;gif;png;pdf;csv;doc;docx;ods;odt;odp;ppt;pptx;rtf;txt;xls;xlsx';

    /**
     * Vlozi do databaze novy dokument. Vraci id dokumentu.
     * @param Database $database
     * @param string $data      Obsah dokumentu
     * @param string $fileName  Jmeno souboru
     * @param int    $typ       Typ dokumentu (odkaz do tabulky TypDokumentu)
     * @param string $popis
     * @param object|string $logSrcDescription
     * @return RonPortalDBRecord
     * @throws DBException
     * @throws \Exception
     */
    public static function vytvorDokumentRec($database, $data, $fileName, $typ, $popis, $logSrcDescription)
    {
        // zkontroluje, jestli jsou zadana povinna pole
        if (!isset($typ) || $typ === '') {
            throw new \Exception( TextManager::_('Neni_zadan_typ_dokumentu') );
        } else if (!isset($popis) || $popis == '') {
            throw new \Exception( TextManager::_('Neni_zadan_popis_dokumentu') );
        }

        $fileName = FileUtils::truncateFileName($fileName, 160);

        $data = gzcompress($data);

        $rec = new RonPortalDBRecord($database, 'Dokument', $logSrcDescription);
        $rec ->addField('Id', 'serial');
        $rec ->addField('Popis', '', false, 160, $popis);
        $rec ->addField('Typ', 'int', false, 0, (int)$typ);
        $rec ->addField('Datum', 'timestamp', false, 0, DateUtils::now());
        $rec ->addField('TypKomprimace', 'int', false, 0, 1);
        $rec ->addField('JmenoSouboru', '', false, 160, $fileName);
        $rec ->addField('Insert_UserName', '', false, 25, RonPortalUtils::getLogUserName());
        $rec ->addField('Insert_TimeStamp', 'timestamp', false, 0, time());
        $rec ->addField('Data', 'binary', false, 0, $data);
        $rec ->insertRecord();

        return $rec;
    }


    /**
     * Vlozi do databaze novy dokument. Vraci id dokumentu.
     * @param Database $database
     * @param string $data      Obsah dokumentu
     * @param string $fileName  Jmeno souboru
     * @param int    $typ       Typ dokumentu (odkaz do tabulky TypDokumentu)
     * @param string $popis
     * @param object|string $logSrcDescription
     * @return int
     * @throws DBException
     * @throws \Exception
     */
    public static function vytvorDokument($database, $data, $fileName, $typ, $popis, $logSrcDescription = 'Dokumenty')
    {
        $rec = self::vytvorDokumentRec($database, $data, $fileName, $typ, $popis, $logSrcDescription);
        return $rec['Id'];
    }


    public static function fileInputAcceptAttribute()
    {
        $mimeTypes = RonPortalSettings::getValue('upload_mime_types', self::UPLOAD_MIME_TYPES);
        if (!$mimeTypes) {
            return '';
        }

        return 'accept="' . StringUtils::htmlTextConvert(str_replace(';', ',', $mimeTypes)) . '"';
    }

    /**
     * Vraci false v pripade, ze neni zadany mime typ povolen pro upload
     * @param string $fileMimeType  Mime typ nahravaneho souboru
     * @return bool
     */
    public static function mimeTypeAllowed($fileMimeType)
    {
        $mimeTypes = RonPortalSettings::getValue('upload_mime_types', self::UPLOAD_MIME_TYPES);
        if (!$mimeTypes) {
            // mime typy nejsou omezeny
            return true;
        }

        $allowedTypes = explode(';', $mimeTypes);

        foreach ($allowedTypes as $allowedType) {
            $allowedType = trim($allowedType);

            if ($allowedType === '*') {
                // * => povoleny vsechny mime typy
                return true;
            }

            if (!strcasecmp($allowedType, $fileMimeType)) {
                // string odpovida
                return true;
            }

            if (fnmatch($allowedType, $fileMimeType)) {
                // string odpovida masce (napr. image/*)
                return true;
            }
        }

        return false;
    }


    /**
     * Vraci false v pripade, ze neni mime typ zadaneho souboru povolen pro upload
     * @param $fileName
     * @return bool
     * @throws \Exception
     */
    public static function fileMimeTypeAllowed($fileName)
    {
        if (!function_exists("mime_content_type")) {
            // nelze zkontrolovat mime_type - neni zapnuto rozsireni fileinfo
            return true;
        }

        if (!RonPortalSettings::getValue('upload_mime_types', self::UPLOAD_MIME_TYPES)) {
            return true;
        }

        $fileMimeType = mime_content_type($fileName);
        if (!$fileMimeType) {
            throw new \Exception('Pri zjistovani typu souboru doslo k chybe!');
        }

        return self::mimeTypeAllowed($fileMimeType);
    }


    /**
     * Vraci true, pokud je pripona souboru povolena pro upload
     * @param $fileName
     * @return bool
     */
    public static function fileExtensionAllowed($fileName)
    {
        $ext = strtolower(pathinfo($fileName, PATHINFO_EXTENSION));
        $extensions = strtolower(RonPortalSettings::getValue('upload_extensions', self::UPLOAD_EXTENSIONS));
        if ($extensions === '*') {
            return true;
        }

        return in_array($ext, explode(';', $extensions));
    }


    /**
     * Zkontroluje typ nahraneho souboru. V pripade, ze se jedna o nepovoleny typ souboru, tak vyhodi vyjimku.
     * @param $htmlField
     * @throws \Exception
     */
    public static function verifyFileType($htmlField)
    {
        if (!self::fileMimeTypeAllowed($_FILES[$htmlField]['tmp_name'])) {
            throw new \Exception( sprintf(TextManager::_('Err_nepovoleny_typ_souboru'), mime_content_type($_FILES[$htmlField]['tmp_name'])) );
        }

        if (!self::fileExtensionAllowed($_FILES[$htmlField]['name'])) {
            throw new \Exception( TextManager::_('Err_nepovolena_pripona_souboru') );
        }
    }


    public static function handleUploadError($htmlField)
    {
        switch ($_FILES[$htmlField]['error']) {
            case UPLOAD_ERR_OK :
                break;

            case UPLOAD_ERR_INI_SIZE :
            case UPLOAD_ERR_FORM_SIZE :
                throw new \Exception( TextManager::_('Err_velky_soubor') );

            case UPLOAD_ERR_PARTIAL :
                throw new \Exception( TextManager::_('Err_nekompletni_soubor') );

            case UPLOAD_ERR_NO_FILE :
                throw new \Exception( TextManager::_('Err_chybi_soubor'));

            case UPLOAD_ERR_NO_TMP_DIR :
                throw new \Exception( TextManager::_('Err_chybi_slozka') );

            case UPLOAD_ERR_CANT_WRITE :
                throw new \Exception( TextManager::_('Err_chyba_zapisu') );

            case UPLOAD_ERR_EXTENSION :
            default :
                throw new \Exception( sprintf(TextManager::_('Err_chyba_nahrani_souboru'), $_FILES[$htmlField]['error']) );
        }

        if (!is_uploaded_file($_FILES[$htmlField]['tmp_name'])) {
            throw new \Exception( TextManager::_('Err_chybi_soubor'));
        }
    }

    /**
     * Vlozi do databaze novy dokument. Vraci id dokumentu.
     * @param Database $database
     * @param string $htmlField
     * @param int $typ
     * @param string $popis
     * @param object|string $logSrcDescription
     * @param bool $utf8    pokud je true, tak se popis a nazev souboru prevedou z utf8 (je to potreba pri uploadu z javascriptu)
     * @return int
     * @throws DBException
     * @throws \Exception
     */
    public static function vlozDokument($database, $htmlField, $typ, $popis, $logSrcDescription = 'Dokumenty', $utf8 = false)
    {
//        if ($_FILES[$htmlField]['error'] == UPLOAD_ERR_INI_SIZE ||
//            $_FILES[$htmlField]['error'] == UPLOAD_ERR_FORM_SIZE
//        ) {
//            throw new \Exception( \Ron\ronText('Err_velky_soubor') );
//        }
        self::handleUploadError($htmlField);
        self::verifyFileType($htmlField);

        $fileName = $_FILES[$htmlField]['name']; // jmeno puvodniho souboru
        $tmpFileName = $_FILES[$htmlField]['tmp_name']; // jmeno docasneho souboru (data pro upload do databaze)

        if ($utf8) {
            $popis = StringUtils::fromUTF8($popis);
            $fileName = StringUtils::fromUTF8($fileName);
        }

        $data = file_get_contents($tmpFileName);

        return self::vytvorDokument($database, $data, $fileName, $typ, $popis, $logSrcDescription);
    }


    /**
     * Vlozi do databaze novy dokument a do zadaneho $outputObejctu naplni informace o vlozenem souboru. Vraci id dokumentu.
     * @param Database $database
     * @param string $htmlField
     * @param int $typ
     * @param string $popis
     * @param JsonOutputObject $outputObject
     * @param object|string $logSrcDescription
     * @return int
     * @throws \Exception
     */
    public static function vlozDokumentJson($database, $htmlField, $typ, $popis, $outputObject, $logSrcDescription = 'Dokumenty')
    {
        self::handleUploadError($htmlField);
        self::verifyFileType($htmlField);

        $fileName = $_FILES[$htmlField]['name']; // jmeno puvodniho souboru
        $tmpFileName = $_FILES[$htmlField]['tmp_name']; // jmeno docasneho souboru (data pro upload do databaze)

        $popis = StringUtils::fromUTF8($popis);
        $fileName = StringUtils::fromUTF8($fileName);

        $data = file_get_contents($tmpFileName);

        $rec = self::vytvorDokumentRec($database, $data, $fileName, $typ, $popis, $logSrcDescription);

        $outputObject ->addData('id', $rec['Id']);
        $outputObject ->addData('typ', $typ);
        $outputObject ->addText('popis', $rec['Popis']);
        $outputObject ->addText('jmenoSouboru', $rec['JmenoSouboru']);

        $params = $database ->createParameters();
        $popisTypu = DBUtils::getSqlValue($database,
            'SELECT Popis FROM TypDokumentu WHERE Kod = ' . $params ->addParameter((int)$typ),
            $params);

        $outputObject ->addText('popisTypu', $popisTypu);

        return $rec['Id'];
    }


    /**
     * Smaze z databaze dokument se zadanym Id. Pokud je zadana podrizenaTabulka, tak nejdrive smaze zaznam z ni.
     * @param Database $database
     * @param int $id
     * @param object|string $logDescription
     * @param string $podrizenaTabulka
     * @param string $filtr
     * @param Parameters $filtrParams
     * @param string $foreignKeyName    Jmeno pole s ID dokumentu v podrizene tabulce
     * @throws DBException
     * @throws \Exception
     */
    public static function smazDokument($database, $id, $logDescription, $podrizenaTabulka = '', $filtr = '', $filtrParams = null, $foreignKeyName = 'Id')
    {
        // smazani zaznamu z podrizene tabulky
        if ($podrizenaTabulka) {
            $params = $database ->createParameters();
            $sql =
                'DELETE FROM ' . $podrizenaTabulka .
                " WHERE $foreignKeyName = " . $params ->addParameter((int)$id) .   // ? presunout tuto podminku do filtru?
                ' AND (' . $filtr . ')';
            if ($filtrParams) {
                $params ->appendParameters($filtrParams);
            }

            try {
                $database->sql($sql, $params);
            } catch (DBException $e) {
                Log::addRecordToDBLog($database,
                    $podrizenaTabulka, 'ERROR', $logDescription,
                    "Chyba pri mazani dokumentu!\n" . $e ->dbMessage . "\n\n$sql\n\nId = $id");

                 throw $e;
            }

            Log::addRecordToDBLog($database, $podrizenaTabulka, 'DELETE', $logDescription, $sql . "\n\n$foreignKeyName = $id");
        }

        $rec = new RonPortalDBRecord($database, 'Dokument', $logDescription);
        $rec ->addField('Id', 'int');
        $rec ->addField('Popis', '');
        $rec ->addField('Typ', 'int');
        $rec ->addField('Datum', 'timestamp');
        $rec ->addField('JmenoSouboru', '');
        $rec ->addField('Insert_UserName', '');
        $rec ->addField('Insert_Timestamp', 'timestamp');

        $params = $database ->createParameters();
        $rec ->deleteRecord('Id = ' . $params ->addParameter((int)$id), $params);
    }

    /**
     * @param Database $database
     * @return bool
     * @throws \Exception
     */
    public static function lzeZobrazitVyplatniListky($database)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT zobrazitNaIntrawebu FROM TypDokumentu WHERE kod = " . $params->addParameter(-1);

        try {
            $cursor = $database->getSqlResult($sql, $params);
        } catch (\Exception $exception) {
            return false;
        }

        $row = $cursor->fetch();

        $cursor->close();

        return !!$row[0];
    }

}
