<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Editace;

use Exception;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Dochazka\MesicniVysledek;
use RonPortal\Dochazka\MesicniVysledekR;
use RonPortal\Dochazka\MzdovaPolozka;
use RonPortal\Mzdy\MzdyUtils;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\Rozdeleni;
use RonPortal\RozdeleniOsob;
use RonPortal\XML\RonPortalXMLForm;
use Tracy\Debugger;

/**
 * Data pro zobrazeni/editaci mesicnich vysledku - bud hromadne (EditaceMesicnichHromadna)
 * nebo jednotlive pod zpracovanou dochazkou (EditaceMesicnich)
 */
abstract class MesicniData
{
    const SETTINGS_UZAMKNOUT_MESICNI = 'uzamknoutmesicni'; // 0 = neuzamykat, 1 (default) = pri schvaleni alespon urovni 1 do konce mesice, 2 = ..., 3 = ...
    const SETTINGS_UZAMKNOUT_MZDOVE = 'uzamknoutmzdove';   //  -||-

    /** @var \Ron\DB\Database */
    var $database;

    /** @var PolozkaEditace[]   seznam editovanych polozek (radku/sloupcu)*/
    var $polozky;

    /** @var Obdobi */
    var $obdobi;

    var $f_readonly;

    /** @var string     prefix poli v HTML formulari */
    var $prefixHTML;
    /** @var string     css styl zobrazene tabulky */
    var $cssStyl;
    /** @var int        sirka prole pro editaci (vychozi je 55) */
    var $sirkaPole;

    /** @var string[]   strednikem oddelene kody casovych slozek, ktere se pouziji pro nastaveni class jednotlivych radku v hromadne editaci */
    var $classMV;

    /** @var int */
    var $f_TypRozdeleni;
    /** @var string     rozdeleni pro mzdove polozky */
    var $f_Rozdeleni;

    /** @var bool */
    var $vcetnePodrizenych;

    /** @var int        urcuje, jestli budou zobrazeni zamestnanci, kteri nastoupili/vystoupili v prubehu obdobi
     *                  tj. zamestnanci musi byt zarazeni v rozdeleni
     *                  0 = kdykoliv, 1 = na zacatku obdobi, 2 = na konci obdobi
     *                  POZOR - v pripade 0 muze byt zamestnanec zapocitan na vice rozdelenich!
     */
    var $vyberPPVObdobi;

    /** @var PravaNaOsoby   pravo, podle ktereho se ridi zobrazeni seznamu osob. Pokud neni zadano, pouzije se [f_pravaMP, f_pravaMV] */
    var $pravaZobrazeni;

    /** @var PravaNaOsoby   prava uzivatele pro zobrazeni mzdovych polozek */
    var $f_pravaMP;
    /** @var PravaNaOsoby   prava pro editaci MP */
    var $f_pravaMPEdit;

    /** @var PravaNaOsoby  jestli muze zobrazit mesicni vysledky */
    var $f_pravaMV;
    /** @var PravaNaOsoby  jestli muze editovat mesicni vysledky */
    var $f_pravaMVEdit;
    // pokud je pouzito v XML, tak se nastavi vse na true - ridi se pouze pravem na XML

    var $style;

    /** @var string     Urcuje, jakym zpusobem se kontroluje uzavrene obdobi
     *      'ads' - obdobi nesmi byt uzavreno v dochazce
     *      'mzdy' - obdobi nesmi byt uzavreno ve mzdach
     *      'prvniotevrene' - obdobi nesmi byt uzavreno a predchozi jiz musi byt uzavreno
     */
    protected $zpusobKontrolyObdobi = 'ads';

    protected $kodAplikace = 'ADS';  // TODO: aby to fungovalo v databazi, kde jsou jen mzdy, tak bude potreba vice uprav (napr. odstranit nacitani "SchvaleniDochazky")


    function __construct($database)
    {
        $this ->database = $database;

        $this ->polozky = [];

        //$this ->f_styl = 'display:inline-table';
        $this ->sirkaPole = 55;
    }


    /**
     * Nastavi editovane obdobi. Provede kontrolu uzavreneho obdobi a podle toho pripadne nastavi "readonly". Mozne hodnoty pro
     * @param int|Obdobi $obdobi
     * @throws Exception
     */
    function nastavObdobi($obdobi)
    {
        $this ->obdobi = ($obdobi instanceof Obdobi)
            ? $obdobi
            : Obdobi::vratObdobi($this->database, $this ->kodAplikace, $obdobi);

        if (!$this ->kontrolaObdobi()) {
            $this ->f_readonly = true;
        }
    }


    /**
     * @param PolozkaEditace $polozka
     * @param bool $naZacatek   Pokud je true, vlozi se polozka na zacatek
     * @return PolozkaEditace
     */
    function pridejPolozku($polozka, $naZacatek = false)
    {
        if ($naZacatek) {
            array_unshift($this ->polozky, $polozka);
        } else {
            $this->polozky[] = $polozka;
        }

        return $polozka;
    }


    function pridejPolozkuPodleNastaveni($settingsArray, $naZacatek = false)
    {
        $polozka = new PolozkaEditace();
        $polozka ->loadSettings($settingsArray);

        $this ->pridejPolozku($polozka, $naZacatek);

        return $polozka;
    }

    /**
     * @param string $name
     * @param string $value
     * @return void   Zpracuje XML atribut hlavniho tagu
     */
    function processXMLAttribute($name, $value)
    {
        switch (strtolower($name)) {
            case 'sirkapole' :
                $this->sirkaPole = (int)$value ?: 55;
                break;

            case 'styl' :
                $this->cssStyl = $value;
                break;

            case 'prefix' :
                if ($value) {
                    $this->prefixHTML = $value;
                }
                break;

            case 'classmv' :
                $this ->classMV = explode(';', $value);
                break;

            case 'prava' :
                if ($value) {
                    $this ->pravaZobrazeni = Prava::vratPrava($value);
                }
                break;

            case 'vcetnepodrizenych' :
                $this ->vcetnePodrizenych =
                    ($value === '')
                        ? null
                        : (int)$value;
        }
    }


    /**
     * @param string $name
     * @param array  $settingsArray
     */
    function loadChildNode($name, $settingsArray)
    {
        switch ($name) {
            case 'style' :
                $this ->style = $settingsArray['#text'];
                break;

            case 'sloupeceditacemesicnich' :
            case 'radekeditacemesicnich' : // pro editaci ve zpracovane dochazce
                $this->pridejPolozkuPodleNastaveni($settingsArray['#attr']);
                break;
        }
    }


    function loadSettings($settingsArray)
    {
        // nacte atributy hlavniho tagu
        if (isset($settingsArray['#attr'])) {
            foreach ($settingsArray['#attr'] as $name => $value) {
                $this ->processXMLAttribute($name, $value);
            }
        }

        // nacteni vnorenych elementu - radku/sloupcu atd.
        foreach ($settingsArray['#children'] as $child) {
            $this ->loadChildNode($child['name'], $child['element']);
        }
    }

    /**
     * @param $element \DOMNode
     * @return void
     * @deprecated
     */
    function loadDOMElement($element)
    {
        $loader = new XmlLoader();
        $loader ->convertFromUtf8 = true;
        $settingsArray = $loader ->readDOMElement($element);
        $this ->loadSettings($settingsArray);
    }


    /**
     * @param $xmlText string
     * @return void
     * @throws Exception
     */
    function loadXML($xmlText)
    {
        $settingsArray = XmlLoader::loadXmlToArray($xmlText);
        $this ->loadSettings($settingsArray);
    }


    /**
     * @param $nastaveni string
     */
    function nactiNastaveni($nastaveni)
    {
        $this ->loadXML($nastaveni);
    }


    /** @return bool   vraci true, pokud alespon jeden z radku je mesicni vysledek */
    function obsahujeMV()
    {
        foreach ($this ->polozky as $polozka)
            if ($polozka ->zobrazit) {
                if ($polozka->typ == 'MV' || $polozka->typ == 'PMV') {
                    return true;
                }
            }

        return false;
    }


    /** @return bool   vraci true, pokud alespon jeden z radku je mzdova polozka */
    function obsahujeMP()
    {
        foreach ($this ->polozky as $polozka)
            if ($polozka ->zobrazit) {
                if ($polozka->typ == 'MP' || $polozka->typ == 'PMP') {
                    return true;
                }
            }

        return false;
    }


    /**
     * @param PolozkaEditace $polozka
     * @param OsobaProEditaci $osoba
     * @return bool  Vraci true, v pripade, ze ma uzivatel pravo na editaci zadane polozky
     * @throws Exception
     */
    protected function jePravoNaPolozku($polozka, $osoba)
    {
        if ($polozka ->pravaEditace) {
            return Prava::vratPrava($polozka ->pravaEditace) ->lzeZobrazitPP($osoba ->idPP);
        }

        if ($polozka ->typ === 'MP'  ||  $polozka ->typ === 'PMP') {
            if (Prava::existujePravo('vkladani_mzdpol_' . $polozka ->kod)) {
                return Prava::vratPrava('vkladani_mzdpol_' . $polozka->kod) ->lzeZobrazitPP($osoba ->idPP);

            }
            return $osoba ->f_PravoEditovatMP;
        }

        if ($polozka ->typ === 'MV'  ||  $polozka ->typ === 'PMV') {
            if (Prava::existujePravo('editace_mv_' . $polozka ->kod)) {
                return Prava::vratPrava('editace_mv_' . $polozka->kod) ->lzeZobrazitPP($osoba ->idPP);
            }

            return $osoba ->f_PravoEditovatMV;
        }

        return false;
    }


    /**
     * @param OsobaProEditaci $osoba
     * @param PolozkaEditace $sloupec
     * @param $mesicniVysledek
     * @return bool
     */
    protected function jeStejneRozdeleniMV($osoba, $sloupec, $mesicniVysledek)
    {
        if ($mesicniVysledek instanceof MesicniVysledekR) {
            return
                $sloupec ->rozlisitRozdeleni  &&
                $this ->jeStejneRozdeleniZaznamu($osoba, $sloupec, $mesicniVysledek ->kodTypuRozdeleni, $mesicniVysledek ->rozdeleni);
        }

        return !$sloupec ->rozlisitRozdeleni;
    }


    /**
     * @param $osoba OsobaProEditaci
     * @param MesicniVysledek $vysledek
     */
    protected function nastavMesicniVysledek($osoba, $vysledek)
    {
        foreach ($this ->polozky as $i => $polozka) {
            if ($polozka->typ === 'PMV' && $polozka->kod === $vysledek->typCasoveSlozkyMzdy) {
                if ($this ->jeStejneRozdeleniMV($osoba, $polozka, $vysledek)) {
                    $osoba->f_data[$i]->f_hodnota = null;
                    $osoba->f_data[$i]->f_text = $vysledek->poznamka;
                    $osoba->f_data[$i]->f_puvodniText = $vysledek->poznamka;
                    $osoba->f_data[$i]->f_hint = $vysledek->vratHint();

                    if (isset($polozka->defaultValue) && $polozka->defaultValue !== '' && (!isset($vysledek->poznamka) || $vysledek->poznamka === '')) {
                        $osoba->f_data[$i]->f_text = $polozka->defaultValue;
                    }
                }
            } else if ($polozka->typ === 'MV' && $polozka->kod === $vysledek->typCasoveSlozkyMzdy) { // MV
                if ($this ->jeStejneRozdeleniMV($osoba, $polozka, $vysledek)) {
                    $osoba->f_data[$i]->f_hodnota = $vysledek->vratHodnotu($polozka->jednotka);
                    $osoba->f_data[$i]->f_text = $polozka->vratTextProHodnotu($osoba->f_data[$i]->f_hodnota);
                    $osoba->f_data[$i]->f_puvodniText = $osoba->f_data[$i]->f_text;
                    $osoba->f_data[$i]->f_hint = $vysledek->vratHint();
                }

                // pokracuje dale - stejna casova slozka tam muze byt vicekrat (napr. hodiny i dny)
            }

            if ($polozka->typ === 'MV' && $polozka->kodLimitMaximum === $vysledek->typCasoveSlozkyMzdy) {
                // hodnota je limitem pro aktualni sloupec
                switch ($polozka->jednotka) {
                    case 'D' :
                        $osoba->f_data[$i]->f_maxHodnota = $vysledek->dny;
                        break;

                    case 'K' :
                        $osoba->f_data[$i]->f_maxHodnota = $vysledek->kalendarniDny;
                        break;

                    default :
                        $osoba->f_data[$i]->f_maxHodnota = $vysledek->hodiny;
                        break;
                }
            }

            if ($polozka->typ === 'MV' && $polozka->kodLimitMinimum == $vysledek->typCasoveSlozkyMzdy) {
                // hodnota je spodnim limitem pro aktualni sloupec
                switch ($polozka->jednotka) {
                    case 'D' :
                        $osoba->f_data[$i]->f_minHodnota = $vysledek->dny;
                        break;

                    case 'K' :
                        $osoba->f_data[$i]->f_minHodnota = $vysledek->kalendarniDny;
                        break;

                    default :
                        $osoba->f_data[$i]->f_minHodnota = $vysledek->hodiny;
                        break;
                }
            }
        }
    }


    /**
     * @param $osoba OsobaProEditaci
     * @throws DBException
     */
    protected function nactiMesicniVysledkyOsoby($osoba)
    {
        $typMV = [];
        $typMVR = [];
        $typMVRE = [];

        if ($this ->classMV) {
            foreach ($this ->classMV as $kodMV) {
                $typMV[] = $kodMV;
            }
        }

        foreach ($this ->polozky as $polozka) {
            if ($polozka ->typ === 'MV'  ||  $polozka ->typ === 'PMV') {
                if ($polozka ->rozlisitRozdeleni) {
                    if ($polozka ->lzeEditovat) {
                        $typMVRE[] = $polozka ->kod;
                    } else {
                        $typMVR[] = $polozka ->kod;
                    }
                } else {
                    $typMV[] = $polozka->kod;
                }

                if ($polozka ->typ === 'MV') {
                    if ($polozka->kodLimitMaximum) {
                        $typMV[] = $polozka->kodLimitMaximum;
                    }
                    if ($polozka->kodLimitMinimum) {
                        $typMV[] = $polozka->kodLimitMinimum;
                    }
                }
            }
        }

        if (!count($typMV) && !count($typMVR) && !count($typMVRE)) {
            // zadny sloupec neobsahuje mzdove polozky
            return;
        }

        $vysledky = $typMV
            ? MesicniVysledek::nactiMesicniVysledky($this ->database, $osoba ->idPP, $this ->obdobi ->zacatek, $typMV)
            : [];

        foreach($vysledky as $vysledek) {
            $this ->nastavMesicniVysledek($osoba, $vysledek);

            if ($this ->classMV  &&  (float)$vysledek ->hodiny  &&  in_array($vysledek ->typCasoveSlozkyMzdy, $this ->classMV)) {
                $osoba ->rowClass .= ($osoba ->rowClass ? ' ' : '') . 'MV' . $vysledek ->typCasoveSlozkyMzdy;
            }
        }

        $vysledkyR = $typMVR
            ? MesicniVysledekR::nactiMesicniVysledkyR($this ->database, $osoba ->idPP, $this->obdobi ->zacatek, $this ->f_TypRozdeleni, $this ->f_Rozdeleni, $typMVR)
            : [];

        foreach($vysledkyR as $vysledek) {
            $this->nastavMesicniVysledek($osoba, $vysledek);
        }

        $vysledkyRE = $typMVRE
            ? MesicniVysledekR::nactiMesicniVysledkyRE($this ->database, $osoba ->idPP, $this->obdobi ->zacatek, $this ->f_TypRozdeleni, $this ->f_Rozdeleni, $typMVRE)
            : [];

        foreach($vysledkyRE as $vysledek) {
            $this->nastavMesicniVysledek($osoba, $vysledek);
        }
    }


    /**
     * Urci, jestli se ma zaznam (mzdova polozka, mesicni vysledek nebo poznamka) se zadanym rozdelenim zobrazit ve sloupci $sloupec.
     * @param $osoba
     * @param $sloupec
     * @param ?int $kodTypuRozdeleni    Kod typu rozdeleni zaznamu
     * @param ?string $kodRozdeleni     Kod rozdeleni zaznamu
     * @return bool
     */
    protected function jeStejneRozdeleniZaznamu($osoba, $sloupec, $kodTypuRozdeleni, $kodRozdeleni)
    {
        if (!$sloupec ->rozlisitRozdeleni) {
            // rozdeleni se nerozlisuje - mel by byt jen jeden zaznam a ten se zobrazi
            return true;
        }

        if ($sloupec->rozlisitRozdeleni === 2) {
            // podle rozdeleni osoby - to, ktere je zobrazeno na radku
            return $this->f_TypRozdeleni == $kodTypuRozdeleni && $osoba->kodRozdeleni == $kodRozdeleni;
        }

        if ($sloupec->rozlisitRozdeleni === 3) {
            // podle rozdeleni osoby - nacte se rozdeleni z databaze
            $kodTypuRozdeleni = $sloupec->kodTypuRozdeleni ?: $this->f_TypRozdeleni;
            $rozdeleniOsoby = Rozdeleni::vratRozdeleniOsoby($this->database, $osoba->idPP, $kodTypuRozdeleni, $this->obdobi->zacatek, $this->obdobi->konec, false);
            if (isset($rozdeleniOsoby)) {
                return $rozdeleniOsoby == $kodRozdeleni;
            }

            return false;
        }

        // hodnota 1 = podle rozdeleni formulare
        return $this->f_TypRozdeleni == $kodTypuRozdeleni && $this->f_Rozdeleni == $kodRozdeleni;
    }

    /**
     * @param OsobaProEditaci $osoba
     * @param MzdovaPolozka $mzdovaPolozka
     * @throws DBException
     */
    protected function nastavMzdovouPolozku($osoba, $mzdovaPolozka) // $kodPolozky, $castka, $cenaZaJednotku, $pocetJednotek, $typRozdeleni, $rozdeleni)
    {
        foreach ($this ->polozky as $i => $polozka) {
            if ($polozka ->typ === 'MP'  &&  $polozka ->kod === $mzdovaPolozka ->mzdovaPolozka) {
                $jeStejneRozdeleni = $this ->jeStejneRozdeleniZaznamu($osoba, $polozka, $mzdovaPolozka ->kodTypuRozdeleni, $mzdovaPolozka ->rozdeleni);
                if ($jeStejneRozdeleni) {
                    $osoba ->f_data[$i] ->f_hodnota = $mzdovaPolozka ->castka;
                    $osoba ->f_data[$i] ->f_text = $polozka ->vratTextProHodnotu($mzdovaPolozka ->castka);
                    $osoba ->f_data[$i] ->f_puvodniText = $osoba ->f_data[$i] ->f_text;
                    $osoba ->f_data[$i] ->f_hint = $mzdovaPolozka ->vratHint();
                }
            }

            if ($polozka ->typ === 'MP'  &&  $polozka ->kodLimitMaximum == $mzdovaPolozka ->mzdovaPolozka) {
                // hodnota je limitem pro aktualni sloupec
                if ($this->jeStejneRozdeleniZaznamu($osoba, $polozka, $mzdovaPolozka ->kodTypuRozdeleni, $mzdovaPolozka ->rozdeleni)) {
                    $osoba->f_data[$i]->f_maxHodnota = $mzdovaPolozka->castka;
                }
            }

            if ($polozka ->typ === 'MP'  &&  $polozka ->kodLimitMinimum == $mzdovaPolozka ->mzdovaPolozka) {
                // hodnota je spodnim limitem pro aktualni sloupec
                if ($this->jeStejneRozdeleniZaznamu($osoba, $polozka, $mzdovaPolozka ->kodTypuRozdeleni, $mzdovaPolozka ->rozdeleni)) {
                    $osoba->f_data[$i]->f_minHodnota = $mzdovaPolozka->castka;
                }
            }
        }
    }


    /**
     * @param OsobaProEditaci $osoba
     */
    protected function nactiMzdovePolozkyOsoby($osoba)
    {
        // sestavi seznam typu mzdovych polozek
        $typMP = [];
        foreach ($this ->polozky as $polozka) {
            if ($polozka ->typ === 'MP') {
                $typMP[] = $polozka ->kod;
                if ($polozka ->kodLimitMaximum) {
                    $typMP[] = $polozka ->kodLimitMaximum;
                }
                if ($polozka ->kodLimitMinimum) {
                    $typMP[] = $polozka ->kodLimitMinimum;
                }
            }
        }

        if (!count($typMP)) {
            // zadna MP k nacteni
            return;
        }

        $polozky = MzdovaPolozka::nactiMzdovePolozky($this ->database, $osoba, $this ->obdobi ->zacatek, $typMP);
        foreach ($polozky as $mp) {
            $this ->nastavMzdovouPolozku($osoba, $mp);
        }
    }


    /**
     * @param OsobaProEditaci $osoba
     */
    function nactiRozdeleniOsoby($osoba)
    {
        foreach ($this ->polozky as $i => $polozka) {
            $typRozdeleni = null;

            if ($polozka ->typ == 'ROZD' || $polozka ->typ == 'ROZDP') {
                // R a RP netreba
                $typRozdeleni = (int)$polozka ->kodTypuRozdeleni;
            }

            if ($typRozdeleni) {
                if ($polozka ->datumRozdeleni < 0) $datumRozdeleni = $this ->vyberPPVObdobi;
                else $datumRozdeleni = $polozka ->datumRozdeleni;

                switch ($datumRozdeleni) {
                    case 1 : // zacatek obdobi
                        $datumOd = $this ->obdobi ->zacatek;
                        $datumDo = $this ->obdobi ->zacatek;
                        break;

                    case 2 : // konec obdobi
                        $datumOd = $this ->obdobi ->konec;
                        $datumDo = $this ->obdobi ->konec;
                        break;

                    default : // kdykoliv behem obdobi
                        $datumOd = $this ->obdobi ->zacatek;
                        $datumDo = $this ->obdobi ->konec;
                        break;
                }

                $params = $this ->database ->createParameters();
                $sql =
                    'SELECT Rozdeleni.Kod, Rozdeleni.Popis ' .
                    ' FROM RozdeleniOsob, Rozdeleni' .
                    ' WHERE RozdeleniOsob.KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni) .
                    '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                    '   AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ' .
                    '   AND RozdeleniOsob.RC = ' . $params ->addParameter($osoba ->rc) .
                    '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($osoba ->osobniCislo) .
                    '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->timestampToParam($osoba ->datumNastupu)) .
                    '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($this ->database ->timestampToParam($datumDo)) .
                    '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($this ->database ->timestampToParam($datumOd)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)';

                if ($polozka ->sqlFilter) {
                    $sql .= ' AND ' . $polozka ->sqlFilter;
                }

                $text = '';
                $hint = '';

                $cursor = $this ->database ->getSqlResult($sql, $params);
                while ($row = $cursor ->fetch()) {
                    if ($text) $text .= ', ';

                    switch ($polozka ->typ) {
                        case 'ROZD' :
                            $text .= $row[0];
                            if ($hint) $hint .= ', ';
                            $hint .= $row[1];
                            break;
                        case 'ROZDP' :
                            $text .= $row[1];
                            break;
                    }
                }
                $cursor ->close();

                $osoba ->f_data[$i] ->f_text = $text;
                $osoba ->f_data[$i] ->f_hint = $hint;
            }
        }
    }

    /**
     * @param $osoba OsobaProEditaci
     */
    function nactiIkonyRozdeleni($osoba)
    {
        foreach ($this ->polozky as $i => $polozka) {
            if ($polozka->typ == 'ROZDIMG' && $polozka->kodTypuRozdeleni) {
                $ikony = RozdeleniOsob::nactiIkonyRozdeleni($this ->database, $osoba->idPP, $polozka ->kodTypuRozdeleni, $this ->obdobi ->zacatek, $this ->obdobi ->konec);
                $osoba ->f_data[$i] ->f_text = RozdeleniOsob::getIkonyHtml($ikony);
            }
        }
    }

    /**
     * @param $osoba OsobaProEditaci
     */
    function nactiPoznamkyMP($osoba)
    {
        foreach ($this ->polozky as $i => $polozka) {
            if ($polozka ->typ == 'PMP' && $polozka ->kod) {
                $params = $this ->database ->createParameters();
                $sql =
                    'SELECT PoznamkaMzdovePolozky.MzdovaPolozka, ' . $this ->database ->castWideMemo('PoznamkaMzdovePolozky.Poznamka') . ',' .
                    '  MzdovaPolozkaVeMzde.KodTypuRozdeleni, MzdovaPolozkaVeMzde.Rozdeleni' .
                    ' FROM PoznamkaMzdovePolozky, MzdovaPolozkaVeMzde ' .
                    ' WHERE PoznamkaMzdovePolozky.RC = ' . $params->addParameter($osoba ->rc) .
                    '   AND PoznamkaMzdovePolozky.OsobniCislo = ' . $params->addParameter($osoba ->osobniCislo) .
                    '   AND PoznamkaMzdovePolozky.DatumNastupu = ' . $params ->addParameter($this ->database ->timestampToParam($osoba ->datumNastupu)) .
                    '   AND PoznamkaMzdovePolozky.Obdobi = ' . $params ->addParameter($this ->database ->timestampToParam($this ->obdobi ->zacatek)) .
                    '   AND PoznamkaMzdovePolozky.MzdovaPolozka = ' . $params ->addParameter($polozka ->kod) .
                    '   AND PoznamkaMzdovePolozky.DruhZdrojeMzdovePolozky = 4' . // jen MP z dochazky
                    // pripoji zaznam mzdove polozky kvuli nacteni rozdeleni
                    '   AND PoznamkaMzdovePolozky.RC = MzdovaPolozkaVeMzde.RC' .
                    '   AND PoznamkaMzdovePolozky.OsobniCislo = MzdovaPolozkaVeMzde.OsobniCislo' .
                    '   AND PoznamkaMzdovePolozky.DatumNastupu = MzdovaPolozkaVeMzde.DatumNastupu' .
                    '   AND PoznamkaMzdovePolozky.Obdobi = MzdovaPolozkaVeMzde.Obdobi' .
                    '   AND PoznamkaMzdovePolozky.MzdovaPolozka = MzdovaPolozkaVeMzde.MzdovaPolozka' .
                    '   AND PoznamkaMzdovePolozky.DruhZdrojeMzdovePolozky = MzdovaPolozkaVeMzde.DruhZdrojeMzdovePolozky' .
                    '   AND PoznamkaMzdovePolozky.SerioveCislo = MzdovaPolozkaVeMzde.SerioveCislo';

                $cursor = $this ->database ->getSqlResult($sql, $params);

                $found = false;
                while (($row = $cursor ->fetch())  &&  !$found) {
                    //echo " {$row[2]}-{$row[3]}-{$row[1]} ";
                    if ($this ->jeStejneRozdeleniZaznamu($osoba, $polozka, $row[2], $row[3])) {
                        $osoba->f_data[$i]->f_text = $row[1];
                        $osoba->f_data[$i]->f_puvodniText = $osoba->f_data[$i]->f_text;
                        $found = true;
                    }
                }
                $cursor ->close();
            }
        }
    }


    /**
     * @param OsobaProEditaci $osoba
     * @param int $i
     * @param PolozkaEditace $polozka
     */
    function nactiSQLPolozku($osoba, $i, $polozka)
    {
        $result = $polozka ->loadSQLProOsobu($this ->database, $osoba, $this ->obdobi, $this ->f_TypRozdeleni, $this ->f_Rozdeleni);

        if ($result) {
            $osoba->f_data[$i]->f_hodnota = $result['hodnota'];
            $osoba->f_data[$i]->f_puvodniText = $osoba->f_data[$i]->f_text = $result['text'];
            $osoba->f_data[$i]->f_hint = $result['title'];
        }
    }


    /**
     * @param OsobaProEditaci $osoba
     */
    function nactiDataOsoby($osoba)
    {
        $osoba ->f_data = [];
        foreach ($this ->polozky as $i =>$polozka) {
            $osoba ->f_data[$i] = new EditaceMesicnichField();

            if ($polozka ->defaultValue) {
                $osoba ->f_data[$i] ->f_text = $polozka ->defaultValue;
            }
        }

        // rozdeleni
        $this ->nactiRozdeleniOsoby($osoba);
        $this ->nactiIkonyRozdeleni($osoba);

        // mzdove polozky
        if ($osoba ->f_PravoZobrazitMP || $osoba ->f_PravoEditovatMP) {  // TODO: Nebude fungovat, kdyz bude mit prava jen na konkretni mzdovou polozku!
            foreach ($this ->polozky as $i => $polozka) {
                if (isset($polozka ->maximum)) {
                    $osoba ->f_data[$i] ->f_maxHodnota = $polozka ->maximum;
                }

                if (isset($polozka ->minimum)) {
                    $osoba ->f_data[$i] ->f_minHodnota = $polozka ->minimum;
                }
            }

            $this ->nactiMzdovePolozkyOsoby($osoba);
            $this ->nactiPoznamkyMP($osoba);
        }

        // mesicni vysledky
        if ($osoba ->f_PravoZobrazitMV || $osoba ->f_PravoEditovatMV) {  // TODO: Nebude fungovat, kdyz bude mit prava jen na konkretni casovou slozku!
            $this ->nactiMesicniVysledkyOsoby($osoba);
        }

        // ostatni pole jsou uz nactene v osobe (datum nastupu, ukonceni atd.)
        foreach ($this ->polozky as $i =>$polozka) {
            if ($polozka ->typ === 'SQL') {
                $this ->nactiSQLPolozku($osoba, $i, $polozka);
            }
        }

    }


    /**
     * @param OsobaProEditaci $osoba
     * @throws Exception
     */
    function nastavPravaNaOsobu($osoba)
    {
        // nastaveni prav
        switch ($this ->vyberPPVObdobi) {
            case 1 : // zacatek obdobi
                $datumOd = $this ->obdobi ->zacatek;
                $datumDo = $this ->obdobi ->zacatek;
                break;

            case 2 : // konec obdobi
                $datumOd = $this ->obdobi ->konec;
                $datumDo = $this ->obdobi ->konec;
                break;

            default : // kdykoliv behem obdobi
                $datumOd = $this ->obdobi ->zacatek;
                $datumDo = $this ->obdobi ->konec;
                break;
        }

        $osoba ->f_PravoEditovatMP = $this ->f_pravaMPEdit ->lzeZobrazitPP($osoba ->idPP, $datumOd, $datumDo);
        $osoba ->f_PravoEditovatMV = $this ->f_pravaMVEdit ->lzeZobrazitPP($osoba ->idPP, $datumOd, $datumDo);

        $osoba ->f_PravoZobrazitMV = $osoba ->f_PravoEditovatMV || $this ->f_pravaMV ->lzeZobrazitPP($osoba ->idPP, $datumOd, $datumDo);
        $osoba ->f_PravoZobrazitMP = $osoba ->f_PravoEditovatMP || $this ->f_pravaMP ->lzeZobrazitPP($osoba ->idPP, $datumOd, $datumDo);
    }


    /**
     * @param array $row
     * @return OsobaProEditaci
     */
    function vytvorOsobuPodleRadku($row)
    {
        $osoba = new OsobaProEditaci();
        $osoba ->loadRow($this ->database, $row);

        $this ->nastavPravaNaOsobu($osoba);

        return $osoba;
    }



    /**
     * @param OsobaProEditaci $osoba
     * @param int $index
     * @return string
     */
    function vratHTMLText($osoba, $index, $html = true)
    {
        if ($this->polozky[$index]->prava) {
            if (!$this->polozky[$index]->prava ->lzeZobrazitPP($osoba ->idPP)) {
                return '';
            }
        }

        switch ($this->polozky[$index]->typ) {
            case 'SCHVAL1' :
                return $osoba->schvaleni->schvaleni1
                    ?
                        ($html ? '<SPAN class="textschvaleni1">' : '') .
                        date('j.n.Y', $osoba->schvaleni->schvaleni1) .
                        ($html ? '</SPAN>' : '')
                    : '';

            case 'SCHVAL2' :
                return $osoba->schvaleni->schvaleni2
                    ?
                        ($html ? '<SPAN class="textschvaleni2">' : '') .
                        date('j.n.Y', $osoba->schvaleni->schvaleni2) .
                        ($html ? '</SPAN>' : '')
                    : '';

            case 'SCHVAL3' :
                return $osoba->schvaleni->schvaleni3
                    ?
                        ($html ? '<SPAN class="textschvaleni3">' : '') .
                        date('j.n.Y', $osoba->schvaleni->schvaleni3) .
                        ($html ? '</SPAN>' : '')
                    : '';

            case 'SCHVAL' : // zobrazi nejvyssi nastavenou uroven schvaleni
                return
                    $osoba->schvaleni->schvaleni3 ? '<SPAN class="textschvaleni3">' . date('j.n.Y', $osoba->schvaleni->schvaleni3) . '</SPAN>' :
                        ($osoba->schvaleni->schvaleni2 ? '<SPAN class="textschvaleni2">' . date('j.n.Y', $osoba->schvaleni->schvaleni2) . '</SPAN>' :
                            ($osoba->schvaleni->schvaleni1 ? '<SPAN class="textschvaleni1">' . date('j.n.Y', $osoba->schvaleni->schvaleni1) . '</SPAN>' : '')
                        );

            case 'EDIT' : // odkaz na editaci dochazky
                if (!$html) {
                    return '';
                }

                $targetstr = $this->polozky[$index]->target;
                $targetstr = $targetstr ? 'target="' . $targetstr . '"' : '';

                return
                    '<a ' . $targetstr . ' href="?menuid=zpracovanadochazka&amp;osoba=' . $osoba->idPP .
                    '&amp;obdobi=' . DateUtils::dateToUrlString($this->obdobi->zacatek) . '">' . \Ron\ronText('dochazka') . '</a>';

            case 'JMENOPRIJMENI' :
            case 'JMENO' :
            case 'PRIJMENI' :
            case 'OSOBNICISLO' :
            case 'UZIVATELSKA' :
            case 'UZIVATELSKAPP' :
            case 'SCHVALENI' :
            case 'FOTO' :
            case 'NASTUP' :
            case 'UKONCENI' :
            case 'TYPPP' :
            case 'SCHVALMZDY' :
            case 'IDPP' :
                return $this->polozky[$index]->getText($osoba, $this ->obdobi, null, $html);

            case 'ROZDIMG' :
                return $osoba->f_data[$index]->f_text; // obsahuje primo HTML

            default :
                return $html
                    ? StringUtils::htmlTextConvert($osoba->f_data[$index]->f_text)
                    : $osoba->f_data[$index]->f_text;
        }
    }



    /**
     * Vraci true, pokud jeste obdobi neni uzavreno - zpusob kontroly urcuje vlastnost $kontrolaObdobi
     * @return bool
     */
    function kontrolaObdobi()
    {
        // kontrola uzavreni v dochazce
        switch ($this ->zpusobKontrolyObdobi) {
            case 'mzdy' :
                // obdobi v dochazce se nekontroluje
                break;

            case 'prvniotevrene' :
                if ($this ->obdobi ->uzavreno) {
                    return false;
                } else {
                    // zakaze editaci, pokud predchozi obdobi jeste neni uzavreno
                    $obdobi2 = Obdobi::vratObdobi($this->database, $this ->kodAplikace, DateUtils::incMonth($this ->obdobi->zacatek, -1));
                    if (!$obdobi2->uzavreno) {
                        MessageManager::addWarning(TextManager::_('MV_nelze_editovat_Predchozi_obdobi_neuzavreno'));
                        return false;
                    }
                }
                break;

            case 'nekontrolovat' :
                // kontrola uzavreneho obdobi se vubec neprovadi
                return true;

            case 'jenads' :
                // mzdy se vubec nekontroluji
                return !$this ->obdobi ->uzavreno;

            case 'ads' :
            default :
                if ($this ->obdobi ->uzavreno) {
                    return false;
                }
        }

        // kontrola uzavreni ve mzdach - pokud vubec neexistuje, tak je to OK
        if (Obdobi::jeObdobiUzavreno($this ->database, 'MZDY', $this ->obdobi ->zacatek, $this ->obdobi ->konec)) {
            MessageManager::addWarning(TextManager::_('MV_nelze_editovat_Obdobi_uzavreno_ve_mzdach'));
            return false;
        }

        return true; // OK, muze se editovat
    }



    /**
     * @param $polozka PolozkaEditace
     * @param $osoba OsobaProEditaci
     * @return bool
     */
    function uzamknoutPolozku($polozka, $osoba)
    {
        if (isset($polozka ->uzamknoutSchvalenim)) {
            return $osoba ->uzamknout($this ->obdobi ->konec, $polozka ->uzamknoutSchvalenim);
        }

        if ($polozka ->typ == 'MP'  ||  $polozka ->typ == 'PMP') {
            return $osoba ->uzamknout($this ->obdobi ->konec, RonPortalSettings::getIntValue(self::SETTINGS_UZAMKNOUT_MZDOVE, 1));
        }

        if ($polozka ->typ == 'MV'  ||  $polozka ->typ == 'PMV') {
            return $osoba ->uzamknout($this ->obdobi ->konec, RonPortalSettings::getIntValue(self::SETTINGS_UZAMKNOUT_MESICNI, 1));
        }

        return false;
    }


    /**
     * @param bool $povolitVse
     * Nastavi prava na editaci a prohlizeni mzdovych polozek
     */
    function nastavPrava($povolitVse)
    {
        if ($povolitVse) {
            $this ->f_pravaMP = new PravaNaOsoby(3, '', '', 0);
            $this ->f_pravaMPEdit = new PravaNaOsoby(3, '', '', 0);
            $this ->f_pravaMV = new PravaNaOsoby(3, '', '', 0);
            $this ->f_pravaMVEdit = new PravaNaOsoby(3, '', '', 0);
        } else {
            $this ->f_pravaMP = Prava::vratPrava('zobrazit_mzdpol');
            $this ->f_pravaMPEdit = Prava::vratPrava('vkladani_mzdpol');
            $this ->f_pravaMV = Prava::vratPrava('prohlizeni');
            $this ->f_pravaMVEdit = Prava::vratPrava('editace_mv');
        }
    }


    /**
     * @param PracovniPomer $pp
     * @return bool   Zkontroluje jestli je mzda jeste editovatelna a v pripade potreby doplni zaznam mzdy
     */
    function zkontrolujMzdu($pp)
    {
        if (Obdobi::jeObdobiUzavreno($this ->database, 'MZDY', $this ->obdobi ->zacatek, $this ->obdobi ->zacatek)) {
            return false;
        }

        return MzdyUtils::zkontrolujZaznamMzdy($this ->database, $pp, $this ->obdobi ->zacatek, $this);
    }


    /**
     * Vlozi nebo aktualizuje zaznam v tabulce MzdovaPolozkaVeMzde pro zadanou osobu a sloupec. Vraci seriove cislo zapsane polozky
     * @param OsobaProEditaci $pp
     * @param PolozkaEditace $polozka
     * @param ?float $castka
     * @param float $cenaZaJednotku
     * @param float $pocetJednotek
     * @return int|null
     * @throws Exception
     */
    function zapisMzdovouPolozkuDoDB($pp, $polozka, $castka, $cenaZaJednotku = 0.0, $pocetJednotek = 0.0)
    {
        if (!$this ->zkontrolujMzdu($pp)) {
            throw new Exception('Chyba pri zapisu mzdovych polozek - pravdepodobne je jiz uzamcena mzda!');
        }

        if (!$cenaZaJednotku && !$pocetJednotek) {
            $cenaZaJednotku = (float)$castka;
            if ($cenaZaJednotku) {
                $pocetJednotek = 1;
            }
        }

        $kodTypuRozdeleni = null;
        $rozdeleni = null;

        if ($polozka ->rozlisitRozdeleni == 3) {
            // nemelo by nastat - pokud je rozlisitRozdeleni=3, tak je editace zakazana
            return null;
            //return true;
        } else if ($polozka ->rozlisitRozdeleni == 2) {
            // rozlisuje se rozdeleni - podle rozdeleni zamestnance
            $kodTypuRozdeleni = $this ->f_TypRozdeleni;
            $rozdeleni = $pp ->kodRozdeleni;
            if (!isset($rozdeleni)) {
                throw new Exception('Chyba - ' . $pp ->jmenoPrijmeni . ', ' . ' os. c. ' . $pp ->osobniCislo . ' nema prirazeno rozdeleni!');
            }
        } else if ($polozka ->rozlisitRozdeleni && $this ->f_TypRozdeleni && $this ->f_Rozdeleni) {
            // rozlisuje se rozdeleni - podle uzivatelem vybraneho rozdeleni ve formulari
            $kodTypuRozdeleni = $this->f_TypRozdeleni;
            $rozdeleni = $this->f_Rozdeleni;
        }

        $rec = new MzdovaPolozkaVeMzdeRecord($this ->database,
            $pp, $this ->obdobi ->zacatek, $polozka ->kod,
            $this,
            $kodTypuRozdeleni, $rozdeleni);

        $rec ->zapisHodnotu(
            (float)$castka, (float)$pocetJednotek, (float)$cenaZaJednotku,
            $kodTypuRozdeleni, $rozdeleni);

        return $rec ->fSerial ->value;
    }


    /**
     * Vraci seriove cislo mzdove polozky s kodem $kodMP ve mzde osoby nebo null
     * @param OsobaProEditaci $osoba
     * @param PolozkaEditace $polozka
     * @param string $tabulka
     * @return bool|int|null
     * @throws DBException
     * @throws Exception
     */
    function vratSerioveCisloMP($osoba, $polozka, $tabulka = 'MzdovaPolozkaVeMzde')
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT SerioveCislo FROM ' . $tabulka .
            ' WHERE RC = ' . $params->addParameter($osoba ->rc) .
            '   AND OsobniCislo = ' . $params->addParameter($osoba ->osobniCislo) .
            '   AND DatumNastupu = ' . $params ->addParameter($this ->database ->timestampToParam($osoba ->datumNastupu)) .
            '   AND Obdobi = ' . $params ->addParameter($this ->database ->timestampToParam($this ->obdobi ->zacatek)) .
            '   AND DruhZdrojeMzdovePolozky = 4' . // jen MP z dochazky
            '   AND MzdovaPolozka = ' . $params->addParameter($polozka ->kod);

        switch ($polozka ->rozlisitRozdeleni) {
            case 1:
                // rozliseni podle rozdeleni formulare
                if ($this ->f_TypRozdeleni && $this ->f_Rozdeleni) {
                    // jen mzdove polozky daneho rozdeleni
                    $sql .=
                        ' AND KodTypuRozdeleni = ' . $params ->addParameter((int)$this ->f_TypRozdeleni) .
                        ' AND Rozdeleni = ' . $params->addParameter($this ->f_Rozdeleni);
                }
                break;

            case 2:
                // rozliseni podle rozdeleni osoby
                if (!isset($osoba ->kodRozdeleni)) {
                    throw new Exception("Chyba pri editaci mzdovych polozek: osoba {$osoba ->jmenoPrijmeni} nema prirazeno rozdeleni!");
                }

                $sql .=
                    ' AND KodTypuRozdeleni = ' . $params ->addParameter((int)$this ->f_TypRozdeleni) .
                    ' AND Rozdeleni = ' . $params->addParameter($osoba ->kodRozdeleni);
                break;
        }


        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : null;
    }


    /**
     * @param OsobaProEditaci $osoba
     * @param PolozkaEditace $polozka
     * @param $poznamka
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    public function zapisPoznamkuMzdovePolozkyDoDB($osoba, $polozka, $poznamka)
    {
        // nejdrive zjisti seriove cislo MP
        $serial = $this ->vratSerioveCisloMP($osoba, $polozka);
        if (!$serial) {
            if (!$poznamka) {
                // poznamka je prazdna a MP neexistuje (neexistuje ani poznamka) => neni co zapsat
                return true;
            }

            $serial = $this ->zapisMzdovouPolozkuDoDB($osoba, $polozka, 0, 0.0, 0.0);
            // $serial = $this ->vratSerioveCisloMP($osoba, $polozka);
        }

        // POZOR, priznak "rozlisitRozdeleni" nefunguje spravne!
        $rec = new RonPortalDBRecord($this ->database, 'PoznamkaMzdovePolozky', $this);
        $rec ->addStringField('RC', true, 15, $osoba ->rc);
        $rec ->addStringField('OsobniCislo', true, 15, $osoba ->osobniCislo);
        $rec ->addDateField('DatumNastupu', true, $osoba ->datumNastupu);
        $rec ->addField('', '', true, 0, '   ' . $osoba ->jmenoPrijmeni);
        $rec ->addDateField('Obdobi', true, $this ->obdobi ->zacatek);
        $rec ->addIntField('DruhZdrojeMzdovePolozky', true, 4);
        $rec ->addStringField('MzdovaPolozka', true, 10, $polozka ->kod);
        $rec ->addIntField('SerioveCislo', true, $serial);
        $rec ->addMemoField('Poznamka', false, $poznamka);

        $rec ->addStringField('Insert_UserName', true, 25, RonPortalUtils::getLogUserName(true));
        $rec ->addTimestampField('Insert_Timestamp', true, time());

        $f = $rec ->addStringField('Update_UserName', false, 25, RonPortalUtils::getLogUserName(true));
        $f ->updateLog = true;
        $f = $rec ->addTimestampField('Update_Timestamp', false, time());
        $f ->updateLog = true;

        $params = $this ->database ->createParameters();
        $whereSQL =
            'RC = ' . $params->addParameter($osoba ->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($osoba ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($this ->database ->timestampToParam($osoba ->datumNastupu)) .
            ' AND Obdobi = ' . $params ->addParameter($this ->database ->timestampToParam($this ->obdobi ->zacatek)) .
            ' AND DruhZdrojeMzdovePolozky = 4' . // jen MP z dochazky
            ' AND MzdovaPolozka = ' . $params->addParameter($polozka ->kod) .
            ' AND SerioveCislo = ' . $params ->addParameter((int)$serial);
        $rec ->setWhere($whereSQL, $params);

        $rec ->updateOrInsert();

        return true;
    }


    /**
     * @param OsobaProEditaci $osoba
     * @param string $eventName
     * @param bool $jeZmena
     * @return bool
     */
    function executeXmlEvent($osoba, $eventName, $jeZmena)
    {
        try {
            $variables = [
                'idPP' => $osoba->idPP,
                'RC' => $osoba->rc,
                'osobniCislo' => $osoba->osobniCislo,
                'datumNastupu' => $osoba->datumNastupu,
                'jmeno' => $osoba->jmeno,
                'prijmeni' => $osoba->prijmeni,
                'jmenoPrijmeni' => $osoba->jmenoPrijmeni,
                'obdobi' => $this->obdobi->zacatek,
                'jeZmena' => $jeZmena ? 1 : 0
            ];

            return RonPortalXMLForm::xmlEvents($this ->database, $eventName, $variables, '', false, false);
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            return false;
        }
    }
}
