<?php

namespace RonPortal\Jidelna;

use Exception;
use Ron\DateUtils;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use Ron\DB\Database;
use Ron\TextManager;
use RonPortal\Log;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;

class Burza
{
    const BURZA_KONEC_PRIME_PRED_VYDEJEM = 'primaburzapredvydejem'; // puvodne bylo default=30

    /**
     * Vraci true, pokud lze pouzit primou burzu (podle nastaveni a casu vydeje zadane polozky)
     * @param Database $database
     * @param PolozkaJidelnicku $polozkaJidelnicku
     * @return bool
     */
    static function lzePouzitPrimouBurzu($database, $polozkaJidelnicku)
    {
        $vydej_array = JidelnaUtils::casVydejePolozkyJidelnicku($database, $polozkaJidelnicku ->f_id);

        //return time() < DateUtils::incMinute($vydej_array['vydejOd'], -RonPortalSettings::getIntValue(self::BURZA_KONEC_PRIME_PRED_VYDEJEM));
        return time() < DateUtils::incMinute($vydej_array['vydejOd'], -$polozkaJidelnicku ->konecPrimeBurzy);
    }


    /**
     * Vlozi zadanou polozku jidelnicku do burzy
     * - tj najde platnou objednavku a nastavi u ni DatumCasVlozeniNaBurzu a priznak StavVlozeniNaBurzu
     * @param Database $database
     * @param PracovniPomer $ppomer
     * @param PolozkaJidelnicku $polozkaJidelnicku
     * @throws Exception
     */
    public static function vlozPolozkuDoBurzy($database, $ppomer, $polozkaJidelnicku, $idObjednavky = null)
    {
        // Nacte vsechny platne objednavky osoby, ktere nejsou v burze
        $params = $database ->createParameters();
        $sql =
            'SELECT Id, StavPrevzetiZBurzy ' .
            ' FROM ObjednaneJidlo ' .
            ' WHERE IdPolozkyJidelnicku = ' . $params ->addParameter($polozkaJidelnicku ->f_id) .
            '  AND RC = ' . $params->addParameter($ppomer ->rc) .
            '  AND OsobniCislo = ' . $params->addParameter($ppomer ->osobniCislo) .
            '  AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($ppomer ->datumNastupu)) .
            ($idObjednavky ? ' AND Id = ' . $params ->addParameter($idObjednavky) : '') .   // konkretni objednavka
            '  AND DatumCasZruseni IS NULL ' .
            '  AND DatumCasVlozeniNaBurzu IS NULL ' .
            '  AND DatumCasVydani IS NULL ' .
            '  AND IDPuvodnihoOdberu IS NULL ' .
            ' ORDER BY PoradiVydani DESC';

        $database ->beginTransaction();

        try {
            $cursor = $database ->getSqlResult($sql, $params);

            $OK = false;
            $infoAktualizace = false; // priznak, jestli se ma zobrazit info o aktualizaci v terminalu

            $row = $cursor ->fetch();

            while (!$OK  &&  $row) { // najde objednavku, kterou je mozne vlozit do burzy
                switch ($row[1]) {
                    case null :
                    case 0 :
                    case 3 :
                        $OK = true; // nasel objednavku, kterou lze vlozit do burzy
                        break;

                    case 1 :
                    /** @noinspection PhpMissingBreakStatementInspection */
                    case 2 :
                        $infoAktualizace = true; // jeste nebyla aktualizovana v terminalu
                        // ne break!

                    default :
                        $row = $cursor ->fetch();
                        break;
                }
            }

            $cursor ->close();

            if ($row) {
                $lzePouzitPrimou = self::lzePouzitPrimouBurzu($database, $polozkaJidelnicku);

                // Provede UPDATE -> vlozi na burzu
                // Je nutne otestovat, jestli jsou vsechny hodnoty shodne - tim se zajisti,
                // ze paralelne neprobehla nejaka operace s touto objednavkou
                $paramsUpdate = $database ->createParameters();
                $sql =
                    'UPDATE ObjednaneJidlo  ' .
                    ' SET DatumCasVlozeniNaBurzu = ' . $paramsUpdate ->addParameter($database ->timestampToParam(time())) . ', ' .
                    '     StavVlozeniNaBurzu = ' . $paramsUpdate ->addParameter($lzePouzitPrimou ? '3' : '1') . ', ' .
                    '     StavPrevzetiZBurzy = NULL ' .
                    ' WHERE Id = ' . $paramsUpdate ->addParameter($row[0]) .
                    '  AND IdPolozkyJidelnicku = ' . $paramsUpdate ->addParameter($polozkaJidelnicku ->f_id) .
                    '  AND RC = ' . $paramsUpdate->addParameter($ppomer ->rc) .
                    '  AND OsobniCislo = ' . $paramsUpdate->addParameter($ppomer ->osobniCislo) .
                    '  AND DatumNastupu = ' . $paramsUpdate->addDateParameter($ppomer ->datumNastupu) .
                    '  AND DatumCasZruseni IS NULL ' .
                    '  AND DatumCasVlozeniNaBurzu IS NULL ' .
                    '  AND DatumCasVydani IS NULL ' .
                    '  AND IDPuvodnihoOdberu IS NULL ';

                $database ->sql($sql, $paramsUpdate);

                $database ->commitTransaction();
            } else {
                // Neni zadna objednavka, ktera jde vlozit na burzu
                throw new Exception( TextManager::_($infoAktualizace ? 'Info_aktualizece_burzy' : 'Neni_co_vlozit_na_burzu') );
            }

        } catch (Exception $e) {
            $database ->rollbackTransaction();
            throw $e;
        }

        Log::addRecordToDBLog($database, 'ObjednaneJidlo', 'UPDATE', 'Burza',
            "Vlozeni do burzy:\n" .
            $ppomer->jmenoPrijmeni . "\n" .
            date('j.n.Y', $polozkaJidelnicku ->f_den) . ' ' . $polozkaJidelnicku ->f_typdennihojidla . ' ' . $polozkaJidelnicku ->f_cislonabidky . ' - ' . $polozkaJidelnicku ->f_popisnabidky . "\n" .
            "Prima burza: " . ($lzePouzitPrimou ? 'ano' : 'ne') .
            "\n\n" .
            $sql
        );

    }


    /**
     * @param Database $database
     * @param PracovniPomer $ppomer
     * @param PolozkaJidelnicku $polozkaJidelnicku
     * @param int $idObjednavky
     * @throws Exception
     * @throws \Ron\DB\DBException
     */
    static function vlozKopiiZBurzyAZrusPuvodni($database, $ppomer, $polozkaJidelnicku, $idObjednavky)
    {
        $idObjednavky = (int)$idObjednavky;

        // Zrusi puvodni objednavku
        $rec = new RonPortalDBRecord($database, 'ObjednaneJidlo', 'Burza');
        $rec ->addLogText('Zruseni objednavky v burze');
        $rec ->addIntField('Id', true);
        $rec ->addIntField('IdPolozkyJidelnicku', true);
        $rec ->addStringField('RC', true);
        $rec ->addStringField('OsobniCislo', true);
        $rec ->addTimestampField('DatumNastupu', true);
        $rec ->addIntField('StavVlozeniNaBurzu', true);
        $rec ->addTimestampField('DatumCasVlozeniNaBurzu', true);
        $rec ->addTimestampField('DatumCasZruseni', false, time()); // zrusi objednavku

        $params = $database ->createParameters();
        $rec ->setWhere('Id = ' . $params ->addParameter($idObjednavky), $params);
        if (!$rec ->loadRecord()) {
            throw new Exception('Chyba pri vyzvednuti z burzy - puvodni objednavka nebyla nalezena!');
        }
        $rec ->updateRecord();

        // Vlozi novou objednavku
        $newId = SQLUtils::newSerialNumber($database, 'ObjednaneJidlo', 'ID');
        $newPoradi = $polozkaJidelnicku->novePoradiVydani($database,
            $ppomer ->rc, $ppomer ->osobniCislo, $ppomer ->datumNastupu);

        $stav = self::lzePouzitPrimouBurzu($database, $polozkaJidelnicku) ? '3' : '1';

        $rec = new RonPortalDBRecord($database, 'ObjednaneJidlo', 'Burza');
        $rec ->addLogText('Prevzeti z burzy (vlozeni nove objednavky)');
        $rec ->addField('Id', 'int', true, 0, $newId);
        $rec ->addField('IdPolozkyJidelnicku', 'int', true, 0, $polozkaJidelnicku ->f_id);
        $rec ->addField('', '', true, 0, '   ' . date('j.n.Y', $polozkaJidelnicku ->f_den) . ' ' . $polozkaJidelnicku ->f_typdennihojidla . ' ' . $polozkaJidelnicku ->f_cislonabidky);
        $rec ->addField('RC', '', true, 15, $ppomer ->rc);
        $rec ->addField('OsobniCislo', '', true, 15, $ppomer ->osobniCislo);
        $rec ->addField('DatumNastupu', 'timestamp', true, 0, $ppomer ->datumNastupu);
        $rec ->addField('', '', true, 0, '   ' . $ppomer->jmenoPrijmeni);
        $rec ->addField('PoradiVydani', 'int', true, 0, $newPoradi);
        $rec ->addField('DatumCasObjednani', 'timestamp', true, 0, time());
        $rec ->addField('Uzivatel', '', true, 25, RonPortalUtils::getLogUserName());
        $rec ->addField('IDobjednavkyZBurzy', 'int', true, 0, $idObjednavky);
        $rec ->addField('StavPrevzetiZBurzy', 'int', true, 0, $stav);

        $rec ->insertRecord();
    }


    /**
     * @param Database $database
     * @param int $idObjednavky
     * @throws Exception
     */
    private static function zrusVlozeniNaBurzu($database, $idObjednavky)
    {
        $idObjednavky = (int)$idObjednavky;

        $rec = new RonPortalDBRecord($database, 'ObjednaneJidlo', 'Burza');
        $rec ->addLogText('Vyzvednuti vlastni objednavky zpet z burzy');
        $rec ->addField('Id', 'int', true);
        $rec ->addField('IdPolozkyJidelnicku', 'int', true);
        $rec ->addField('RC', '', true);
        $rec ->addField('OsobniCislo', '', true);
        $rec ->addField('DatumNastupu', 'timestamp', true);
        $rec ->addField('StavVlozeniNaBurzu', 'int', false, 0, 0);  // opraveno 15.4.2016 - nastavi na 0
        $rec ->addField('DatumCasVlozeniNaBurzu', 'timestamp', false, 0, null); // nastavi na null
        $rec ->addField('DatumCasZruseni', 'timestamp', true);

        $params = $database ->createParameters();
        $rec ->setWhere('Id = ' . $params ->addParameter($idObjednavky), $params);

        if (!$rec ->loadRecord()) {
            throw new Exception('Chyba pri vyzvednuti vlastni objednavky z burzy - objednavka nebyla nalezena!');
        }

        $rec ->updateRecord();
    }


    /**
     * Vyzvedne zadanou polozku jidelnicku z burzy
     * Vraci true, pokud nasel objednavku, ktera se da vyzvednout z burzy
     * (pozor - to neznamena, ze vyzvednuti probehlo v poradku - pokud se prave
     *  do terminalu posila vlastni objednavka, tak ji nevyzvedne - musi se pockat az na ukonceni odesilani)
     * @param Database $database
     * @param PracovniPomer $ppomer
     * @param PolozkaJidelnicku $polozkaJidelnicku
     * @param bool $vlastni Urcuje, jestli ma vyzvednout vlastni objednavku
     * @return bool
     * @throws Exception
     */
    private static function vyzvednutiPolozkyZBurzy($database, $ppomer, $polozkaJidelnicku, $vlastni)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Id, StavVlozeniNaBurzu  FROM ObjednaneJidlo  ' .
            'WHERE IdPolozkyJidelnicku = ' . $params ->addParameter($polozkaJidelnicku ->f_id) .
            '  AND DatumCasZruseni IS NULL ' .
            '  AND DatumCasVydani IS NULL ' .
            '  AND IDPuvodnihoOdberu IS NULL ' .
            '  AND DatumCasVlozeniNaBurzu IS NOT NULL ';

        if ($vlastni) {
            // neni treba kontrolovat stav uctu stravnika a pocet objednavek
            $sql .=
                '  AND RC = ' . $params->addParameter($ppomer ->rc) .
                '  AND OsobniCislo = ' . $params->addParameter($ppomer->osobniCislo) .
                '  AND DatumNastupu = ' . $params->addParameter($database ->dateToParam($ppomer ->datumNastupu)) .
                // Q.SQL.Add('AND StavVlozeniNaBurzu = 3'); - nesmi byt ! ! !
                // nacita vsechny platne objednavky osoby bez ohledu na StavVlozeniNaBurzu
                // nesmi prevzit jinou objednavku, pokud ma stravnik svou objednavku na burze
                // proto musi nacist i objednavky, ktere jeste nebyly zneplatnene
                ' ORDER BY StavVlozeniNaBurzu';
        } else {
            $sql .=
                ' AND StavVlozeniNaBurzu = 3 ' .  // pouze potvrzene
                ' ORDER BY DatumCasVlozeniNaBurzu ';

            // jeste zkontroluje stav uctu a pocet objednavek
            $polozkaJidelnicku->nactiInfoZBurzy($database, $ppomer);

            $cenovakategorie = new CenovaKategorie($ppomer->rc, $ppomer->osobniCislo, $ppomer->datumNastupu);
            $cenovakategorie->nactiVlastnostiKDatu($database, $polozkaJidelnicku->f_den);

            if (!$polozkaJidelnicku->lzeVyzvednoutZBurzy) return false;
        }

        $result = false;

        $database ->beginTransaction();
        try {
            $cursor = $database ->getSqlResult($sql, $params);
            $row = $cursor ->fetch();
            $cursor ->close();

            if ($row) {
                if ($vlastni) $result = true;  // vlastni nalezena - nebude zkouset vyzvednout cizi, proto vzdy vraci true

                switch ($row[1]) { // stav vlozeni na burzu
                    case 1 :  // zatim nepotvrzeno - lze vyzvednout okamzite, je to vlastni objednavka
                        self::zrusVlozeniNaBurzu($database, $row[0]);
                        break;

                    case 2 : // probiha update v terminalu, je to vlastni objednavka => nutno pockat
                        $database ->rollbackTransaction();
                        MessageManager::addErrorMessage( TextManager::_('Ted_nelze_vyzvednout_z_burzy') );
                        break;

                    case 3 : // potvrzeno
                        if ($vlastni && self::lzePouzitPrimouBurzu($database, $polozkaJidelnicku)) {
                            self::zrusVlozeniNaBurzu($database, $row[0]);
                        } else {
                            //PotvrdVyzvednutiZBurzyAVlozKopii(_connection, Q, RC, OsobniCislo, DatumNastupu, username);
                            //result := true;
                            self::vlozKopiiZBurzyAZrusPuvodni($database, $ppomer, $polozkaJidelnicku, $row[0]);
                            $result = true;
                        }
                        break;

                    default :
                        throw new Exception('Chyba - neznamy stav vlozeni objednavky na burzu!');
                }
            }

            $database ->commitTransaction();

        } catch (Exception $e) {
            $database ->rollbackTransaction();
            throw $e;
        }

        return $result;
    }


    /**
     * @param Database $database
     * @param PracovniPomer $ppomer
     * @param PolozkaJidelnicku $polozkaJidelnicku
     * @throws Exception
     */
    static function vyzvedniZBurzy($database, $ppomer, $polozkaJidelnicku)
    {
        if (!self::vyzvednutiPolozkyZBurzy($database, $ppomer, $polozkaJidelnicku, true))    // zkusi vlastni
            if (!self::vyzvednutiPolozkyZBurzy($database, $ppomer, $polozkaJidelnicku, false)) // nevyslo - zkusi jinou
                throw new Exception( TextManager::_('Nelze_vyzvednout_z_burzy') );
    }

}
