<?php

namespace RonPortal\Jidelna;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use RonPortal\PracovniPomer;


class JidelnaUtils
{
    /**
     * @param Database $database
     * @param int    $id        Pokud je zadano $id, tak vraci pocet objednavek teto polozky jidelnicku,
     * @param string $typ       ... jinak pokud je zadan typ tak vraci pocet objednavek pro zadany typ,
     * @param int    $datum     ... jinak vraci pocet vsech objednavek osoby na zadany datum
     * @param bool   $pouzeNevydane
     * @param PracovniPomer $pp
     * @param int    $stavBurza
     *      0 - bez ohledu na burzu
     *      3 - potvrzene v burze
     *      4 - jakekoliv v burze (nepotvrzene, prave posilane do terminalu i potvrzene)
     * @return int
     * @throws Exception
     */
    static function pocetObjednavek($database, $id, $typ, $datum, $pouzeNevydane = false, $pp = null, $stavBurza = 0)
    {
        $params = $database ->createParameters();

        if ($id)
            $sql =
                'SELECT COUNT(*) FROM ObjednaneJidlo ' .
                ' WHERE ObjednaneJidlo.IDPolozkyJidelnicku = ' . $params ->addParameter((int)$id);
        else if ($typ)
            $sql =
                'SELECT COUNT(*) FROM ObjednaneJidlo, PolozkaJidelnicku ' .
                ' WHERE PolozkaJidelnicku.Id = ObjednaneJidlo.IDPolozkyJidelnicku ' .
                ' AND PolozkaJidelnicku.typDennihoJidla  = ' . $params->addParameter($typ) .
                ' AND PolozkaJidelnicku.den  = ' . $params ->addDateParameter($datum);
        else
            $sql =
                'SELECT COUNT(*) FROM ObjednaneJidlo, PolozkaJidelnicku ' .
                ' WHERE PolozkaJidelnicku.Id = ObjednaneJidlo.IDPolozkyJidelnicku ' .
                ' AND PolozkaJidelnicku.Den = ' . $params ->addDateParameter($datum);

        $sql .= // zrusene nepocita
            ' AND ObjednaneJidlo.DatumCasZruseni IS NULL ';

        if ($pp)
            $sql .=
                ' AND ObjednaneJidlo.RC = ' . $params->addParameter($pp ->rc) .
                ' AND ObjednaneJidlo.OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
                ' AND ObjednaneJidlo.DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu);

        if ($pouzeNevydane)
            $sql .=
                ' AND ObjednaneJidlo.DatumCasVydani IS NULL ';

        // burza
        switch ($stavBurza) {
            case 0 : // bez ohledu na burzu
                break;

            case 1 : // pouze nepotvrzene
            case 2 : // prave odesilane
            case 3 : // pouze potvrzene
                $sql .= ' AND ObjednaneJidlo.DatumCasVlozeniNaBurzu IS NOT NULL  AND  ObjednaneJidlo.StavVlozeniNaBurzu = ' . $params ->addParameter((int)$stavBurza);
                break;

            case 4 : // vsechny v burze, bez ohledu na stav
                $sql .= " AND ObjednaneJidlo.DatumCasVlozeniNaBurzu IS NOT NULL ";
                break;
        }

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : 0;
    }


    private static $cacheObjednavekDatumOd;
    private static $cacheObjednavekDatumDo;
    private static $cacheObjednavekJidelna;

    /**
     * Nastavi obdobi a jidelnu pro nacitani objednavek do cache
     * @param int $datumOd
     * @param int $datumDo
     * @param string $kodJidelny
     * @return void
     */
    public static function nastavCacheObjednavek(int $datumOd, int $datumDo, string $kodJidelny)
    {
        self::$cacheObjednavekDatumOd = $datumOd;
        self::$cacheObjednavekDatumDo = $datumDo;
        self::$cacheObjednavekJidelna = $kodJidelny;
    }

    /** @var array  Cache zaznamu s celkovym poctem objednavek v burze (jen potvrzene) */
    private static $cacheVBurzeCelkem = null;
    /** @var array  Cache zaznamu s poctem objednavek osoby v burze (vcetne nepotvrzenych, aby si je mohl vyzvednout zpet) */
    private static $cacheVBurzeOsoba = null;

    /**
     * @param Database $database
     * @param string $sql
     * @param Parameters $params
     * @return array
     */
    private static function nactiCache($database, $sql, $params)
    {
        $result = [];

        $cursor = $database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $result[(int)$row[0]] = $row[1];
        }
        $cursor ->close();

        return $result;
    }

    /**
     * Vraci celkovy pocet objednavek zadane polozky jidelnicku v burze
     * @param Database $database
     * @param int $idPolozkyJidelnicku
     * @return int
     * @throws Exception
     */
    public static function pocetVBurzeCelkem($database, $idPolozkyJidelnicku)
    {
        if (self::$cacheObjednavekDatumOd) {
            if (!isset(self::$cacheVBurzeCelkem)) {
                // je zapnuta cache => nacte pocet jidel v burze hromadne pro vsechny polozky jidelnicku
                $params = $database->createParameters();
                $sql =
                    'SELECT PolozkaJidelnicku.Id, COUNT(*)' .
                    "\nFROM PolozkaJidelnicku, ObjednaneJidlo" .
                    "\nWHERE ObjednaneJidlo.IdPolozkyJidelnicku = PolozkaJidelnicku.Id" .
                    '  AND PolozkaJidelnicku.Den >= ' . $params->addDateParameter(self::$cacheObjednavekDatumOd) .
                    '  AND PolozkaJidelnicku.Den <= ' . $params->addDateParameter(self::$cacheObjednavekDatumDo) .
                    '  AND PolozkaJidelnicku.Jidelna = ' . $params->addParameter(self::$cacheObjednavekJidelna) .
                    '  AND ObjednaneJidlo.DatumCasZruseni IS NULL' .
                    '  AND ObjednaneJidlo.DatumCasVydani IS NULL' .
                    '  AND ObjednaneJidlo.DatumCasVlozeniNaBurzu IS NOT NULL  AND  ObjednaneJidlo.StavVlozeniNaBurzu = 3' . // pouze potvrzene
                    "\nGROUP BY PolozkaJidelnicku.Id";

                self::$cacheVBurzeCelkem = self::nactiCache($database, $sql, $params);
            }

            return (float)(self::$cacheVBurzeCelkem[$idPolozkyJidelnicku] ?? 0);
        }

        // cache neni povolena, nebo nenasel hodnotu v cache
        return self::pocetObjednavek($database, $idPolozkyJidelnicku, null, 0, true, null, 3); // jen nevydane = true, pp = null, 3 = potvrzene
    }


    /**
     * @param Database $database
     * @param int $idPolozkyJidelnicku               Id polozky jidelnicku
     * @param PracovniPomer $pp
     * @return int
     */
    public static function pocetVBurze_Osoba($database, $idPolozkyJidelnicku, $pp)
    {
        if (self::$cacheObjednavekDatumOd) {
            if (!isset(self::$cacheVBurzeOsoba)) {
                // je zapnuta cache => nacte pocet jidel v burze hromadne pro vsechny polozky jidelnicku
                $params = $database->createParameters();
                $sql =
                    'SELECT PolozkaJidelnicku.Id, COUNT(*)' .
                    "\nFROM PolozkaJidelnicku, ObjednaneJidlo" .
                    "\nWHERE ObjednaneJidlo.IdPolozkyJidelnicku = PolozkaJidelnicku.Id" .
                    '  AND PolozkaJidelnicku.Den >= ' . $params->addDateParameter(self::$cacheObjednavekDatumOd) .
                    '  AND PolozkaJidelnicku.Den <= ' . $params->addDateParameter(self::$cacheObjednavekDatumDo) .
                    '  AND PolozkaJidelnicku.Jidelna = ' . $params->addParameter(self::$cacheObjednavekJidelna) .
                    '  AND ObjednaneJidlo.RC = ' . $params->addParameter($pp ->rc) .
                    '  AND ObjednaneJidlo.OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
                    '  AND ObjednaneJidlo.DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu) .
                    '  AND ObjednaneJidlo.DatumCasZruseni IS NULL' .
                    '  AND ObjednaneJidlo.DatumCasVydani IS NULL' .
                    '  AND ObjednaneJidlo.DatumCasVlozeniNaBurzu IS NOT NULL  AND  ObjednaneJidlo.StavVlozeniNaBurzu IS NOT NULL' . // vsechny v burze (i nepotvrzene)
                    "\nGROUP BY PolozkaJidelnicku.Id";

                self::$cacheVBurzeOsoba = self::nactiCache($database, $sql, $params);
            }

            return (float)(self::$cacheVBurzeOsoba[$idPolozkyJidelnicku] ?? 0);
        }


        return self::pocetObjednavek($database, $idPolozkyJidelnicku, null, 0, true, $pp, 4); // jen nevydane, jen osoba, 4 = vsechny v burze
    }


    /**
     * @param $database
     * @param $id
     * @return int
     * @throws Exception
     */
    public static function pocetObjednavekPolozkyCelkovy($database, $id)
    {
        return self::pocetObjednavek($database, $id, null, 0, false);
    }


    private static $cacheCasovaPredvolba = null;

    private static function nactiCacheCasovaPredvolba($database)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT PolozkaJidelnicku.Id, PolozkaJidelnicku.Den, MIN(CasovaPredvolbaVydeje.CasOd), MAX(CasovaPredvolbaVydeje.CasDo), MIN(CasovaPredvolbaVydeje.JidelnicekZPredchozihoDne)'.
            "\nFROM PolozkaJidelnicku, CasovaPredvolbaVydeje, Terminal, MapovaniJidlaVTerminalu".
            "\nWHERE PolozkaJidelnicku.Den >= " . $params->addDateParameter(self::$cacheObjednavekDatumOd) .
            '  AND PolozkaJidelnicku.Den <= ' . $params->addDateParameter(self::$cacheObjednavekDatumDo) .
            '  AND PolozkaJidelnicku.Jidelna = ' . $params->addParameter(self::$cacheObjednavekJidelna) .
            "\n AND Terminal.KodTypuRozdeleni = PolozkaJidelnicku.KodTypuRozdeleni".
            ' AND Terminal.KodRozdeleni = PolozkaJidelnicku.Jidelna'.
            "\n AND MapovaniJidlaVTerminalu.Terminal = Terminal.Kod".
            ' AND MapovaniJidlaVTerminalu.TypDennihoJidla = PolozkaJidelnicku.TypDennihoJidla'.
            "\n AND CasovaPredvolbaVydeje.Terminal = Terminal.Kod".
            ' AND (CasovaPredvolbaVydeje.KodTypuVTerminalu = MapovaniJidlaVTerminalu.KodTypuVTerminalu'.
            ' OR CasovaPredvolbaVydeje.Priloha1 = MapovaniJidlaVTerminalu.KodTypuVTerminalu'.
            ' OR CasovaPredvolbaVydeje.Priloha2 = MapovaniJidlaVTerminalu.KodTypuVTerminalu)'.
            "\nGROUP BY PolozkaJidelnicku.Id, PolozkaJidelnicku.Den" .
            "\nORDER BY PolozkaJidelnicku.Id";

        $cursor = $database ->getSqlResult($sql, $params);

        self::$cacheCasovaPredvolba = [];
        while ($row = $cursor ->fetch()) {
            $den = $database ->stringToTimestamp($row[1]);
            if ($row[4]) {
                // vydej z jidelnicku predchoziho dne
                $den = DateUtils::incDayStart($den, 1);
            }
            self::$cacheCasovaPredvolba[(int)$row[0]] = [
                'vydejOd' => $den + 3600 * $database ->stringToTimeOfDay($row[2]),
                'vydejDo' => $den + 3600 * $database ->stringToTimeOfDay($row[3]),
                'jidelnicekPredchozihoDne' => (int)$row[4]
            ];
        }
        $cursor ->close();
    }

    /**
     * Vraci pole, ktere obsahuje casOd a casDo - cas vydeje zadane polozky jidelnicku (podle nastaveni u terminalu)
     * @param Database $database
     * @param int $idPolozkyJidelnicku
     * @return array
     * @throws DBException
     */
    public static function casVydejePolozkyJidelnicku($database, $idPolozkyJidelnicku)
    {
        if (self::$cacheObjednavekDatumOd) {
            if (!isset(self::$cacheCasovaPredvolba)) {
                self::nactiCacheCasovaPredvolba($database);
            }

            return self::$cacheCasovaPredvolba[$idPolozkyJidelnicku] ?? null;
        }

        $params = $database ->createParameters();
        $sql =
            "SELECT DISTINCT PolozkaJidelnicku.den, CasovaPredvolbaVydeje.CasOd, CasovaPredvolbaVydeje.CasDo, CasovaPredvolbaVydeje.JidelnicekZPredchozihoDne".
            " FROM PolozkaJidelnicku, CasovaPredvolbaVydeje, Terminal, MapovaniJidlaVTerminalu".
            ' WHERE PolozkaJidelnicku.Id = ' . $params ->addParameter((int)$idPolozkyJidelnicku) .
            " AND Terminal.kodTypuRozdeleni = PolozkaJidelnicku.kodTypuRozdeleni".
            " AND Terminal.KodRozdeleni = PolozkaJidelnicku.jidelna".
            " AND MapovaniJidlaVTerminalu.Terminal = Terminal.kod".
            " AND MapovaniJidlaVTerminalu.TypDennihoJidla = PolozkaJidelnicku.TypDennihoJidla".
            " AND CasovaPredvolbaVydeje.Terminal = Terminal.kod".
            " AND ( CasovaPredvolbaVydeje.KodTypuVTerminalu =  MapovaniJidlaVTerminalu.KodTypuVTerminalu".
            " OR CasovaPredvolbaVydeje.Priloha1 =  MapovaniJidlaVTerminalu.KodTypuVTerminalu".
            " OR CasovaPredvolbaVydeje.Priloha2 =  MapovaniJidlaVTerminalu.KodTypuVTerminalu )".
            " ORDER BY CasovaPredvolbaVydeje.JidelnicekZPredchozihoDne, CasovaPredvolbaVydeje.CasOd";

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            $den = $database ->stringToTimestamp($row[0]);
            if ($row[3]) {
                // vydej z jidelnicku predchoziho dne
                $den = DateUtils::incDayStart($den, 1);
            }
            return [
                'vydejOd' => $den + 3600 * $database ->stringToTimeOfDay($row[1]),
                'vydejDo' => $den + 3600 * $database ->stringToTimeOfDay($row[2]),
                'jidelnicekPredchozihoDne' => (int)$row[3]
            ];
        }

        return null;
    }
}

