<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\RonException;

class Kalendar
{
    private static $svatkyVObdobi = null;

    /** @var string */
    public $kod;
    /** @var string */
    public $popis;

    /** @var int */
    public $dochazkaPodlePlanKapacit;

    /** @var int    Platnost prirazeni kalendare od */
    public $platnostOd;
    /** @var int    Platnost prirazeni kalendare do */
    public $platnostDo;

    /**
     * Nacte seznam svatku pro zadane obdobi.
     * Pokud je datumOd (datumDo) = 0, tak pouzije datum podle aktualne zvoleneho obdobi
     * @param Database $database
     * @param int $datumOd
     * @param int $datumDo
     * @param string $kalendar
     * @param bool $forceReload
     * @return array
     * @throws \Ron\DB\DBException
     */
    public static function nactiSvatkyVObdobi($database, $datumOd, $datumDo, $kalendar = '', $forceReload = false)
    {
        if (!$forceReload  &&  isset(self::$svatkyVObdobi)) {
            return self::$svatkyVObdobi;
        }

        $result = [];

        $params = $database ->createParameters();
        $sql =
            'SELECT DISTINCT Den FROM DenniUdajKalendare ' .
            ' WHERE Den >= ' . $params ->addDateParameter($datumOd) .
            ' AND Den <= ' . $params ->addDateParameter($datumDo) .
            ' AND (TypDne = 2 OR TypDne = 3) ' . // placeny nebo neplaceny svatek
            ($kalendar
                ? ' AND Kalendar = ' . $params ->addParameter($kalendar)
                : '') .
            ' ORDER BY Den ';

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $result[] = $cursor ->stringToTimestamp($row[0]);
        }

        $cursor ->close();

        self::$svatkyVObdobi = $result;

        return $result;
    }


    /**
     * Nacte seznam svatku pro zadany pracovni pomer (podle prirazenych kalendaru) v zadanem obdobi.
     * Pokud je datumOd (datumDo) = 0, tak pouzije datum podle aktualne zvoleneho obdobi
     * @param Database  $database
     * @param PracovniPomer $pp
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $forceReload
     * @return array
     * @throws \Ron\DB\DBException
     */
    public static function nactiSvatkyOsoby($database, $pp, $datumOd, $datumDo, $forceReload = false)
    {
        if (!$forceReload  &&  isset(self::$svatkyVObdobi)) {
            return self::$svatkyVObdobi;
        }

        $result = [];

        $params = $database ->createParameters();
        $sql =
            'SELECT DISTINCT DenniUdajKalendare.Den' .
            " FROM DenniUdajKalendare, PrirazeniKalendare\n" .
            " WHERE PrirazeniKalendare.Kalendar = DenniUdajKalendare.Kalendar\n" .
            ' AND DenniUdajKalendare.Den >= ' . $params ->addDateParameter($datumOd) .
            ' AND DenniUdajKalendare.Den <= ' . $params ->addDateParameter($datumDo) .
            " AND (TypDne = 2 OR TypDne = 3)\n" . // placeny nebo neplaceny svatek
            ' AND PrirazeniKalendare.RC = ' . $params ->addParameter($pp ->rc) .
            ' AND PrirazeniKalendare.OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND PrirazeniKalendare.DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu) . "\n" .
            ' AND PrirazeniKalendare.PlatnostOd <= DenniUdajKalendare.Den' .
            " AND (PrirazeniKalendare.PlatnostDo >= DenniUdajKalendare.Den  OR  PrirazeniKalendare.PlatnostDo IS NULL)\n" .
            ' ORDER BY Den';

        $cursor = $database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $result[] = $cursor ->stringToTimestamp($row[0]);
        }

        $cursor ->close();

        self::$svatkyVObdobi = $result;

        return $result;
    }




    /**
     * Vraci true, pokud je zadany den svatek
     * @param Database $database
     * @param int $datum
     * @param string $kalendar
     * @return bool
     */
    static function jeSvatek($database, $datum, $kalendar)
    {
        try {
            self::nactiSvatkyVObdobi($database, DateUtils::firstDayOfMonth($datum), DateUtils::lastDayOfMonth($datum), $kalendar);
        } catch (RonException $e) {
            MessageManager::addDebugLog($e ->getMessage() . "\n" . $e ->getErrorDescription());
        } catch (\Exception $e) {
            MessageManager::addDebugLog($e ->getMessage());
        }

        return in_array($datum, self::$svatkyVObdobi);
    }

    const VIKEND = 1;
    const SVATEK = 2;

    /**
     * Vraci 0 pro pracovni den, 1 pro vikend, 2 pro svatek, 3 pro vikend-svatek
     * pokud je $nacistsvatky = false, tak svatky nevyhledava
     * (napr. v historii pruchodu, protoze v APS nejsou kalendare)
     * pokud pri nacitani svatku neni zadan kalendar, tak hleda svatky ve vsech kalendarich
     * POZOR! Pokud je metoda volana s nejakym kalendarem, tak pri dalsim volani uz se pouzije ten samy!
     *
     * @param Database $database
     * @param int    $datum
     * @param bool   $nacistSvatky
     * @param string $kalendar
     * @return int
     */
    static function vratTypDne($database, $datum, $nacistSvatky = true, $kalendar = '')
    {
        $jeSvatek = $nacistSvatky ? self::jeSvatek($database, $datum, $kalendar) : false;

        switch (date('w', $datum)) {
            case 6 : // sobota
            case 0 : // nedele
                return self::VIKEND + ($jeSvatek ? self::SVATEK : 0); // vikend (pripadne i svatek)

            default :
                return $jeSvatek ? self::SVATEK : 0; // svatek / normalni den
        }
    }


    /**
     * Vraci jmeno css tridy pro zadany index dne (nedele=0, pondeli=1, ...)
     * @param $dayIndex
     * @return string
     */
    static function getStyleForDay($dayIndex)
    {
        if ( $dayIndex == 6 || $dayIndex == 0 ) {
            // vikend
            return 'browser_row_high';
        }

        return '';
    }


    /**
     * Vraci jmeno css tridy pro zadany datum - pouziva se pro radky seznamu (zpracovana dochazka atd.)
     * @param $database
     * @param $datum
     * @param bool|true $nacistSvatky
     * @param string $kalendar
     * @return string
     */
    static function getStyleForDate($database, $datum, $nacistSvatky = true, $kalendar = '')
    {
        $result = (DateUtils::startOfTheDay($datum) == DateUtils::today()) ? 'today ' : '';

        if ($nacistSvatky) {
            $typdne = self::vratTypDne($database, $datum, $nacistSvatky, $kalendar);
            switch ($typdne) {
                case 1 :
                    return $result . 'browser_row_high vikend';

                case 2 :
                    return $result . 'browser_row_high svatek';

                case 3 :
                    return $result . 'browser_row_high vikend svatek';

                default :
                    break;
            }
        }

        return $result . self::getStyleForDay(date('w', $datum));
    }


    /**
     * Vraci jmeno css tridy pro zadany datum - pouziva se bunky v tabulce (napr. rocni prehled)
     * @param $database
     * @param $datum
     * @param bool|true $nacistSvatky
     * @param string $kalendar
     * @return string
     */
    static function getBackgroundStyleForDate($database, $datum, $nacistSvatky = true, $kalendar = '')
    {
        $result = (DateUtils::startOfTheDay($datum) == DateUtils::today()) ? 'today ' : '';

        if ($nacistSvatky) {
            $typDne = self::vratTypDne($database, $datum, $nacistSvatky, $kalendar);
            switch ($typDne) {
                case 1 :
                    return $result . 'background-vikend';

                case 2 :
                    return $result . 'background-svatek';

                case 3 :
                    return $result . 'background-vikend background-svatek';

                default :
                    break;
            }
        }

        return $result . self::getStyleForDay(date('w', $datum));
    }


    /**
     * @param Database $database
     * @param string $kod
     * @return Kalendar
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public static function nactiKalendar($database, $kod)
    {
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult(
            'SELECT Kod, Popis, DochazkaPodlePlanKapacit' .
            ' FROM Kalendar WHERE Kod = ' . $params ->addParameter($kod),
            $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new \Exception('Chyba ve funkci Kalendar::nactiKalendar - zadany kalendar neexistuje!');
        }

        $result = new Kalendar();
        $result ->kod = $row[0];
        $result ->popis = $row[1];
        $result ->dochazkaPodlePlanKapacit = $row[2];

        return $result;
    }


    /**
     * Nacte kalendare prirazene PP v zadanem obdobi
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return Kalendar[]
     */
    static function nactiKalendareOsoby($database, $pp, $obdobiOd, $obdobiDo)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Kalendar.Kod, Kalendar.Popis, Kalendar.DochazkaPodlePlanKapacit, PrirazeniKalendare.PlatnostOd, PrirazeniKalendare.PlatnostDo' .
            ' FROM PrirazeniKalendare, Kalendar ' .
            ' WHERE Kalendar.Kod = PrirazeniKalendare.Kalendar' .
            '   AND RC = '. $params ->addParameter($pp ->rc) .
            '   AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            '   AND DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu) .
            '   AND PlatnostOd <= ' . $params ->addDateParameter($obdobiDo) .
            '   AND (PlatnostDo >= ' . $params ->addDateParameter($obdobiOd) . ' OR PlatnostDo IS NULL)' .
            ' ORDER BY PlatnostOd ';

        $result = [];

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani prirazeneho kalendare!');
        while ($row = $cursor ->fetch()) {
            $kalendar = new static();

            $kalendar ->kod = $row[0];
            $kalendar ->popis = $row[1];
            $kalendar ->dochazkaPodlePlanKapacit = $row[2];

            $kalendar ->platnostOd = $database ->stringToTimestamp($row[3]);
            $kalendar ->platnostDo = $database ->stringToTimestamp($row[4]);

            $result[] = $kalendar;
        }
        $cursor ->close();

        return $result;
    }


    /**
     * Vraci kod kalendare, ktery je prirazen pracovnimu pomeru v zadanem obdobi
     * @param Database $database
     * @param string $rc
     * @param string $osobniCislo
     * @param int  $datumNastupu
     * @param int  $obdobiOd
     * @param int  $obdobiDo
     * @return string|null
     */
    static function vratKalendarOsoby($database, $rc, $osobniCislo, $datumNastupu, $obdobiOd, $obdobiDo)
    {
        if (!isset($rc)) {  // kvuli ibase
            return null;
        }

        global $bezdochazky;
        if (isset($bezdochazky) && $bezdochazky) {
            // v pristupaku nejsou kalendare
            return null;
        }

        $params = $database ->createParameters();
        $sql =
            'SELECT Kalendar FROM PrirazeniKalendare ' .
            ' WHERE RC = '. $params ->addParameter($rc) .
            '   AND OsobniCislo = ' . $params ->addParameter($osobniCislo) .
            '   AND DatumNastupu = ' . $params ->addDateParameter($datumNastupu) .
            '   AND PlatnostOd <= ' . $params ->addDateParameter($obdobiDo) .
            '   AND (PlatnostDo >= ' . $params ->addDateParameter($obdobiOd) . ' OR PlatnostDo IS NULL)' .
            ' ORDER BY PlatnostOd ';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani prirazeneho kalendare!');
        $row = $cursor ->fetch();
        $cursor ->close();
        return $row ? $row[0] : null;
    }


    /**
     * Vraci kod kalendare, ktery je prirazen aktualnimu uzivateli v zadanem obdobi.
     * @param Database $database
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return string|null
     */
    static function vratKalendarUzivatele($database, $obdobiOd, $obdobiDo)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Kalendar FROM PrirazeniKalendare' .
            ' WHERE RC = '. $params ->addParameter(User::getRC()) .
            '   AND PlatnostOd <= ' . $params ->addDateParameter($obdobiDo) .
            '   AND (PlatnostDo >= ' . $params ->addDateParameter($obdobiOd) . ' OR PlatnostDo IS NULL)' .
            ' ORDER BY PlatnostOd';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani prirazeneho kalendare!');
        $row = $cursor ->fetch();
        $cursor ->close();
        return $row ? $row[0] : null;
    }


    /**
     * Vraci kod kalendare, ktery je prirazen zadanemu rozdeleni.
     * @param Database $database
     * @param int $kodTypuRozdeleni
     * @param string $kodRozdeleni
     * @return null|string
     */
    static function vratKalendarRozdeleni($database, $kodTypuRozdeleni, $kodRozdeleni)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT Kalendar FROM Rozdeleni' .
            ' WHERE KodTypuRozdeleni = '. $params ->addParameter((int)$kodTypuRozdeleni) .
            '   AND Kod = ' . $params ->addParameter($kodRozdeleni);

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani prirazeneho kalendare!');
        $row = $cursor ->fetch();
        $cursor ->close();
        return $row ? $row[0] : null;
    }
}
