<?php
/**
 * stanislav.elbl, 25.02.2019
 * Formular pro editaci podminek planovani pro rozdeleni
 */

namespace RonPortal\Kapacity;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\StringUtils;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\Prava;
use RonPortal\Rozdeleni;

/**
 * Class HodnotaPodminekPlanovani
 * Data pro jednu bunku formulare.
 * @package Ron\Kapacity
 */
class HodnotaPodminekPlanovani
{
    /** @var int    Id podminky, ktera se edituje (prioritni) */
    public $idPodminky;
    /** @var int    Hodnota z platne podminky. Po editaci se zmeni na editovanou. */
    public $hodnota;
    /** @var int    Hodnota z vychozi podminky (pro porovnani a zvyrazneni rozdilu) */
    public $vychoziHodnota;

    /** @var int    Priznak, jestli lze editovat */
    public $lzeEditovat;

    // TODO: umoznit omezeni pomoci maximalni hodnoty z vychozi podminky?

    /** @var string Hodnota jako text - nacita se i pri POST */
    public $hodnotaStr = '';
    /** @var int    Hodnota pred editaci (nactena z databaze) */
    public $puvodniHodnota;

    /** @var string */
    public $updateUserName;
    /** @var int */
    public $updateTimestamp;
    /** @var string */
    public $updateTitle;

    /** @var string Jmeno html inputu */
    public $inputName;


    /**
     * Nastavi data podle zadane podminky.
     * @param PodminkaPlanovaniKapacit $podminka
     * @param bool $prioritni   Urcuje, jestli se jedna o prioritni podminku
     */
    public function nastavPodlePodminky($podminka, $prioritni)
    {
        if ($prioritni) {
            $this ->idPodminky = $podminka ->id;
            $this ->hodnota = $podminka ->pocetMin;
            $this ->hodnotaStr = (string)$this ->hodnota;
            $this ->puvodniHodnota = $this ->hodnota;

            $this ->updateUserName = $podminka ->updateUserName;
            $this ->updateTimestamp = $podminka ->updateTimestamp;

            $this ->updateTitle = $podminka ->updateUserName
                ? $podminka ->updateUserName . ', ' . StringUtils::format('finedatetime', $podminka ->updateTimestamp)
                : '';
        } else {
            $this ->vychoziHodnota = $podminka ->pocetMin;
        }

        $this ->lzeEditovat = $podminka ->lzeEditovat;
    }

    public function nastavPodleVychoziPodminky()
    {
        if ($this ->idPodminky) {
            return;
        }

        $this ->hodnota = $this ->vychoziHodnota;
        $this ->hodnotaStr = (string)$this ->hodnota;
        $this ->puvodniHodnota = $this ->hodnota;
    }

    public function getHtmlClass()
    {
        if ($this ->hodnota > $this ->vychoziHodnota) {
            return 'raising';
        }

        if ($this ->hodnota < $this ->vychoziHodnota) {
            return 'missing';
        }

        return '';
    }

    public function getHtmlTitle()
    {
        if ($this ->updateTitle  ||  $this ->hodnota != $this ->vychoziHodnota) {
            return sprintf(\Ron\ronText('Puvodni_hodnota_podminek'), $this ->updateTitle,"" . ((int)$this ->vychoziHodnota));
        }

        return '';
    }
}


/**
 * Class RadekPodminekPlanovani
 * Radek formulare - obsahuje bunky pro jednu podminku (nikoliv ID, ale podle nastaveni)
 * @package Ron\Kapacity
 */
class RadekPodminekPlanovani
{
    /** @var PodminkaPlanovaniKapacit  */
    public $podminka;

    /** @var HodnotaPodminekPlanovani[] */
    public $data;

    public function getHtmlPopis()
    {
        return $this ->podminka ? $this ->podminka ->getPopis() : '';
    }


    public static function sortRows(&$rows)
    {
        usort($rows, function($radek1, $radek2) {
            return PodminkaPlanovaniKapacit::compare($radek1 ->podminka, $radek2 ->podminka); });
    }
}



class DataPodminekPlanovani
{

}


class PodminkyPlanovaniForm extends BaseViewObject
{
    /** @var Database */
    protected $database;

    /** @var Obdobi */
    public $obdobi;

    /** @var Rozdeleni */
    public $rozdeleni;

    /** @var PlanovaniKapacit_Den[] */
    public $dny;

    /** @var  SadaPodminekPlanovani[]   Sady podminek nactene z databaze */
    public $sadyPodminek;

    /** @var  RadekPodminekPlanovani[] */
    public $radky;

    /** @var SchvaleniPodminekPlanovani */
    public $schvaleni;

    public $editace = false;
    public $pravoEditovat = false;


    /**
     * PodminkyPlanovaniForm constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        parent::__construct('ronportal/kapacity/view_podminkyform.php');
        $this->database = $database;
    }




    /**
     * @param PodminkaPlanovaniKapacit $podminka
     * @param bool $addNewLine
     * @return RadekPodminekPlanovani
     */
    public function najdiRadekProPodminku($podminka, $addNewLine = false)
    {
        foreach ($this ->radky as $radek) {
            if ($radek ->podminka ->isSameAs($podminka)) {
                return $radek;
            }
        }

        if ($addNewLine) {
            $radek = new RadekPodminekPlanovani();
            $radek ->podminka = $podminka;
            $radek ->data = [];
            foreach ($this ->dny as $index => $den) {
                $radek ->data[] = new HodnotaPodminekPlanovani();
            }

            $this ->radky[] = $radek;

            return $radek;
        }

        return null;
    }


    public function zpracujSadyPodminek()
    {
        $this ->radky = [];

        $pocetDnu = DateUtils::daysBetween($this ->obdobi ->zacatek, $this ->obdobi ->konec) + 1;

        /** @var SadaPodminekPlanovani[] $prirazeneSady - sady podminek prirazene k jednotlivym dnum
         * Pro kazdy den muze byt jen jedna sada (vychozi, nebo prioritni)
         */
        $prirazeneSady = [];

        // najde podminky platne pro konkretni dny
        foreach ($this ->sadyPodminek as $sada) {
            // TODO: Doresit omezeni na typ dne/den v tydnu
            if ($sada ->prioritni <= 1) { // jen vychozi (0) a prioritni (1)
                $indexFrom = $sada->datumOd <= $this->obdobi->zacatek
                    ? 0
                    : DateUtils::daysBetween($this->obdobi->zacatek, $sada->datumOd);

                $indexTo = (!$sada->datumDo || $sada->datumDo >= $this->obdobi->konec)
                    ? $pocetDnu - 1
                    : DateUtils::daysBetween($this->obdobi->zacatek, $sada->datumDo);

                foreach ($sada ->podminky as $podminka) {
                    if ($podminka ->lzeEditovat) {
                        $radek = $this->najdiRadekProPodminku($podminka, true);

                        for ($i = $indexFrom; $i <= $indexTo; $i++) {
                            $radek->data[$i]->nastavPodlePodminky($podminka, $sada->prioritni);
                        }
                    }
                }

                for ($i = $indexFrom; $i <= $indexTo; $i++) {
                    if (!isset($prirazeneSady[$i]) || (!$prirazeneSady[$i]->prioritni && $sada->prioritni)) {
                        $prirazeneSady[$i] = $sada;
                    }
                }
            }
        }

        // doplni vychozi hodnoty ve dnech, kde neni prioritni sada
        //for ($i = 0; $i < count($this ->dny); $i++) {
        foreach ($this ->dny as $i => $den) {
            if (isset($prirazeneSady[$i])  &&  !$prirazeneSady[$i] ->prioritni) {
                foreach ($this ->radky as $indexRadku => $radek) {
                    $this->radky[$indexRadku] ->data[$i] ->nastavPodleVychoziPodminky();
                }
            }
        }

        // nakonec radky seradi (typu podminky a casu)
        RadekPodminekPlanovani::sortRows($this ->radky);
    }

    /**
     * @param Obdobi $obdobi
     * @param Rozdeleni $rozdeleni
     * @param bool $editace
     * @throws DBException|NeniPravoException
     */
    public function load($obdobi, $rozdeleni, $editace = false)
    {
        $this ->obdobi = $obdobi;
        $this ->rozdeleni = $rozdeleni;

        // naplni seznam dnu
        $this ->dny = PlanovaniKapacit_Den::nactiDnyProObdobi($this ->database, $this ->obdobi ->zacatek, $this ->obdobi ->konec, null);

        $this ->schvaleni = SchvaleniPodminekPlanovani::nactiSchvaleni($this ->database, $obdobi ->zacatek, $rozdeleni ->typ, $rozdeleni ->kod);

        // podle schvaleni nastavi moznost editace
        $lzeEditovat = true;
        for ($i = count($this ->dny) - 1;  $i >= 0;  $i--) {
            $lzeEditovat = $lzeEditovat && $this ->schvaleni ->lzeEditovatDen($this ->dny[$i] ->datum);
            $this ->dny[$i] ->lzeEditovat = $lzeEditovat;
        }

        PlanovaniKapacit_Den::nactiPoznamky($this ->database, $this ->dny, $rozdeleni ->typ, $rozdeleni ->kod);

        $this ->sadyPodminek = SadaPodminekPlanovani::nactiPodminky($this ->database,
            $rozdeleni ->typ, $rozdeleni ->kod,
            $obdobi ->zacatek, $obdobi ->konec);
            // 'LzeEditovat > 0'); - nacita bez filtru, jinak se nenactou sady bez podminek!

        $this ->zpracujSadyPodminek();

        $prava = Prava::vratPrava('kapacity_pocty');
        $this ->pravoEditovat = $prava ->jePravoNaRozdeleni($rozdeleni ->typ, $rozdeleni ->kod, $obdobi ->zacatek, $obdobi ->konec);
        if ($editace && !$this->pravoEditovat) {
            throw new NeniPravoException();
        }
        $this ->editace = $editace;

        if ($this ->editace) {
            foreach ($this->radky as $radek) {
                foreach ($radek ->data as $index => $bunka) {
                    if ($bunka ->lzeEditovat != 1) {
                        // podminka se jen zobrazi
                    } else if ($bunka ->idPodminky) {
                        $bunka ->inputName = 'pri_' . $bunka ->idPodminky; // prioritni sada
                    } else {
                        $bunka ->inputName = 'vych_' . $radek ->podminka ->id . '_' . $index;
                    }
                    // TODO: vyresit situaci, kdy existuje prioritni sada, ale neni v ni prislusna podminka
                }
            }
        }
    }


    /**
     * @param $datum
     * @return SadaPodminekPlanovani
     */
    private function vratPrioritniSaduProDen($datum)
    {
        foreach ($this ->sadyPodminek as $sada) {
            if ($sada ->prioritni === 1  &&  $sada ->datumOd <= $datum  &&  ($sada ->datumDo >= $datum  ||  !isset($sada ->datumDo))) {
                return $sada;
            }
        }

        return null;
    }


    /**
     * Vytvori novou sadu podminek a priradi ji aktualnimu rozdeleni. Vraci Id nove vytvoreneho zaznamu.
     * @param int $indexDne
     * @return SadaPodminekPlanovani
     * @throws DBException
     */
    private function vytvorSaduPodminekProDen($indexDne)
    {
        $newId = SadaPodminekPlanovani::vytvorSaduPodminekProDen($this ->database, $this ->rozdeleni, $this ->dny[$indexDne] ->datum, $this);

        $sada = new SadaPodminekPlanovani();
        $sada ->datumOd = $sada ->datumDo = $this ->dny[$indexDne] ->datum;
        $sada ->id = $newId;
        $sada ->prioritni = 1;
        $sada ->typOmezeni = 0;

        return $sada;
    }


    /**
     * @param PodminkaPlanovaniKapacit $podminka
     * @param int $hodnota
     * @throws \Exception|DBException
     */
    private function editujPodminku($podminka, $hodnota)
    {
        $rec = PodminkaPlanovaniRecord::loadId($this ->database, $podminka ->id, $this);
        $rec ->fPocetMin ->setNewValue($hodnota);
        $rec ->setUpdateUser();
        $rec ->updateRecord();
    }


    /**
     * @param PodminkaPlanovaniKapacit $podminka
     * @param SadaPodminekPlanovani $sada
     * @param int $hodnota
     * @param int $datum
     * @throws \Exception
     */
    private function zkopirujPodminku($podminka, $sada, $hodnota, $datum)
    {
        // nacte puvodni podminku
        $rec = PodminkaPlanovaniRecord::loadId($this ->database, $podminka ->id, $this);

        //$id = SQLUtils::newSerialNumber($this ->database, 'PodminkaPlanovaniKapacit', 'Id');

        //$rec ->fId ->setNewValue($id);
        $rec ->fIdSady ->setNewValue($sada ->id);

        if ($rec ->fPocetMin ->value != $hodnota) {
            // hodnota je oproti puvodni hodnote zmenena
            $rec ->fPocetMin ->setNewValue($hodnota);
            $rec ->setUpdateUser();
        } else {
            $rec ->fUpdateUserName ->setNewValue(null);
            $rec ->fUpdateTimestamp ->setNewValue(null);
        }

        $rec ->addLogText(
            'Kopie podminky pro rozdeleni ' . $this ->rozdeleni ->kod . ' pro den ' . date('j.n.Y', $datum) . "\n" .
            'Id puvodni podminky: ' . $podminka ->id . "\n"
        );

        $rec ->insertRecord();
    }


    /**
     * @throws DBException
     * @throws \Exception
     */
    private function zpracujDataProDen($indexDne)
    {
        // zjisti, jestli je v danem dni zmena oproti puvodne nactenym hodnotam
        $jeZmena = false;

        foreach ($this ->radky as $indexRadku => $radek) {
            if ($radek ->data[$indexDne] ->lzeEditovat  &&  $radek ->data[$indexDne] ->hodnota != $radek ->data[$indexDne] ->puvodniHodnota) {
                $jeZmena = true;
            }
        }

        if (!$jeZmena) {
            return;
        }

        // najde prioritni sadu pro aktualni den nebo vytvori novou
        $sada = $this ->vratPrioritniSaduProDen($this ->dny[$indexDne] ->datum);
        if ($sada) {
            if ($sada ->datumOd < $sada ->datumDo) {
                throw new \Exception('Chyba - nelze editovat sadu podminek platnou pro vice dnu!');
            }
        } else {
            // vytvori novou prioritni sadu
            $sada = $this ->vytvorSaduPodminekProDen($indexDne);
        }

        // doplnit / smazat / upravit podminky podle zadanych dat
        foreach ($this ->radky as $indexRadku => $radek) {
            if ($radek ->data[$indexDne] ->lzeEditovat) {
                $podminka = $sada->najdiPodminku($radek->podminka);
                if ($podminka) {
                    if ($podminka->pocetMin != $radek->data[$indexDne]->hodnota) {
                        $this->editujPodminku($podminka, $radek->data[$indexDne]->hodnota);
                    }
                } else {
                    //if ($radek->data[$indexDne]->hodnota) { - zkopiruje vzdycky - aby byl zalogovan uzivatel
                        $this->zkopirujPodminku($radek->podminka, $sada, $radek->data[$indexDne]->hodnota, $this->dny[$indexDne]->datum);
                    //}
                }
            }
        }
    }


    public function zpracujPost()
    {
        // nacte hodnoty zadane uzivatelem
        foreach ($this ->radky as $radek) {
            foreach ($radek ->data as $index => $bunka) {
                if ($this ->dny[$index] ->lzeEditovat == 1 &&  $radek ->data[$index] ->lzeEditovat) { // jde editovat konkretni podminka i den
                    if (isset($_POST[$bunka->inputName])) {
                        $bunka->hodnotaStr = $_POST[$bunka->inputName];
                        try {
                            $bunka->hodnota = trim($bunka->hodnotaStr) === ''
                                ? null
                                : StringUtils::stringToInteger($bunka->hodnotaStr);
                        } catch (\Exception $e) {
                            MessageManager::addErrorMessage($e->getMessage(), $bunka->inputName);
                        }
                    }
                }
            }
        }

        if (MessageManager::getInstance() ->isError()) {
            return;
        }

        try {
            foreach ($this->dny as $index => $den) {
                if ($den ->lzeEditovat) {
                    $this->zpracujDataProDen($index);
                }
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
        }
    }
}
