<?php
/**
 * User: pavel.rudol
 * 29.4.2016 11:03
 */

namespace RonPortal\Kapacity;


use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\System\Report;
use RonPortal\System\ReportOutputObject;

class ZastupyController extends JsonController
{
    private $database;

    private $pravaVlozit;
    private $pravaSchvalit;
    private $pravaPriraditPP;
    private $pravaZrusit;

    /**
     * ZastupyController constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct($database)
    {
        $this->database = $database;

        TextManager::getInstance()->loadStrings('zastupy.ini');

        HtmlHeaders::getInstance()->addScript('script/zastupy.js');
        $this ->addTranslationsScript('zastup');

        Prava::nastavObdobiProPrava(time(),time());

        $this->pravaVlozit = Prava::vratPrava('zastupy_vlozeni');
        $this->pravaSchvalit = Prava::vratPrava('zastupy_schvaleni');
        $this->pravaPriraditPP = Prava::vratPrava('zastupy_prideleni');
        $this->pravaZrusit = Prava::vratPrava('zastupy_zruseni');
        //$this->pravaZobrazit = Prava::vratPrava('zastupy_prohl');
    }

    /**
     * Seznam zadosti o zastup.
     * @return ZastupyBrowser
     * @throws \Exception
     */
    private function seznamZadosti()
    {
        $b = new ZastupyBrowser($this->database, isset($_GET['schvalovani'])  &&  $_GET['schvalovani']);
        $b->load();

        return $b;
    }

    /**
     * Presmerovani na detail zastupu.
     * @param int $id
     * @return Redirect
     */
    private function redirectToView($id)
    {
        return new Redirect(null, '?menuid=zastup&action=detail&record='.$id);
    }


    /**
     * Presmerovani na seznam zadosti.
     * @return Redirect
     */
    private function redirectToBrowser()
    {
        return new Redirect(null, '?menuid=zastup');
    }

    /**
     * Zobrazeni a zpracovani formulare pro vlozeni/prirazeni zastupu.
     * @param string $formAction
     * @return Redirect|ZastupyForm
     * @throws NeniPravoException
     * @throws \Exception
     */
    private function zastupyFormular($formAction)
    {
        if (!($this->pravaVlozit->jsouNejakePrava() || $this->pravaPriraditPP->jsouNejakePrava())) {
            throw new NeniPravoException();
        }

        $prirazeniPP = false;
        $zastup = null;

        if (($formAction == 'edit') || ($formAction == 'save')) {
            $idZaznamu = isset($_POST['record']) ? $_POST['record'] : $_GET['record'];
            $zastup = Zastup::nactiZastup($this->database, $idZaznamu);

            if (is_null($zastup)) {
                throw new \Exception('Zastup nebyl nalezen.');
            }

            if (isset($_GET['action']) && ($_GET['action'] == 'priradit')) {
                $prirazeniPP = true;
            }

            if ($prirazeniPP) {
                if (!($zastup->lzePriraditOsobu)) {
                    throw new NeniPravoException();
                }

                //pokud uzivatel meni prirazenou osobu, je nutne puvodnimu zamestnanci zrusit plan
                if ($zastup->idPP) {
                    try {
                        $this->database->beginTransaction();
                        if (ZastupyUtils::zrusPlanovaniKapacit($this->database, $zastup->idPP, $zastup->datum, $this)) {
                            $zastupRec = new ZastupRecord($this->database, $zastup->id, $this);
                            $zastupRec->zrusZamestnance();
                            $this->database->commitTransaction();
                        }
                    }
                    catch (\Exception $e) {
                        $this->database->rollbackTransaction();
                        MessageManager::addErrorMessage($e);
                    }
                }
            }
            elseif (!$zastup->lzeEditovat) {
                throw new NeniPravoException();
            }

        }

        $form = new ZastupyForm($this->database,
            $formAction, $prirazeniPP, $zastup,
            isset($_GET['rozdeleni']) ? $_GET['rozdeleni'] : null);

        //pokud ma uzivatel pravo na schvaleni pri vlozeni, zaznam se rovnou schvali
        if ($form->handleAction())
        {
            if ($form->idZaznamu) {
                return $this->redirectToView($form->idZaznamu);
            } else {
                return $this->redirectToBrowser();
            }
        }

        return $form;
    }

    /**
     * Zobrazeni detailu zastupu s moznosti schvaleni.
     * @param int $id
     * @return BaseViewObject
     * @throws \Exception
     */
    private function zobrazDetailZaznamu($id)
    {
        $zastup = Zastup::nactiZastup($this->database,$id);

        if (!$zastup->lzeZobrazitZaznam()) {
            throw new NeniPravoException();
        }

        Model::addData('zastup',$zastup);

        $reportList = Report::loadFormReports($this ->database, 'Zastup', 'Zastup');
        Model::addData('reportList', $reportList);

        return new BaseViewObject('ronportal/kapacity/view_zastup.php');
    }

    /**
     * Provede schvaleni, pri uspechu presmeruje na view, jinak zobrazi view.
     * @param int $id
     * @param int $hodnota 0 = nerozhodnuto, 1 = schvaleno, 2 = zamitnuto, 3 = zrusena zadost (jiz je prirazen zamestnanec)
     * @return Redirect|ViewObject
     * @throws \Exception
     */
    private function schvaleni($id, $hodnota)
    {
        $rec = new ZastupRecord($this ->database, $id, $this);
        $result = $rec ->schval($hodnota == 3 ? $this->pravaZrusit : $this->pravaSchvalit, $hodnota);

        if ($result) {
            return $this ->redirectToView($id);
        } else {
            return $this ->zobrazDetailZaznamu($id);
        }
    }

    /**
     * @param int $id   Id zadosti
     * @return Redirect
     * @throws DBException
     */
    private function smazZastup($id)
    {
        $zastup = Zastup::nactiZastup($this->database, $id);

        try {
            if (!$zastup->lzeEditovat) {
                throw new NeniPravoException();
            }

            $rec = new ZastupRecord($this->database, $id, $this);
            $rec->deleteRecord();
            MessageManager::addInfoMessage(TextManager::_('Zastup_zruseni_ok'));
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e->getMessage());
        }

        return $this->redirectToBrowser();
    }

    /**
     * Provede smazani planovani kapacit pro stejny den a prac. pomer jako je zastup.
     * @param int $idZaznamu
     * @return Redirect|ViewObject
     * @throws \Exception
     */
    private function zrusPlanovaniKapacit($idZaznamu)
    {
        $zastup = Zastup::nactiZastup($this->database, $idZaznamu);

        if (!$zastup->lzeZrusitPlan) {
            throw new NeniPravoException();
        }

        $result = false;

        try {
            $this->database->beginTransaction();
            ZastupyUtils::zrusPlanovaniKapacit($this->database, $zastup->idPP, $zastup->datum);
            $result = $this->schvaleni($idZaznamu, 3);
        }
        catch (\Exception $e) {
            $this->database->rollbackTransaction();
            MessageManager::addErrorMessage($e);
        }

        if ($result instanceof Redirect) {
            $this->database->commitTransaction();
            return $result;
        }
        else {
            $this->database->rollbackTransaction();
            return $this->zobrazDetailZaznamu($idZaznamu);
        }
    }

    /**
     * Zobrazi zvolenou sestavu zacinajici v kodu 'Zastup'.
     * @param int $idZaznamu
     * @return ReportOutputObject
     * @throws \Exception
     */
    private function report($idZaznamu)
    {
        $zastup = Zastup::nactiZastup($this->database, $idZaznamu);

        Prava::nastavObdobiProPrava($zastup->datum, $zastup->datum);

        if (!$zastup->lzeZobrazit) {
            throw new NeniPravoException();
        }

        $result = new ReportOutputObject();
        $result->loadReportFromGet($this ->database, 'Zastup', false, 'Zastup');
        $result ->nastavObdobiOdDo($zastup->datum, $zastup->datum);
        $result->integerParams = ['IdZastupu' => $zastup->id];

        return $result;
    }

    /**
     * Vrati JsonObject se smenou a rozdelenim z planovani kapacit, pokud existuje.
     * @param JsonOutputObject $output
     * @param int $idPP
     * @param int $datum
     * @throws NeniPravoException|DBException
     */
    private function kontrolaPlanovaniKapacit($output, $idPP, $datum)
    {
        if (!$this->pravaPriraditPP->lzeZobrazitPP($idPP, $datum, $datum)) {
            throw new NeniPravoException();
        }

        $params = $this->database->createParameters();

        $sql = "SELECT Smena, Rozdeleni
                FROM PlanovaniKapacit
                JOIN PracovniPomer ON (PracovniPomer.RC = PlanovaniKapacit.RC AND PracovniPomer.OsobniCislo = PlanovaniKapacit.OsobniCislo AND PracovniPomer.DatumNastupu = PlanovaniKapacit.DatumNastupu)
                WHERE PracovniPomer.ID = " . $params->addParameter($idPP) .
            " AND PlanovaniKapacit.Datum = " . $params->addParameter($this->database->dateToParam($datum));

        MessageManager::addDebugLog($sql);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        $output ->addData('existujeplan', (bool)$row);

        if ($row) {
            $output ->addData('smena', StringUtils::jsonText($row[0]));
            $output ->addData('rozdeleni', StringUtils::jsonText($row[1]));
        }
    }


    function provedHromadneSchvaleni()
    {
        ZastupRecord::hromadneSchvaleniZastupu($this ->database, $this);
        return new Redirect(null, '?menuid=zastup');
    }


    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'kontrolaplanu' :
                $this->kontrolaPlanovaniKapacit($output, (int)$_GET['idpp'], (int)$_GET['datum']);
                break;
        }
    }


    /**
     * @param string $action
     * @return ViewObject|OutputObject|Redirect
     * @throws \Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_KAPACITY)) {
            throw new \Exception('Tato funkce neni povolena.');
        }

        if (isset($_POST['formaction'])) {
            $formaction = $_POST['formaction'];
        }
        else if (isset($_GET['formaction'])) {
            $formaction = $_GET['formaction'];
        }
        else {
            $formaction = '';
        }

        if ($formaction) {
            // formular (editace, insert)
            return $this ->zastupyFormular($formaction);
        }

        if (isset($_POST['hromadneSchvaleni'])  &&  $_POST['hromadneSchvaleni']) {
            return $this ->provedHromadneSchvaleni();
        }

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'report' :
                    return $this->report((int)$_GET['id']);

                case 'detail' :
                    return $this->zobrazDetailZaznamu((int)$_GET['record']);

                case 'schval' :
                    return $this->schvaleni((int)$_GET['record'], 1);

                case 'zamitni' :
                    return $this->schvaleni((int)$_GET['record'], 2);

                case 'zrusplan' :
                    return $this->zrusPlanovaniKapacit((int)$_GET['record']);

                case 'zrusitzastup' :
                    return $this->smazZastup((int)$_GET['record']);

                case 'kontrolaplanu' :
                    return parent::handleAction($action);   // json

                case 'translations' :   // texty pro javascripty
                    return $this->translations();
            }
        }

        return $this->seznamZadosti();
    }
}
