<?php
/**
 * stanislav.elbl, 20.02.2021
 */

namespace RonPortal;

use Ron\DB\DBException;
use Ron\DB\DBRecord;
use RonPortal\XML\RonPortalXMLForm;


class RonPortalDBRecord extends DBRecord
{
    var $logDescription;
    var $logText;

    /** @var string */
    public $beforePostEvent;
    /** @var string */
    public $afterPostEvent;
    /** @var array */
    public $eventsParameters = null;

    /**
     * @param \Ron\DB\Database $database
     * @param string $tableName
     * @param string|object $logDescription
     */
    public function __construct($database, $tableName, $logDescription = 'Intraweb')
    {
        parent::__construct($database, $tableName);
        $this->setLogDescription($logDescription);
    }


    /**
     * Prida logovaci pole Insert_Timestamp a Insert_UserName
     * @param string $timestampFieldName
     * @param string $userFieldName
     * @return $this
     */
    public function addInsertLogFields($timestampFieldName = 'Insert_Timestamp', $userFieldName = 'Insert_UserName')
    {
        if ($timestampFieldName) {
            $f = $this->addTimestampField($timestampFieldName, true, time());
            $f ->setUpdateLog(true);
        }

        if ($userFieldName) {
            $f = $this->addStringField($userFieldName, true, 25, RonPortalUtils::getLogUserName());
            $f ->setUpdateLog(true);
        }

        return $this;
    }


    /**
     * Prida logovaci pole Update_Timestamp a Update_UserName
     * @param string $timestampFieldName
     * @param string $userFieldName
     * @return $this
     */
    public function addUpdateLogFields($timestampFieldName = 'Update_Timestamp', $userFieldName = 'Update_UserName')
    {
        if ($timestampFieldName) {
            $f = $this->addTimestampField($timestampFieldName, false, time());
            $f
                ->setUpdateLog(true)
                ->setNoInsert(true); // pri insert se nevklada
        }

        if ($userFieldName) {
            $f = $this->addStringField($userFieldName, false, 25, RonPortalUtils::getLogUserName());
            $f
                ->setUpdateLog(true)
                ->setNoInsert(true);
        }

        return $this;
    }


    /**
     * @param string $beforePostEvent
     * @param string $afterPostEvent
     * @param array $eventParameters
     * @return RonPortalDBRecord
     */
    public function setXmlEvents($beforePostEvent, $afterPostEvent, $eventParameters = null)
    {
        $this ->beforePostEvent = $beforePostEvent;
        $this ->afterPostEvent = $afterPostEvent;

        $this ->eventsParameters = $eventParameters;

        return $this;
    }

    /**
     * @param string|object $description
     * @return RonPortalDBRecord
     */
    public function setLogDescription($description)
    {
        if (is_object($description)) {
            $this ->logDescription = substr(strrchr(get_class($description), "\\"), 1);
        } else {
            $this ->logDescription = $description;
        }

        return $this;
    }


    /**
     * @param string   $eventType
     * @param string   $logText
     */
    public function log($eventType, $logText)
    {
        if ($this->logText) {
            $logText = $this->logText . "\n" . $logText;
        }
        Log::addRecordToDBLog($this ->database, $this->tableName, $eventType, $this->logDescription, $logText);
    }


    /**
     * @param string $eventName
     * @param string $context
     * @param array $variables Asociativni pole ve formatu ['nazevpromenne' => hodnota]. Automaticky se pridava promenna record
     * @param bool $exceptionOnError Pokud je true, tak v pripade chyby ve formulari vyhodi vyjimku. Jinak jen vraci false.
     * @return bool
     * @throws \Exception
     */
    public function executeXmlEvents($eventName, $context = '', $variables = null, $exceptionOnError = true)
    {
        if (!$variables) {
            $variables = [];
        }
        $variables['record'] = $this;

        return RonPortalXMLForm::xmlEvents($this ->database, $eventName, $variables, $context, false, $exceptionOnError);
    }


    public function updateRecord($filtr = '', $filtrParams = null)
    {
        $changed = $this ->isChanged();
        if ($changed  &&  $this->beforePostEvent) {
            $this->executeXmlEvents($this->beforePostEvent, 'update', $this ->eventsParameters);
        }

        $result = parent::updateRecord($filtr, $filtrParams);

        if ($changed  &&  $this->afterPostEvent) {
            $this->executeXmlEvents($this->afterPostEvent, 'update', $this ->eventsParameters);
        }

        return $result;
    }


    /**
     * @return void
     * @throws DBException
     * @throws \Exception
     */
    public function insertRecord()
    {
        if ($this ->beforePostEvent) {
            $this ->executeXmlEvents($this ->beforePostEvent, 'insert', $this ->eventsParameters);
        }

        parent::insertRecord();

        if ($this->afterPostEvent) {
            $this->executeXmlEvents($this->afterPostEvent, 'insert', $this ->eventsParameters);
        }
    }


    /**
     * @throws DBException
     */
    public function doDelete($filtr, $filtrParams = null)
    {
        if ($this ->beforePostEvent) {
            $this ->executeXmlEvents($this ->beforePostEvent, 'delete', $this ->eventsParameters);
        }

        parent::doDelete($filtr, $filtrParams);

        if ($this->afterPostEvent) {
            $this->executeXmlEvents($this->afterPostEvent, 'delete', $this ->eventsParameters);
        }
    }
}
